/* This library function writes the resulting data for the program which
   uses the finite difference method to calculate the Poisson equation
   using adaptive mesh refinement.

                     Updated 4/8/08

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2005  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "amr.h"

#define SMALL   1.0e-20

extern int sof, nx, ny, nz, num_subgrid;
extern int flag_2D, flag_3D;
extern double dx, dy, dz;
extern double dt, time_current;
extern int nstep, cstep;

int meshBCset( int *, char *);

int writer_2D (BOUND bc, MATL *matl, char *name, INC_NODE *node)
{
	int i, j, k, dum, node_num, bc_type, check, name_length, numnp;
	char *ccheck;
	int dum2,dum3,dum4;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, prs, ene, enthal, mach;
	double gam, R;
	char text[10], out[30];
	FILE *o3;

/* Every output file is named after the input file with
   a ".ofd" extension */

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	memset(out,0,30*sizeof(char));

	ccheck = strncpy(out, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(out+name_length, ".ofd", 4);
	if(!ccheck) printf( " Problems with strncpy \n");

	o3 = fopen( out,"w" );

	gam = matl[0].gam;
	R = matl[0].R;

	numnp = nx*ny;

	fprintf( o3, "total nodes, nodes in x, nodes in y for the shock bubble\n");
	fprintf( o3, "%4d    %4d %4d\n", numnp, nx, ny);
	fprintf( o3, "Time step, number of steps, current step, current time\n");
	fprintf( o3, "%16.8e %7d %7d %16.8e\n", dt, nstep, cstep, time_current);
	fprintf( o3, "spatial discretization dx, dy\n");
	fprintf( o3, "%16.8e %16.8e\n", dx, dy);
	fprintf( o3, "Ideal Gas Constant, Ratio of specific heats\n");
	fprintf( o3, "%16.8e %16.8e\n", R, gam);

	fprintf( o3, "   i    j    bc         density      temperature  pressure     velocity x");
	fprintf( o3, "   velocity y   enthalpy     energy       mach number\n");

	dum= 0;
	for ( node_num = 0; node_num < numnp; node_num++)
	{
		dns    = 0.0;
		temp   = 0.0;
		vx     = node[node_num].vx;
		vy     = node[node_num].vy;
		prs    = node[node_num].prs;
		ene    = 0.0;
		enthal = 0.0;
		mach   = node[node_num].vmag;
		bc_type = node[node_num].bc_type;

		i = node[node_num].i;
		j = node[node_num].j;

		memset(text, 0, 10*sizeof(char));

		check = meshBCset( &bc_type, text);
		if(!check) printf( " Problems with meshBCset\n");

		fprintf( o3, "%4d %4d  %10s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
			i, j, text, dns, temp, prs, vx, vy, enthal, ene, mach);

		//fprintf(o3, "%4d %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e\n",
		//  dum, temp, vx, vy, prs, ene, dns); 
		//fprintf(o3, "%4d %4d %4d %14.6e %14.6e %14.6e\n",
		//    i, j, dum, fdum1, temp, dns); 
		/*fprintf(o3, "%4d %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n",
			dum, fdum1, temp, dns, prs, ene, enthal);*/
		/*fprintf(o3, "%4d %4d %4d %4d %4d %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n",
			dum, i, j, node_num, bc_type, fdum1, temp, dns, prs, ene, enthal); */
	}

	fprintf( o3, "%4d ",-10);

	return 1;
}


int writer_SLFFEA_2D_AMR (BOUND bc, MATL *matl, INC_NODE *node, SUBGRID *subgrid)
{
	int i, i2, j, j2, k, dum, node_num;
	int dof, nmat, numel, numed, numnp, plane_stress_flag;
	int dum1, dum2,dum3,dum4;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, vz, prs, ene, enthal;
	double gam, R;
	double side_length, side_width, el_length, el_width;
	int length_el_num, width_el_num, el_type;
	FILE *o1;

	o1 = fopen( "rect","w" );

	gam = matl[0].gam;
	R = matl[0].R;

	nmat = 1;
	plane_stress_flag = 1;
	numel = (nx - 1)*(ny - 1);
	numnp = nx*ny;
	el_length = dx;
	el_width = dy;

    // ofstream fout("ouput");

    // San: Print initial conditions for internal nodes(of nodeList) from 1/3(ymax) < y < ymax


	fprintf( o1, "   numel numnp nmat nmode plane_stress_flag");
	fprintf( o1, "  This is the rectangular waveguide problem \n ");
	fprintf( o1, "    %4d %4d %4d %4d %4d \n ",
	    num_subgrid+1,numnp,nmat,0,plane_stress_flag);

	fprintf( o1, "matl no., E modulus, Poisson Ratio, density \n");
	for( i = 0; i < nmat; ++i )
	{
	    fprintf( o1, "%3d ",0);
	    fprintf( o1, " %9.4f %9.4f     %9.4f \n ",10000000.0, 0.3, 2.77e3);
	}

	fprintf( o1, "el no., connectivity, matl no \n");

	i = nx - 1; j = 0;      dum2 = i + nx*j;
	i = nx - 1; j = ny - 1; dum3 = i + nx*j;
	i = 0;      j = ny - 1; dum4 = i + nx*j;

	fprintf( o1, "%4d ",0);
	fprintf( o1, " %4d %4d %4d %4d", 0, dum2, dum3, dum4);
	fprintf( o1, "%4d ",0);
	fprintf( o1, "\n");

	for( i2 = 0; i2 < num_subgrid; ++i2 )
	{

		i = subgrid[i2].istart;
		j = subgrid[i2].jstart;
		dum1 = i + nx*j;

		i = subgrid[i2].iend;
		j = subgrid[i2].jstart;
		dum2 = i + nx*j;

		i = subgrid[i2].iend;
		j = subgrid[i2].jend;
		dum3 = i + nx*j;

		i = subgrid[i2].istart;
		j = subgrid[i2].jend;
		dum4 = i + nx*j;

		fprintf( o1, "%4d ",i2+1);
		fprintf( o1, " %4d %4d %4d %4d", dum1, dum2, dum3, dum4);
		fprintf( o1, "%4d ",0);
		fprintf( o1, "\n");
	}

/*
	for( i = 0; i < ny - 1; ++i )
	{
		for( j = 0; j < nx - 1; ++j )
		{
			fprintf( o1, "%4d ",dum);
			fprintf( o1, " %4d %4d %4d %4d",i*nx + j,
				i*nx+j+1, (i+1)*nx+j+1,
				(i+1)*nx+j);
			fprintf( o1, "%4d ",0);
			fprintf( o1, "\n");
			++dum;
		}
	}
*/

	dum = 0;
	fprintf( o1, "node no., coordinates \n");
	for( i = 0; i < ny; ++i )
	{
		for( j = 0; j < nx; ++j )
		{
			fdum1 = el_length*((double)j);
			fdum2 = el_width*((double)i);
			fprintf( o1, "%d ",dum);
			fprintf( o1, "%9.4f %9.4f ",fdum1,fdum2);
			fprintf( o1, "\n");
			++dum;
		}
	}

	dum= 0;
	fprintf( o1, "prescribed displacement x: node  disp value\n");
	/*for( i2 = 0; i2 < numnp; ++i2 )
	{
		i = node[i2].i;
		j = node[i2].j;
		dum = i + nx*j;
		fprintf( o1, "%4d %16.8e\n", dum, node[i2].prs);
	}*/
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "prescribed displacement y: node  disp value\n");
	fprintf( o1, "%4d\n ",-10);

	dum = 0;
	fprintf( o1, "node with point load and load vector in x,y\n");
	for( i2 = 0; i2 < numnp; ++i2 )
	{
		if(node[i2].refine_flag == 1)
		{
			i = node[i2].i;
			j = node[i2].j;
			dum = i + nx*j;
			fprintf( o1, "%4d %9.4f %9.4f\n", dum, 0.0, 0.0);
		}
	}
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "node no. with mach, temp, dns\n");
	fprintf( o1, "%4d ",-10);

	return 1;
}


int writer_3D (BOUND bc, MATL *matl, char *name, INC_NODE *node)
{
	int i, j, k, dum, node_num, bc_type, check, name_length, numnp;
	char *ccheck;
	int dum2,dum3,dum4;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, vz, prs, ene, enthal, mach;
	double gam, R;
	char text[6], out[30];
	FILE *o3;

/* Every output file is named after the input file with
   a ".ofd" extension */

	name_length = strlen(name);
	if( name_length > 25) name_length = 25;

	memset(out,0,30*sizeof(char));

	ccheck = strncpy(out, name, name_length);
	if(!ccheck) printf( " Problems with strncpy \n");

	ccheck = strncpy(out+name_length, ".ofd", 4);
	if(!ccheck) printf( " Problems with strncpy \n");

	o3 = fopen( out,"w" );

	gam = matl[0].gam;
	R = matl[0].R;

	numnp = nx*ny*nz;

	fprintf( o3, "total nodes, nodes in x, nodes in y, nodes in z for the shock bubble\n");
	fprintf( o3, "%4d    %4d %4d %4d\n", numnp, nx, ny, nz);
	fprintf( o3, "Time step, number of steps, current step, current time\n");
	fprintf( o3, "%16.8e %7d %7d %16.8e\n", dt, nstep, cstep, time_current);
	fprintf( o3, "spatial discretization dx, dy, dz\n");
	fprintf( o3, "%16.8e %16.8e %16.8e\n", dx, dy, dz);
	fprintf( o3, "Ideal Gas Constant, Ratio of specific heats\n");
	fprintf( o3, "%16.8e %16.8e\n", R, gam);

	fprintf( o3, "   i    j    k  bc     density      temperature  pressure     velocity x");
	fprintf( o3, "   velocity y   velocity z   enthalpy     energy       mach number\n");

	dum= 0;
	for ( node_num = 0; node_num < numnp; node_num++)
	{
		dns    = 0.0;
		temp   = 0.0;
		vx     = node[node_num].vx;
		vy     = node[node_num].vy;
		vz     = node[node_num].vz;
		prs    = node[node_num].prs;
		ene    = 0.0;
		enthal = 0.0;
		mach   = node[node_num].vmag;
		bc_type = node[node_num].bc_type;

		i = node[node_num].i;
		j = node[node_num].j;
		k = node[node_num].k;

		memset(text, 0, 6*sizeof(char));

		check = meshBCset( &bc_type, text);
		if(!check) printf( " Problems with meshBCset\n");

		fprintf( o3, "%4d %4d %4d  %4s %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e %12.4e\n",
			i, j, k, text, dns, temp, prs, vx, vy, vz, enthal, ene, mach);

		//fprintf(o3, "%4d %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e %12.6e\n",
		//  dum, temp, vx, vy, vz, prs, ene, dns); 
		//fprintf(o3, "%4d %4d %4d %14.6e %14.6e %14.6e\n",
		//    i, j, dum, fdum1, temp, dns); 
		/*fprintf(o3, "%4d %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n",
			dum, fdum1, temp, dns, prs, ene, enthal);*/
		/*fprintf(o3, "%4d %4d %4d %4d %4d %4d %14.6e %14.6e %14.6e %14.6e %14.6e %14.6e\n",
			dum, i, j, k, node_num, bc_type, fdum1, temp, dns, prs, ene, enthal); */
	}

	fprintf( o3, "%4d ",-10);

	return 1;
}

int writer_SLFFEA_3D (BOUND bc, MATL *matl, INC_NODE *node, SUBGRID *subgrid)
{
	int i, j, k, dum, node_num, bc_type;
	int dof, nmat, numel, numed, numnp, plane_stress_flag;
	int dum2,dum3,dum4;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, vz, prs, ene, enthal;
	double gam, R;
	double dxx;
	double side_length, side_width, side_height, el_length, el_width, el_height;
	int length_el_num, width_el_num, height_el_num, el_type;
	FILE *o1;

	o1 = fopen( "rect","w" );

	gam = matl[0].gam;
	R = matl[0].R;

	nmat = 1;
	plane_stress_flag = 1;
	numel = (nx - 3)*(ny - 3)*(nz - 3);
	numnp = (nx-2)*(ny-2)*(nz-2);
	el_length = dx;
	el_width = dy;
	el_height = dz;

	fprintf( o1, "   numel numnp nmat nmode");
	fprintf( o1, "  This is the CFD problem \n ");
	fprintf( o1, "    %4d %4d %4d %4d\n ",
	    numel,numnp,nmat,0);

	fprintf( o1, "matl no., E modulus, Poisson Ratio, density \n");
	for( i = 0; i < nmat; ++i )
	{
	    fprintf( o1, "%3d ",0);
	    fprintf( o1, " %9.4f %9.4f     %9.4f \n ",10000000.0, 0.3, 2.77e3);
	}

	fprintf( o1, "el no., connectivity, matl no \n");

	dum = 0;
	dum2 = 0;
	dum3 = (nx-2)*(ny-2) + (nx-2) + 1;
	for( k = 1; k < nz - 2; ++k )
	{
	    for( i = 1; i < ny - 2; ++i )
	    {
		for( j = 1; j < nx - 2; ++j )
		{
			fprintf( o1, "%4d ",dum);
			fprintf( o1, " %4d %4d %4d %4d %4d %4d %4d %4d",
				k*(nx-2)*(ny-2) + i*(nx-2) + j - dum3,
				k*(nx-2)*(ny-2) + i*(nx-2)+j+1 - dum3,
				k*(nx-2)*(ny-2) + (i+1)*(nx-2)+j+1 - dum3,
				k*(nx-2)*(ny-2) + (i+1)*(nx-2)+j - dum3,
				(k+1)*(nx-2)*(ny-2) + i*(nx-2) + j - dum3,
				(k+1)*(nx-2)*(ny-2) + i*(nx-2)+j+1 - dum3,
				(k+1)*(nx-2)*(ny-2) + (i+1)*(nx-2)+j+1 - dum3,
				(k+1)*(nx-2)*(ny-2) + (i+1)*(nx-2)+j- dum3);
			fprintf( o1, "%4d ",0);
			fprintf( o1, "\n");
			++dum;
		}
	    }
	}

	dum = 0;
	fprintf( o1, "node no., coordinates \n");
	for( k = 1; k < nz-1; ++k )
	{
	    for( j = 1; j < ny-1; ++j )
	    {
		for( i = 1; i < nx-1; ++i )
		{
			fdum1 = el_length*((double)i);
			fdum2 = el_width*((double)j);
			fdum3 = el_height*((double)k);
			fprintf( o1, "%d ",dum);
			fprintf( o1, "%16.8e   %16.8e   %16.8e ",fdum1,fdum2,fdum3);
			fprintf( o1, "\n");
			++dum;
		}
	    }
	}

	dum= 0;
	fprintf( o1, "prescribed displacement x: node  disp value\n");
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "prescribed displacement y: node  disp value\n");
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "prescribed displacement z: node  disp value\n");
	fprintf( o1, "%4d\n ",-10);

	dum = 0;
	fprintf( o1, "node with point load and load vector in x,y\n");
	fprintf( o1, "%4d\n ",-10);
	fprintf( o1, "node no. with mach, temp, dns, prs, ene, enthal\n");
	fprintf( o1, "%4d ",-10);

	return 1;
}


