/* The functions in this file do the regridding from one coarse grid to another
   coarse grid.

                        Updated 4/9/07

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2007  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 */

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "amr.h"

extern double dx, dy, dt;

int setgrid( SUBGRID *subgrid2 )
{
/* This code does 2 things:

   1) Sets the value of dx, dy, and dt for the subgrid2.

   2) Sets the dimensions of the subgrid2. 

                        Updated 4/9/07
*/

	subgrid2[0].nx = subgrid2[0].iend - subgrid2[0].istart + 1;
	subgrid2[0].ny = subgrid2[0].jend - subgrid2[0].jstart + 1;

	subgrid2[0].dx = dx;
	subgrid2[0].dy = dy;
	subgrid2[0].dt = dt;

	return 1;
}


int GridSet1( double *vec1, double *vec2, SUBGRID subgrid1, SUBGRID subgrid2 )
{
/*

                        Updated 4/9/07
*/

	int i, i2, j, j2, node_num, dum, nx2, ny2, ny1;
	int ce, ce2, xm1, xm2, xp1, ym1, ym2, yp1, yp2, xp1yp1;

	nx2 = subgrid2.nx;
	ny2 = subgrid2.ny;

	ny1 = subgrid1.ny;

	j2 = 0;
	for( j = subgrid2.jstart; j < subgrid2.jend + 1; ++j)
	{
	    i2 = 0;
	    for( i = subgrid2.istart; i < subgrid2.iend + 1; ++i)
	    {

/* For the fine nodes which directly fall on the coarse nodes (i,j) */

		ce = ny1*i + j;
		ce2 = ny2*i2 + j2;
		vec2[ce2] = vec1[ce];
		/*printf("%7d %7d %7d %7d %7d %7d\n", i, j, i2, j2, ce, ce2);*/
		++i2;
	    }
	    ++j2;
	}

	return 1;
}


int GridSet2( double *vec1, double *vec2, SUBGRID subgrid1, SUBGRID subgrid2 )
{
/*
                        Updated 4/9/07
*/
	int i, i2, j, j2, node_num, dum, nx2, ny2, ny1;
	int ce, ce2, xm1, xm2, xp1, ym1, ym2, yp1, yp2, xp1yp1;

	nx2 = subgrid2.nx;
	ny2 = subgrid2.ny;

	ny1 = subgrid1.ny;

	j2 = 0;
	for( j = subgrid2.jstart; j < subgrid2.jend + 1; ++j)
	{
	    i2 = 0;
	    for( i = subgrid2.istart; i < subgrid2.iend + 1; ++i)
	    {

/* For the fine nodes which directly fall on the coarse nodes (i,j) */

		ce = ny1*i + j;
		ce2 = ny2*i2 + j2;
		vec1[ce] = vec2[ce2];
		/*printf("%7d %7d %7d %7d %7d %7d\n", i, j, i2, j2, ce, ce2);*/ 
		++i2;
	    }
	    ++j2;
	}

	return 1;
}
