/* This subroutine solves Poission's equation for the pressure using
   the SOR algorithm.  f represents the pressure.

                     Updated 4/29/07

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "amr.h"

#define w               1.5
#define BIG             10.0e8
#define err             1.0e-10

extern int symmetric_flag;

int sor( double *Y, double *prs, SUBGRID subgrid)
{
	FILE *out;
	int i, j, kk, m, dum;
	int ce, xm1, xm2, xm3, xp1, ym1, ym2, yp1;
	double dx, dy, dt;
	int nx, ny;
	double bets, fdum, pi, df;
	out = fopen( "o2", "w" );
	
	nx = subgrid.nx;
	ny = subgrid.ny;
	dx = subgrid.dx;
	dy = subgrid.dy;
	dt = subgrid.dt;

	fdum=0.0;	
	bets=dx*dx/(dy*dy);	
	pi=acos(-1.0);	
	df=0.;

	for( kk = 0; kk < 10000; ++kk )
	{
	   for( j = 1; j < ny-1; ++j )
	   {
		for( i = 1; i < nx-1; ++i )
		{
		   ce  = ny*i + j;
		   xm1 = ny*(i-1) + j;
		   xm2 = ny*(i-2) + j;
		   xp1 = ny*(i+1) + j;
		   ym1 = ny*i + (j-1);
		   ym2 = ny*i + (j-2);
		   yp1 = ny*i + (j+1);

		   df = 1./(2.+2*bets)*(prs[xp1]+bets*prs[yp1]+prs[xm1] +
			bets*prs[ym1]-2.*(1.+bets)*prs[ce] + dx*dx*Y[ce]);

			prs[ce]=prs[ce]+w*df;

/*		printf(  "eee %i %i  %9.5f %9.5f \n",i,j,prs[ce],df );*/
			if( fabs(df) > fdum ) 
			{
				fdum=fabs(df);
			}
		 }
	   }

/*  Set the symmetrical pressure at the right. */
	
	   if(symmetric_flag)
	   {
/* With the symmetric flag, I am assuming a Derivative(or Neumann) boundary
   condition so that the boundary on the right side is such that the 
   pressure gradient is zero.  The methods recommended to set 
   the pressure, prs, at the boundary is given by:

     Hoffman, Joe D., Numerical Methods for Engineers and Scientist, 4th Ed.,
         McGraw-Hill Inc., New York, 1992. 

   on pages 433-440.  There are two possible methods including:

   1) Interior Point FDE at the Boundary  (given by equation 9.85)
   2) Cubic Extrapolation                 (given by equation 9.89)
*/
		i = nx - 1;
		for( j = 1; j < ny-1; ++j )
		{
		   ce  = ny*i + j;
		   xm1 = ny*(i-1) + j;
		   xm2 = ny*(i-2) + j;
		   xm3 = ny*(i-3) + j;
		   ym1 = ny*i + j-1;
		   yp1 = ny*i + j+1;
		   prs[ce] = (prs[yp1] + 2.0*prs[xm1] + prs[ym1])/4.0;
		   prs[ce] = (18.0*prs[xm1] - 9.0*prs[xm2] + 2.0*prs[xm3])/11.0;
		}
		i = nx - 1;
		j = 0;
		ce  = ny*i + j;
		xm1 = ny*(i-1) + j;
		xm2 = ny*(i-2) + j;
		xm3 = ny*(i-3) + j;
		prs[ce] = prs[xm1];
		prs[ce] = (18.0*prs[xm1] - 9.0*prs[xm2] + 2.0*prs[xm3])/11.0;
		i = nx - 1;
		j = ny-1;
		ce  = ny*i + j;
		xm1 = ny*(i-1) + j;
		xm2 = ny*(i-2) + j;
		xm3 = ny*(i-3) + j;
		prs[ce] = prs[xm1];
		prs[ce] = (18.0*prs[xm1] - 9.0*prs[xm2] + 2.0*prs[xm3])/11.0;
	   }

	   if( kk > 9999 ) printf(  "Maximum iterations reached %4d %4d %4d  %16.8e %16.8e \n", i, j, kk, prs[ce], df );
	   if( fdum < err || fdum > 10000 ) 
	   {
		m = kk;
		printf(  "zzz  %4d %4d %4d  %16.8e %16.8e \n", i, j, kk, prs[ce], df );
		kk = 1000000;
	   }
	   fdum=0.0;

	}

	return 1;
}	

