/* In this version, the MAC version is totally used, where u and v are updated
   based on F and G.  See lines 779 onward.

   results: stable*

   I had originally thought it was unstable but then I modified the boundary
   conditions for the right side exit for F and G.


   This program uses the finite difference method to calculate an incompressible
   laminar CFD flow field.  It reads in the data and calculates the various
   quantities for each point in the rectangular CFD mesh.

   The algorithm for the code is based on the MAC(Marker and Cell) SCHEME.
   The SOR method is used to calculate the pressure corrector.
   The main references are:

   More is given on this method on page 337-338 of:

Fletcher, Clive A. J., Computational Techniques for Fluid Dynamics: Vol II, second
        edition, John Wiley & Sons Inc., New York, 1991.

I have extrapolated the 2-D version to 3-D.

                     Updated 3/29/07

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "../compressible/flwconst.h"
#include "../compressible/flwstruct.h"
#include "../AMR/amr.h"

#define ANGLE               1

int writer_2D( BOUND , MATL *, char *, NODE *);

int pathlines(FILE *, double *, double *, double *, double *);

int streaklines(int , double *, double *, double *, double *, double *, double *);

int applyBC_free(double *, double *, SUBGRID );

int refineGridSet( double *, double *, SUBGRID , SUBGRID );

int coarseGridSet_AMR( double *, double *, double *, double *, double *, double *,
        SUBGRID , SUBGRID );

int PoisConjGrad_AMR( double *, double *, double *, SUBGRID );

int refineGridSetBC( double *, double *, double *, double *, double *, double *,
        SUBGRID , SUBGRID  );

int refineGridSet_AMR( double *, double *, double *, double *, double *, double *,
        SUBGRID , SUBGRID );

int setSubgrid( SUBGRID * );

int Cyclic_Reduction(double *, double *, double *, double *, int , int );

int PoisConjGrad( double *, double *, double *, SUBGRID );

int sor( double *, double *, double *, double *, double *); 

int MarkerAndCell( double *, double *, double *, double *, SUBGRID );

int applyBC_freestream( double *, double *, double *, double *, double *, SUBGRID );

int applyBC_free2( double *, double *, double *, double *, double *, SUBGRID );

int applyBC_noslip(double *, double *, double *, double *, double *, SUBGRID, int, int, int, int);

int applyBC_inflow( double *, SUBGRID );

int applyBC_box(double *, double *, double *, SUBGRID, int, int, int, int);

int bcset( BOUND *, int *, NODE * );

int flwMemory( double **, int , int **, int , MATL **, int, XYZI **, int , NODE **, int,
        SUBGRID **, int );

int dof, nmat, numel, numnp, sof, nx, ny, nz;
double dx, dy, dz;
int flag_2D = 0, flag_3D = 1, print_flag = 0;
double dt, time_current, err;         /* time step for time integration */
int nmax, nmax2, nstep, cstep;
int imax, jmax, jb, jt, kb, kt;
int hbox, lbox, xpbox, ypbox, stn;
double q, alp, Re;



int main(int argc, char **argv)
{
	int i, i2, j, j2, k, n, n2, jxp, kyp, dum, dum2, node_num;
        int check, name_length, counter, MemoryCounter;
	int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        double time, time_final;
        double pxm1, pxp1, pym1, pyp1;
        NODE *node, *node1, *node2, *node3, *mem_node;
        SUBGRID subgrid0, *subgrid1, *subgrid2, *subgrid3, *mem_SUBGRID;
        XYZI *mem_XYZI; 
        int *mem_int, sofmi, sofmf, sofmXYZI, sofmNODE, sofmSUBGRID, ptr_inc;
        MATL *matl; 
        double fdum, fdum2, factor;
        double dns, temp, vx, vy, pres, ene, enthal, mach;
        double *mem_double;
	double *u, *v, *prs, *F, *G, *u1, *v1, *prs1, *F1, *G1, *u2, *v2, *prs2, *F2, *G2;
	double usq1, usq2, vsq1, vsq2, uv1, uv2;
	double uh, uhm, vh, vhm, uh2, uhm2, vh2, vhm2, umax;
	double cr, cl, cvr, cvl, cr2, cl2, cvr2, cvl2, px, py; 
	double gx, gy, perx, pery;
	double *xst, *yst, *xsto, *ysto, *xp, *yp;
	double *D0, *F0, *vel_grad;
        char name[30], buf[ BUFSIZ ], *ccheck, one_char;
	FILE *o1, *o3, *o4, *box, *ostream[22], *onmax2, *datastream, *gplot;
        BOUND bc;
	int subypbox, subxpbox, sublbox, subhbox;


        memset(name,0,30*sizeof(char));
        ccheck = strncpy(name, "abcd", 4);
        if(!ccheck) printf( " Problems with strncpy \n");

        factor = 1.00;
	nmax = 3;

        nx = imax = factor*68*3/2;
        ny = jmax = factor*45;

        jb = factor*14;
        jt = factor*68;
        kb = factor*1;
        kt = factor*45;
        hbox = factor*4;
        lbox = factor*5;
        xpbox = factor*23;
        ypbox = factor*23;

/*
        nx = imax = factor*68;
        ny = jmax = factor*65;

        jb = factor*1;
        jt = factor*93;
        kb = factor*1;
        kt = factor*33;
        hbox = factor*5;
        lbox = factor*5;
        xpbox = factor*23;
        ypbox = factor*30;
*/

        stn = 21;

        q = 0.375;
	Re = 100.0; /* Reynolds number */
        alp = 1.0/Re;

        dx = 0.266/factor;
        dy = 0.266/factor;
        time = 0.0;
        counter = 0;
        dt = 0.020/factor;
        /*dt = 0.150;*/


	onmax2 = fopen( "nmax2", "r" );
        fscanf( onmax2, "%d\n", &nmax2);
        fscanf( onmax2, "%lf", &err);
        /*nmax2 *= factor;*/

	o1 = fopen( "o1", "w" );
	o3 = fopen( "o3", "w" );
	o4 = fopen( "o4", "w" );
	box = fopen( "box", "w" );

        datastream = fopen( "datastream", "w" );
        gplot = fopen( "gplot", "w" );
	ostream[0] = fopen( "s0", "w" );
	ostream[1] = fopen( "s1", "w" );
	ostream[2] = fopen( "s2", "w" );
	ostream[3] = fopen( "s3", "w" );
	ostream[4] = fopen( "s4", "w" );
	ostream[5] = fopen( "s5", "w" );
	ostream[6] = fopen( "s6", "w" );
	ostream[7] = fopen( "s7", "w" );
	ostream[8] = fopen( "s8", "w" );
	ostream[9] = fopen( "s9", "w" );
	ostream[10] = fopen( "s10", "w" );
	ostream[11] = fopen( "s11", "w" );
	ostream[12] = fopen( "s12", "w" );
	ostream[13] = fopen( "s13", "w" );
	ostream[14] = fopen( "s14", "w" );
	ostream[15] = fopen( "s15", "w" );
	ostream[16] = fopen( "s16", "w" );
	ostream[17] = fopen( "s17", "w" );
	ostream[18] = fopen( "s18", "w" );
	ostream[19] = fopen( "s19", "w" );
	ostream[20] = fopen( "s20", "w" );
	ostream[21] = fopen( "s21", "w" );

	/* Establishing initial conditions for u,v, and pressure */

	gx=alp*dt/(dx*dx);
	gy=alp*dt/(dy*dy);
        dum = jmax;
        numnp = nx*ny;
        nmat = 1;

/*   Begin allocation of meomory */

        MemoryCounter = 0;

/* For the doubles */
        sofmf = 15*numnp + 2*stn*nmax2*nmax + 4*stn + 2*imax*imax;
        MemoryCounter += sofmf*sizeof(double);
        printf( "\n Memory requrement for doubles is %15d bytes\n",MemoryCounter);

/* For the integers */
        sofmi= 1 + numnp;
        MemoryCounter += sofmi*sizeof(int);
        printf( "\n Memory requrement for integers is %15d bytes\n",MemoryCounter);

/* For the XYZI integers */
        sofmXYZI=4;
        MemoryCounter += sofmXYZI*sizeof(XYZI);
        printf( "\n Memory requrement for XYZI integers is %15d bytes\n",MemoryCounter);

/* For the NODE structs */
        sofmNODE=numnp + 1;
        MemoryCounter += sofmNODE*sizeof(NODE);
        printf( "\n Memory requrement for NODE integers is %15d bytes\n",MemoryCounter);

/* For the SUBGRID structs */
        sofmSUBGRID=6*numnp;
        MemoryCounter += sofmSUBGRID*sizeof(SUBGRID);
        printf( "\n Memory requrement for SUBGRID integers is %15d bytes\n",MemoryCounter);

        check = flwMemory( &mem_double, sofmf, &mem_int, sofmi, &matl, nmat, &mem_XYZI,
                sofmXYZI, &node, sofmNODE, &mem_SUBGRID, sofmSUBGRID );
        if(!check) printf( " Problems with flwMemory\n");

/* For the doubles */
        mem_double=(double *)calloc(sofmf,sizeof(double));

                                                ptr_inc=0;
        u =(mem_double+ptr_inc);                ptr_inc += numnp;
        v =(mem_double+ptr_inc);                ptr_inc += numnp;
        F =(mem_double+ptr_inc);                ptr_inc += numnp;
        G =(mem_double+ptr_inc);                ptr_inc += numnp;
        prs =(mem_double+ptr_inc);              ptr_inc += numnp;
        u1 =(mem_double+ptr_inc);               ptr_inc += numnp;
        v1 =(mem_double+ptr_inc);               ptr_inc += numnp;
        F1 =(mem_double+ptr_inc);               ptr_inc += numnp;
        G1 =(mem_double+ptr_inc);               ptr_inc += numnp;
        prs1 =(mem_double+ptr_inc);             ptr_inc += numnp;
        u2 =(mem_double+ptr_inc);               ptr_inc += numnp;
        v2 =(mem_double+ptr_inc);               ptr_inc += numnp;
        F2 =(mem_double+ptr_inc);               ptr_inc += numnp;
        G2 =(mem_double+ptr_inc);               ptr_inc += numnp;
        prs2 =(mem_double+ptr_inc);             ptr_inc += numnp;
        vel_grad =(mem_double+ptr_inc);         ptr_inc += numnp;
	xst =(mem_double+ptr_inc);              ptr_inc += stn*nmax2*nmax;
        yst =(mem_double+ptr_inc);              ptr_inc += stn*nmax2*nmax;
        xsto =(mem_double+ptr_inc);             ptr_inc += stn;
        ysto =(mem_double+ptr_inc);             ptr_inc += stn;
        xp =(mem_double+ptr_inc);               ptr_inc += stn;
        yp =(mem_double+ptr_inc);               ptr_inc += stn;
        D0 =(mem_double+ptr_inc);               ptr_inc += imax*imax;
        F0 =(mem_double+ptr_inc);               ptr_inc += imax*imax;

/* For the integers */
                                              ptr_inc = 0;
        bc.num_none=(mem_int+ptr_inc);        ptr_inc += 1;

/* For the XYZI integers */
                                              ptr_inc = 0;
        bc.num_symm=(mem_XYZI+ptr_inc);       ptr_inc += 1;
        bc.num_far=(mem_XYZI+ptr_inc);        ptr_inc += 1;
        bc.num_free=(mem_XYZI+ptr_inc);       ptr_inc += 1;
        bc.num_inflow=(mem_XYZI+ptr_inc);     ptr_inc += 1;

/* For the SUBGRID structs*/
                                              ptr_inc = 0;
        subgrid1=(mem_SUBGRID+ptr_inc);       ptr_inc += 2*numnp;
        subgrid2=(mem_SUBGRID+ptr_inc);       ptr_inc += 2*numnp;
        subgrid3=(mem_SUBGRID+ptr_inc);       ptr_inc += 2*numnp;

/* I define subgrid0 to be the original coarsest mesh. */

        subgrid0.jstart = 0;
        subgrid0.jend = jmax;
        subgrid0.istart = 0;
        subgrid0.iend = imax;
        subgrid0.dx = dx;
        subgrid0.dy = dy;
        subgrid0.dt = dt;
        subgrid0.nx = nx;
        subgrid0.ny = ny;
        subgrid0.refine_num = 0;

	for( i = 0; i < numnp; ++i )
	{
             node[i].bc_type = 0;
        }

        for( i = 0; i < 2*numnp; ++i )
        {
                subgrid1[i].refine_num = 1;
                subgrid2[i].refine_num = 2;
                subgrid3[i].refine_num = 3;
        }


/* Set the boundary condition variable, bc, as well as allocate memory for each individual
   boundary condition.  

   This is just temporary.
*/
        check = bcset( &bc, (mem_int+1), node);
        if(!check) printf( " Problems with bcset\n");

	for( i = 0; i < imax; ++i )
	{
	   for( j = 0; j < jmax; ++j )
	   {
                ce  = jmax*i + j;

/*  For zero degree angle of attack */

                 u[ce] = 1.0;
                 v[ce] = 0.0;

/*  For 5 degree angle of attack */

                /*u[ce] = .9962;
                 v[ce] = 0.0872;*/
        
/*  For 15 degree angle of attack */

#if ANGLE
                u[ce] = 0.966;
                v[ce] = -0.25882;
#endif

/*  Initialize F and G */

                 F[ce] = u[ce];
                 G[ce] = v[ce];
                 umax = u[ce];
           }
	}

/* The boundary condtions inside the box are below. */

        /*check = applyBC_box(prs, u, v, subgrid0, hbox, lbox, xpbox, ypbox);*/
	check = applyBC_noslip(prs, F, G, u, v, subgrid0, hbox, lbox, xpbox, ypbox);

/* Below is the initial condition for the pressure */

	for( i = 0; i < imax; ++i )
	{
		for( j = 0; j < jmax; ++j )
		{
                   ce  = jmax*i + j;
		   prs[ce] = 0.0;
		}
	}

/*  The j,k loop below represents the boundary conditions on the
    rectangle.  It is the no slip b.c..
*/
	check = applyBC_noslip(prs, F, G, u, v, subgrid0, hbox, lbox, xpbox, ypbox);

/* These are the initial points for the streaklines and pathlines */

/* For pathlines */

        xp[0] = 3.; yp[0] = 5.2;
        xp[1] = 3.; yp[1] = 5.3;
        xp[2] = 3.; yp[2] = 5.4;
        xp[3] = 3.; yp[3] = 5.5;
        xp[4] = 3.; yp[4] = 5.6;
        xp[5] = 3.; yp[5] = 6.3;
        xp[6] = 3.; yp[6] = 6.4;
        xp[7] = 3.; yp[7] = 6.5;
        xp[8] = 3.; yp[8] = 6.6;
        xp[9] = 3.; yp[9] = 6.7;
        xp[10] = 3.; yp[10] = 6.8;
        xp[11] = 3.; yp[11] = 6.9;

/* For streaklines */

/*
        xsto[0] = 3.; ysto[0] = 5.2;
        xsto[1] = 3.; ysto[1] = 5.3;
        xsto[2] = 3.; ysto[2] = 5.4;
        xsto[3] = 3.; ysto[3] = 5.5;
        xsto[4] = 3.; ysto[4] = 5.6;
        xsto[5] = 3.; ysto[5] = 6.3;
        xsto[6] = 3.; ysto[6] = 6.4;
        xsto[7] = 3.; ysto[7] = 6.5;
        xsto[8] = 3.; ysto[8] = 6.6;
        xsto[9] = 3.; ysto[9] = 6.7;
        xsto[10] = 3.; ysto[10] = 6.8;
        xsto[11] = 3.; ysto[11] = 6.9;
*/
        fdum = 0.1;
        xsto[0] = 3.; ysto[0] = 3.4;
        xsto[0] = 3.; ysto[0] = 6.8;
        xsto[0] = 3.; ysto[0] = 5.9;
#if ANGLE
        xsto[0] = 3.; ysto[0] = 4.7;
        xsto[0] = 3.; ysto[0] = 6.7;
#endif
        xsto[1] = 3.; ysto[1] = ysto[0] + fdum;
        xsto[2] = 3.; ysto[2] = ysto[1] + fdum;
        xsto[3] = 3.; ysto[3] = ysto[2] + fdum;
        xsto[4] = 3.; ysto[4] = ysto[3] + fdum;
        xsto[5] = 3.; ysto[5] = ysto[4] + fdum;
        xsto[6] = 3.; ysto[6] = ysto[5] + fdum;
        xsto[7] = 3.; ysto[7] = ysto[6] + fdum;
        xsto[8] = 3.; ysto[8] = ysto[7] + fdum;
        xsto[9] = 3.; ysto[9] = ysto[8] + fdum;
        xsto[10] = 3.; ysto[10] = ysto[9] + fdum;
        xsto[11] = 3.; ysto[11] = ysto[10] + fdum;
        xsto[12] = 3.; ysto[12] = ysto[11] + fdum;
        xsto[13] = 3.; ysto[13] = ysto[12] + fdum;
        xsto[14] = 3.; ysto[14] = ysto[13] + fdum;
        xsto[15] = 3.; ysto[15] = ysto[14] + fdum;
        xsto[16] = 3.; ysto[16] = ysto[15] + fdum;
        xsto[17] = 3.; ysto[17] = ysto[16] + fdum;
        xsto[18] = 3.; ysto[18] = ysto[17] + fdum;
        xsto[19] = 3.; ysto[19] = ysto[18] + fdum;
        xsto[20] = 3.; ysto[20] = ysto[19] + fdum;


  for( n2 = 0; n2 < nmax2; ++n2 )
  {
     printf("%i \n",n2);
     for( n = 0; n < nmax; ++n )
     {

        /*dt = 0.563*dx/umax;
        printf(  "llll %16.8e %16.8e %16.8e %4d\n", umax, time, dt, counter);
        time += dt;
        counter += 1;
        if(time > 18.0) n2 = nmax2;
        if(umax > 1000) n2 = nmax2;*/
        /*dt = 0.188*dx/umax;
        fdum = 0.188*dx/umax;
        dt = 0.05;
        fdum = 0.188*dx/umax;
        if ( dt > fdum ) dt = fdum;
        printf(  "llll %9.5f %9.5f %9.5f %9.5f\n", umax, time, dt, fdum);*/

/*
        printf(  "ttttt %9.5f\n", dt);

        for( j2 = 40; j2 < 45; ++j2 )
        {
            for( i2 = 50; i2 < 55; ++i2 )
            {
                ce  = jmax*i2 + j2;
                printf(  "mmmmmm %d %d %d %9.5f %9.5f\n", i2, j2, ce, u[ce], v[ce]);
            }
        }
*/
/* This concludes the creation of the velocity field whithout the
   pressure correction.  sor below will then determine the pressure
   field.  The equation for this is based on Equation 17.13 page 339 in

   Fletcher, Clive A. J., Computational Techniques for Fluid Dynamics: Vol II, 2nd
        Ed., Springer-Verlag, Berlin, 1991.

   where the F is replaced with u and G is v.
*/

	check = MarkerAndCell( F, G, u, v, subgrid0);

/* Below is the bondary condition for the inflow at the left. 
   It represents linearally extrapolating the pressure
   from the 2 nodes to the right.
*/
       check = applyBC_inflow( prs, subgrid0);

/* Below is the bondary condition for the outflow at the right. 
   It represents linearally extrapolating the velocity and pressure
   from the previous 2 profiles.
*/
        check = applyBC_free2( F, G, prs, u, v, subgrid0);

/* Below is the bondary condition for the upper and lower part of the
   grid.  It represents saying the velocity is the freestream
   velocity at the top and bottom of the grid, i.e. dv/dn and du/dn
   are zero where n is the normal to the top of the grid. */
	
        check = applyBC_freestream( F, G, prs, u, v, subgrid0);

/* The no slip is reapplied
*/
	check = applyBC_noslip(prs, F, G, u, v, subgrid0, hbox, lbox, xpbox, ypbox);


	/*for( i = 0; i < imax; ++i )
	{
		for( j = 0; j < jmax; ++j )
		{
                   ce  = jmax*i + j;
		   prs[ce] = 0.0;
		}
	}*/

 	check = sor( F, G, u, v, prs);

/* Instead of SOR, a direct Poisson solver in the form of the Conjugate Gradient Method 
   is used instead. 
*/

        /*PoisConjGrad( F, G, prs, subgrid0);*/

        for( j2 = 40; j2 < 45; ++j2 )
        {
            for( i2 = 50; i2 < 55; ++i2 )
            {
                ce  = jmax*i2 + j2;
                printf(  "nnnnn %d %d %d %9.5f %9.5f %9.5f\n", i2, j2, ce, u[ce], v[ce], prs[ce]);
            }
        }



/* The pressure field has been calculated and now the pressure
   correction can be made below.
*/
        umax = 1.0;
	for( i = 1; i < imax-1; ++i )
	{
	   for( j = 0; j < jmax-1; ++j )
	   {
              ce  = jmax*i + j;
              xp1 = jmax*(i+1) + j;
              yp1 = jmax*i + (j+1);

	      u[ce] = F[ce] - dt/dx*(prs[xp1] - prs[ce]);
	      v[ce] = G[ce] - dt/dy*(prs[yp1] - prs[ce]);

              if( umax < fabs(u[ce]) ) umax = fabs(u[ce]);
              if( umax < fabs(v[ce]) ) umax = fabs(v[ce]);
	   }
	}

#if 0
/* Begin adaptive mesh refinement */

        subgrid1[0].jstart = ypbox - 5;
        subgrid1[0].jend = ypbox + hbox + 4;
        subgrid1[0].istart = xpbox + lbox;
        subgrid1[0].istart = 20;
        subgrid1[0].iend = imax - 3;

        subypbox = 2*(ypbox - subgrid1[0].jstart);
        subxpbox = 2*(xpbox - subgrid1[0].istart);
        sublbox = 2*lbox;
        subhbox = 2*hbox;

        if( n2 == 0 && n == 0)
        {
            check = setSubgrid( &subgrid1[0] );
            if(!check) printf( " Problems with setSubgrid\n");

            subgrid1[0].dt = dt;

            check = refineGridSet_AMR( u, v, prs, u1, v1, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with refineGridSet_AMR\n");

            check = refineGridSet_AMR( F, G, prs, F1, G1, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with refineGridSet_AMR\n");

            check = applyBC_noslip(prs1, F1, G1, u1, v1, subgrid1[0],
                subhbox, sublbox, subxpbox, subypbox);

/* It seems it's better not to interpolate prs to get an initial guess.  This is unlike
   when this calculation is contined later.
*/
            check = PoisConjGrad_AMR( F1, G1, prs1, subgrid1[0]);
            if(!check) printf( " Problems with PoisConjGrad_AMR\n");

/* The pressure field has been calculated and now the pressure
   correction can be made below.
*/
            dum = subgrid1[0].ny;
            fdum = subgrid1[0].dt/subgrid1[0].dx;
            fdum2 = subgrid1[0].dt/subgrid1[0].dy;
            for( i = 1; i < subgrid1[0].nx-1; ++i )
            {
                for( j = 1; j < subgrid1[0].ny-1; ++j )
                {
                    ce  = dum*i + j;
                    xp1 = dum*(i+1) + j;
                    yp1 = dum*i + (j+1);

                    u1[ce] = F1[ce] - fdum*(prs1[xp1] - prs1[ce]);
                    v1[ce] = G1[ce] - fdum2*(prs1[yp1] - prs1[ce]);

                }
            }

            check = applyBC_noslip(prs1, F1, G1, u1, v1, subgrid1[0],
                subhbox, sublbox, subxpbox, subypbox);

            /*check = reset( u1, v1, F1, G1, subgrid1[0]);*/

        }
        else
        {
            check = setSubgrid( &subgrid1[0] );
            if(!check) printf( " Problems with setSubgrid\n");

#if 0
            check = refineGridSetBC( u, v, prs, u1, v1, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with PoisConjGrad_AMR\n");

            check = refineGridSetBC( F, G, prs, F1, G1, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with PoisConjGrad_AMR\n");

/* Unlike for the velocities, I extrapolate all of the pressure. */

            /*check = refineGridSet( prs, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with refineGridSet\n");*/
#endif

            for( i2 = 0; i2 < 2; ++i2 )
            {
		check = MarkerAndCell( F1, G1, u1, v1, subgrid1[0]);

                check = applyBC_noslip(prs1, F1, G1, u1, v1, subgrid1[0],
                        subhbox, sublbox, subxpbox, subypbox);

                check = PoisConjGrad_AMR( F1, G1, prs1, subgrid1[0]);
                if(!check) printf( " Problems with PoisConjGrad_AMR\n");

/* The pressure field has been calculated and now the pressure
   correction can be made below.
*/

                dum = subgrid1[0].ny;
                fdum = subgrid1[0].dt/subgrid1[0].dx;
                fdum2 = subgrid1[0].dt/subgrid1[0].dy;
                for( i = 1; i < subgrid1[0].nx; ++i )
                {
                    for( j = 1; j < subgrid1[0].ny; ++j )
                    {
                        ce  = dum*i + j;
                        xp1 = dum*(i+1) + j;
                        yp1 = dum*i + (j+1);

                        u1[ce] = F1[ce] - fdum*(prs1[xp1] - prs1[ce]);
                        v1[ce] = G1[ce] - fdum2*(prs1[yp1] - prs1[ce]);
                    }
                }

                check = applyBC_noslip(prs1, F1, G1, u1, v1, subgrid1[0],
                        subhbox, sublbox, subxpbox, subypbox);
            }

        }

        check = coarseGridSet_AMR( u, v, prs, u1, v1, prs1, subgrid1[0], subgrid0 );
        if(!check) printf( " Problems with coarseGridSet_AMR\n");

        check = coarseGridSet_AMR( F, G, prs, F1, G1, prs1, subgrid1[0], subgrid0 );
        if(!check) printf( " Problems with coarseGridSet_AMR\n");

            check = refineGridSetBC( u, v, prs, u1, v1, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with PoisConjGrad_AMR\n");

            check = refineGridSetBC( F, G, prs, F1, G1, prs1, subgrid1[0], subgrid0 );
            if(!check) printf( " Problems with PoisConjGrad_AMR\n");

/* It seems it's better not to interpolate prs to get an initial guess, but calculate it
   straight.
*/
            dum = subgrid1[0].nx*subgrid1[0].ny;
            memset(prs1,0,dum*sizeof(double));

#endif


#if 0

/* For nodes at the right. */

	for( j = 0; j < jmax-1; ++j )
	{
	      i = imax-1;
              ce  = jmax*i + j;
              xm1 = jmax*(i-1) + j;
              yp1 = jmax*i + (j+1);

              pxp1 = 2.0*prs[ce] - prs[xm1];
	      u[ce] = F[ce] - dt/dx*(pxp1 - prs[ce]);
	      v[ce] = G[ce] - dt/dy*(prs[yp1] - prs[ce]);
	}

/* For freestream nodes at the top. */

	for( i = 0; i < imax-1; ++i )
	{
	      j = jmax-1;
              ce  = jmax*i + j;
              xp1 = jmax*(i+1) + j;
              ym1 = jmax*i + j - 1;

              pyp1 = 2.0*prs[ce] - prs[ym1];
	      u[ce] = F[ce] - dt/dx*(prs[xp1] - prs[ce]);
	      v[ce] = G[ce] - dt/dy*(pyp1 - prs[ce]);
	}

/* For node at top right. */

	i = imax - 1;
	j = jmax - 1;
        ce  = jmax*i + j;
        xm1 = jmax*(i-1) + j;
        ym1 = jmax*i + (j-1);

        pxp1 = 2.0*prs[ce] - prs[xm1];
        pyp1 = 2.0*prs[ce] - prs[ym1];

	u[ce] = F[ce] - dt/dx*(pxp1 - prs[ce]);
	v[ce] = G[ce] - dt/dy*(pyp1 - prs[ce]);
#endif

/* Below is the bondary condition for the inflow at the left. 
   It represents linearally extrapolating the pressure
   from the 2 nodes to the right.
*/
       check = applyBC_inflow( prs, subgrid0);

/* Below is the bondary condition for the outflow at the right. 
   It represents linearally extrapolating the velocity and pressure
   from the previous 2 profiles.
*/
        check = applyBC_free2( F, G, prs, u, v, subgrid0);

/* Below is the bondary condition for the upper and lower part of the
   grid.  It represents saying the velocity is the freestream
   velocity at the top and bottom of the grid, i.e. dv/dn and du/dn
   are zero where n is the normal to the top of the grid. */
	
        check = applyBC_freestream( F, G, prs, u, v, subgrid0);

/* The no slip is reapplied
*/
	check = applyBC_noslip(prs, F, G, u, v, subgrid0, hbox, lbox, xpbox, ypbox);

/* After the pressure correction has been made, the boundary
   conditions mentioned previously are reapplied below.
*/
        /*check = applyBC_free(u, v, subgrid0);
        check = applyBC_free2( F, G, prs, u, v, subgrid0);*/

        check = streaklines( n2*nmax + n, xst, yst, xsto, ysto, u, v);
        check = pathlines( o3, xp, yp, u, v);

     }

  }

     for( i = subgrid1[0].istart; i < subgrid1[0].iend; ++i )
     {
           for( j = subgrid1[0].jstart; j < subgrid1[0].jend; ++j )
           {
              ce  = dum*i + j;

              node_num = ny*i + j;
              node[node_num].bc_type = 9;
           }
     }

     for( i = xpbox; i <= xpbox+lbox; ++i )
     {
            for( j = ypbox; j <= ypbox+hbox; ++j )
            {
                node_num = ny*i + j;
                node[node_num].bc_type = 1;
            }
     }


/* Plotting of the streaklines */

     dum = 0;
     for( i2 = 0; i2 < n2; ++i2 )
     {
        for( j = 0; j < nmax; ++j )
        {
          for( i = 0; i < stn; ++i )
          {
             fdum = xst[stn*dum+i] + yst[stn*dum+i];
             if(fdum > -10000)
             {
                /*printf(  "qqqq %d %d %9.5f %9.5f \n", dum,i,xst[stn*dum+i], yst[stn*dum+i]);*/
                /*fprintf(o4, "%i %i %9.5f %9.5f \n", dum,i,xst[stn*dum+i], yst[stn*dum+i]);*/
                fprintf(o4, "%9.5f %9.5f \n", xst[stn*dum+i], yst[stn*dum+i]);
                fprintf(ostream[i], "%9.5f %9.5f \n", xst[stn*dum+i], yst[stn*dum+i]);
                fprintf(datastream, "%9.5f %9.5f ", xst[stn*dum+i], yst[stn*dum+i]);
             }
             else
             {
                fprintf(datastream, "%9.5f %9.5f ", 0.0, 0.0);
             }
          }
          fprintf(datastream, "\n");
          dum += 1;
        }
     }

	    /*printf(  "%i %i %9.5f %9.5f \n", i,k,xp[i],yp[i] );*/
	    /*printf(  "%i %i %9.5f %9.5f \n", i,k,xp[i],yp[i] );*/
	    /*fprintf(o3,"%9.5f %9.5f \n",xp[i],yp[i]);*/


/*here:       printf(  " had to stop %i %i  \n", i,k );*/

/* Below is the instantaneous velocity vectors */

     for( i2 = 0; i2 < 4; ++i2 )
     {
	for( i = jb-1; i < jt; ++i )
	{
           for( j = kb-1; j < kt; ++j )
           {
                j = j+2;
                ce  = jmax*i + j;
                px = dx*(double)(i+1)+(double)(i2)*.5*dt*u[ce];
                py = dy*(double)(j+1)+(double)(i2)*.5*dt*v[ce];
                if( px > (double)(xpbox)*dx && px < (double)(xpbox+lbox)*dx)
                {
                    if( py > (double)(ypbox)*dy && py < (double)(ypbox+hbox)*dy)
                    {
                        px = dx*(double)jb;
                        py = dy*(double)kb;
                    }
                }
                fprintf(o1,"%9.5f %9.5f \n",px,py);
           }
           i = i+2;
	}
     }

/* To plot the box */
   fprintf(box,"%9.5f %9.5f \n",(double)(xpbox)*dx + .5*dx,(double)(ypbox)*dy + .5*dy);
   fprintf(box,"%9.5f %9.5f \n",(double)(xpbox+lbox)*dx + .5*dx,(double)(ypbox)*dy + .5*dy);
   fprintf(box,"%9.5f %9.5f \n",(double)(xpbox+lbox)*dx + .5*dx,(double)(ypbox+hbox)*dy + .5*dy);
   fprintf(box,"%9.5f %9.5f \n",(double)(xpbox)*dx + .5*dx,(double)(ypbox+hbox)*dy + .5*dy);
   fprintf(box,"%9.5f %9.5f \n",(double)(xpbox)*dx + .5*dx,(double)(ypbox)*dy + .5*dy);

	for( i = 0; i < nx; ++i )
	{
	    for( j = 0; j < ny; ++j )
	    {
                node_num = ny*i + j;

                node[node_num].i = i;
                node[node_num].j = j;

                dns    = node[node_num].dns        = 0.0;
                temp   = node[node_num].temp       = 0.0;
                vx     = node[node_num].vx         = u[node_num];
                vy     = node[node_num].vy         = v[node_num];
                pres   = node[node_num].prs        = prs[node_num];
                ene    = node[node_num].ene        = 0.0;
                enthal = node[node_num].enthal     = 0.0;

                fdum = (vx*vx + vy*vy);
                node[node_num].mach = sqrt(fdum);
                mach   = node[node_num].mach       = fdum;
            }
        }

        matl[0].gam = 1.0;
        matl[0].R = 1.0;

        check = writer_2D(bc, matl, name, node);
        if(!check) printf( " Problems with writer_3D\n");

        printf("\n\n");
        for( i = xpbox-3; i < xpbox+lbox+2; ++i ) printf("   %3d     ", i);
        printf("\n");
        for( j = ypbox+hbox+2-1; j > ypbox-4; --j )
        {
           printf("%d ", j);
           for( i = xpbox-3; i < xpbox+lbox+2; ++i )
           {
                 printf("(%9.5f)", u[jmax*i+j]);
           }
           printf("\n");
        }

        printf("\n\n");
        for( i = xpbox-3; i < xpbox+lbox+2; ++i ) printf("   %3d     ", i);
        printf("\n");
        for( j = ypbox+hbox+2-1; j > ypbox-4; --j )
        {
           printf("%d ", j);
           for( i = xpbox-3; i < xpbox+lbox+2; ++i )
           {
                 printf("(%9.5f)", v[jmax*i+j]);
           }
           printf("\n");
        }

        printf("\n\n");
        for( i = xpbox-3; i < xpbox+lbox+2; ++i ) printf("   %3d     ", i);
        printf("\n");
        for( j = ypbox+hbox+2-1; j > ypbox-4; --j )
        {
           printf("%d ", j);
           for( i = xpbox-3; i < xpbox+lbox+2; ++i )
           {
                 printf("(%9.5f)", prs[jmax*i+j]);
           }
           printf("\n");
        }

        printf("%d ", j);

/* This prepares a gnuplot file. */


        fprintf( gplot, "#set term postscript\n");
        fprintf( gplot, "#set out 'test.ps'\n");
        fprintf( gplot, "#set term png\n");
        fprintf( gplot, "#set out 'test.png'\n");
        /*fprintf( gplot, "set yrange [3:12]\n");
        fprintf( gplot, "set xrange [2.5:18]\n");*/
        fprintf( gplot, "set yrange [0:14]\n");
        fprintf( gplot, "set xrange [2.5:28]\n");
        fprintf( gplot, "plot  \"datastream\" using 1:2 with points 0, \\\n");
        fprintf( gplot, "      \"datastream\" using 3:4 with points 1, \\\n");
        fprintf( gplot, "      \"datastream\" using 5:6 with points 2, \\\n");
        fprintf( gplot, "      \"datastream\" using 7:8 with points 3, \\\n");
        fprintf( gplot, "      \"datastream\" using 9:10 with points 4 , \\\n");
        fprintf( gplot, "      \"datastream\" using 11:12 with points 5, \\\n");
        fprintf( gplot, "      \"datastream\" using 13:14 with points 6, \\\n");
        fprintf( gplot, "      \"datastream\" using 15:16 with points 7, \\\n");
        fprintf( gplot, "      \"datastream\" using 17:18 with points 8, \\\n");
        fprintf( gplot, "      \"datastream\" using 19:20 with points 9, \\\n");
        fprintf( gplot, "      \"datastream\" using 21:22 with points 10, \\\n");
        fprintf( gplot, "      \"datastream\" using 23:24 with points 11, \\\n");
        fprintf( gplot, "      \"datastream\" using 25:26 with points 12, \\\n");
        fprintf( gplot, "      \"datastream\" using 27:28 with points 13, \\\n");
        fprintf( gplot, "      \"datastream\" using 29:30 with points 14, \\\n");
        fprintf( gplot, "      \"datastream\" using 31:32 with points 15, \\\n");
        fprintf( gplot, "      \"datastream\" using 33:34 with points 16, \\\n");
        fprintf( gplot, "      \"datastream\" using 35:36 with points 17, \\\n");
        fprintf( gplot, "      \"datastream\" using 37:38 with points 18, \\\n");
        fprintf( gplot, "      \"datastream\" using 39:40 with points 19, \\\n");
        fprintf( gplot, "      \"box\" with lines\n");
        fprintf( gplot, "#set out\n");
        
        return 1;
}


