/* 
  This utility preforms LU decomposition on a Matrix with pivoting.
  It is to be used in conjunction with lubksb.c.  It is taken from:

  Press, William H., et. al., Numerical Recipes in C 2nd Ed., Cambridge University
        Press, New York, 1992.

  page 46-47.  I have made a few modifications.


        a        coefficient matrix
        b        rhs vector
        indx     record of the pivoting 

		Updated 2/19/99
*/


#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define small		1.0e-20
#define NR_END		1
#define FREE_ARG	char*


int free_vector(double *v, long nl, long nh)
/* free a double vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

int ludcmp(double *a, int n, int *indx, double *d, double *vv)
{
	int i,imax,j,k;
	double big,dum,sum,tp;

	*d=1.0;
	for (i=0;i<=n-1; ++i)
	{
		big=0.0;
		for (j=0;j<=n-1; ++j)
		{
			if ((tp=fabs(*(a+n*i+j))) > big) big=tp;
		}
		if (big == 0.0) 
		{
	 		printf("Singular matrix in routine ludcmp");
		}
		*(vv+i)=1.0/big;
	 	/*printf("%d \n",j);
	 	printf("%d %d %f %f %f %f \n",i,j,a[i][j],tp,big,vv);*/
	}
	for (j=0;j<=n-1;++j) 
	{
		for (i=0;i<j;++i)
		{
			sum=*(a+n*i+j);
			for (k=0;k<i;++k) 
			{
			   sum -=*(a+n*i+k)*(*(a+n*k+j));
			}
			*(a+n*i+j)=sum;
			/*printf("%f ",*(a+i*n+j));*/
		}
		big=0.0;
		for (i=j;i<=n-1;++i)
		{
			sum=*(a+n*i+j);
			for (k=0;k<j;++k)
			{
			   	sum -=*(a+n*i+k)*(*(a+n*k+j));
			}
			*(a+n*i+j)=sum;
			if ((dum=*(vv+i)*fabs(sum)) >= big) 
			{
				big = dum;
				imax=i;
			}
		}
		if (j != imax)
		{
			for (k=0;k<=n-1;++k)
			{
				dum=*(a+n*imax+k);
				*(a+n*imax+k)=*(a+n*j+k);
				*(a+n*j+k)=dum;
			}
			*d = -(*d);
			*(vv+imax)=*(vv+j);
		}
		*(indx+j)=imax;
		if (*(a+n*j+j) == 0.0) 
		{
			*(a+n*j+j)*small;
		}
		if (j != n-1)
		{
			dum=1.0/(*(a+n*j+j));
			for (i=j+1;i<=n-1;++i)
			{
				*(a+n*i+j) *= dum;
			}
		}
	}
	return 1;
}

