/* This program reapplies the boundary conditions for the program
   which uses the finite difference method to calculate an incomprssible
   laminar CFD flow field.  

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/flwconst.h"
#include "../compressible/flwstruct.h"
#include "../AMR/amr.h"

#define PT666 1.0/3.0

extern double dx, dy, dz;
extern double dt, Re;


int applyBC_box(double *prs, double *u, double *v, SUBGRID subgrid, int hbox, int lbox, int xpbox, int ypbox)
{
/*
                     Updated 3/29/07
*/

        int i, j, k, dum;
        int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        int lboxh, hboxh;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

/* The boundary condtions inside the box are below. */

        lboxh = lbox/2;
        hboxh = hbox/2;

/* The velocities on the surface of the box is set to zero. */

        for( i = xpbox-1; i < xpbox+lbox; ++i )
        {
               j = ypbox-1;
               ce = ny*i + j;
               u[ce] = 0.0;
               v[ce] = 0.0;

               j = ypbox+hbox-1;
               ce = ny*i + j;
               u[ce] = 0.0;
               v[ce] = 0.0;
        }
        for( j = ypbox-1; j < ypbox+hbox; ++j )
        {
               i = xpbox-1;
               ce = ny*i + j;
               u[ce] = 0.0;
               v[ce] = 0.0;

               i = xpbox+lbox-1;
               ce = ny*i + j;
               u[ce] = 0.0;
               v[ce] = 0.0;
        }

/* The velocities in the inside of the box based of a parabolic fit of velocities on
   nodes in opposite directions. */

        for( j = ypbox-1; j < ypbox+hbox; ++j )
        {
           for( i = xpbox-1; i < xpbox+lboxh; ++i )
           {
               ce  = ny*i + j;
               xm1 = ny*(i-1) + j;
               xm2 = ny*(i-2) + j;
               xp1 = ny*(i+1) + j;
               xp2 = ny*(i+2) + j;
               ym1 = ny*i + (j-1);
               ym2 = ny*i + (j-2);
               yp1 = ny*i + (j+1);
               yp2 = ny*i + (j+2);
               xm1ym1 = ny*(i-1) + (j-1);
               xm1yp1 = ny*(i-1) + (j+1);
               xp1ym1 = ny*(i+1) + (j-1);
               xp1yp1 = ny*(i+1) + (j+1);

               u[ce] = 4.0*PT666*u[xm1] - PT666*u[xm2];
               v[ce] = 4.0*PT666*v[xm1] - PT666*v[xm2];
               prs[ce] = 4.0*PT666*prs[xm1] - PT666*prs[xm2];
           }
           for( i = xpbox+lboxh; i < xpbox+lbox; ++i )
           {
               ce  = ny*i + j;
               xm1 = ny*(i-1) + j;
               xm2 = ny*(i-2) + j;
               xp1 = ny*(i+1) + j;
               xp2 = ny*(i+2) + j;
               ym1 = ny*i + (j-1);
               ym2 = ny*i + (j-2);
               yp1 = ny*i + (j+1);
               yp2 = ny*i + (j+2);
               xm1ym1 = ny*(i-1) + (j-1);
               xm1yp1 = ny*(i-1) + (j+1);
               xp1ym1 = ny*(i+1) + (j-1);
               xp1yp1 = ny*(i+1) + (j+1);

               u[ce] = 4.0*PT666*u[xp1] - PT666*u[xp2];
               v[ce] = 4.0*PT666*v[xp1] - PT666*v[xp2];
               prs[ce] = 4.0*PT666*prs[xp1] - PT666*prs[xp2];
           }
        }

     return 1;
}

int applyBC_inflow( double *prs, SUBGRID subgrid)
{
/* Below is the bondary condition for the inflow at the left.
   It reprsents linearally extrapolating the pressure from the
   2 nodes to the right.

                     Updated 3/29/07
*/
        int i, j, k, dum;
        int ce, xp1, xp2;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

        for( j = 1; j < ny-1; ++j )
        {
            i = 0;
            ce = ny*i+j;
            xp1  = ny*(i+1) + j;
            xp2  = ny*(i+2) + j;
            prs[ce] = 2.0*prs[xp1] - prs[xp2];
            prs[ce] = 0.0;
        }

        return 1;
}

#if 1
int applyBC_free2( double *F, double *G, double *prs, double *u, double *v, SUBGRID subgrid)
{
/* Below is the bondary condition for the outflow at the right.
   It reprsents linearally extrapolating the velocity and pressure
   from the previous 2 profiles.

                     Updated 3/29/07
*/
        int i, j, kk, m, dum, counter;
        int ce, xm1, xm2, xp1, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        double pxm1, pxp1, pym1, pyp1;
        double bets, fdum;
        double usq1, usq2, vsq1, vsq2, uv1, uv2, pi, df;
        double uxp1, vxp1, vxp1ym1;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

        if(0)
        {
                i = nx - 1;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xp1 = ny*(i+1) + j;
                ym1 = ny*i + (j-1);
                yp1 = ny*i + (j+1);
                xm1ym1 = ny*(i-1) + j-1;

                uxp1 = 2.0*u[ce] - u[xm1];
                vxp1 = 2.0*v[ce] - v[xm1];
                vxp1ym1 = 2.0*v[ym1] - v[xm1ym1];

                usq1 = (u[xm1] + u[ce])/2.0;
                usq1 = usq1*usq1;
                usq2 = (u[ce] + uxp1)/2.0;
                usq2 = usq2*usq2;
                uv1 = (u[ym1] + u[ce])*(vxp1ym1 + v[ym1])/4.0;
                uv2 = (u[ce] + u[yp1])*(vxp1 + v[ce])/4.0;

                F[ce] = u[ce] + dt*(uxp1 - 2*u[ce] + u[xm1])/dx/dx/Re +
                     dt*(u[ym1] - 2*u[ce] + u[yp1])/dy/dy/Re -
                     dt*(usq2 - usq1)/dx - dt*(uv2 - uv1)/dy;

                vsq1 = (v[ym1] + v[ce])/2.0;
                vsq1 = vsq1*vsq1;
                vsq2 = (v[ce] + v[yp1])/2.0;
                vsq2 = vsq2*vsq2;
                uv1 = (u[xm1] + u[xm1yp1])*(v[ce] + v[xm1])/4.0;
                uv2 = (u[ce] + u[yp1])*(vxp1 + v[ce])/4.0;

                G[ce] = v[ce] + dt*(vxp1 - 2*v[ce] + v[xm1])/dx/dx/Re +
                     dt*(v[yp1] - 2*v[ce] + v[ym1])/dy/dy/Re -
                     dt*(vsq2 - vsq1)/dy - dt*(uv2 - uv1)/dx;
        }

        for( j = 1; j < ny; ++j )
        {
                i = nx - 1;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                F[ce] = 2.0*F[xm1] - F[xm2];
                G[ce] = 2.0*G[xm1] - G[xm2];
                u[ce] = 2.0*u[xm1] - u[xm2];
                v[ce] = 2.0*v[xm1] - v[xm2];

        }

        return 1;

}
#endif

#if 0
int applyBC_free2( double *F, double *G, double *prs, double *u, double *v, SUBGRID subgrid)
{
/* Below is the bondary condition for the outflow at the right.
   It reprsents linearally extrapolating the velocity and pressure
   from the previous 2 profiles.

                     Updated 3/29/07
*/
        int i, j, k, dum;
        int ce, xm1, xm2;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

        for( j = 1; j < ny; ++j )
        {
                i = nx - 1;
                ce = ny*i+j;
                xm1  = ny*(i-2) + j;
                xm2  = ny*(i-3) + j;

/*
                u[ce] = u[xm1]+(u[xm1] - u[xm2])/dx;
                v[ce] = v[xm1]+(v[xm1] - v[xm2])/dx;
*/
/*
                u[ce] = 2.0*u[xm1] - u[xm2];
                v[ce] = 2.0*v[xm1] - v[xm2];
                prs[ce] = 2.0*prs[xm1] - prs[xm2];
*/

                u[ce] = u[xm1];
                v[ce] = v[xm1];
                /*prs[ce] = 0.0;*/
        }

        return 1;

}
#endif


#if 0
int applyBC_free2( double *F, double *G, double *prs, double *u, double *v, SUBGRID subgrid)
{
/* Below is the bondary condition for the outflow at the right.
   It reprsents linearally extrapolating the velocity and pressure
   from the previous 2 profiles.

                     Updated 3/29/07
*/
        int i, j, k, dum;
        int ce, xm1, xm2, xp1, ym1, ym2, yp1, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        double uh, uhm, vh, vhm, uh2, uhm2, vh2, vhm2;
        double cr, cl, cvr, cvl, cr2, cl2, cvr2, cvl2, px, py;
        double gx, gy;
        double uxp1, vxp1, vxp1ym1;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

        i = nx - 1;
        j = 1;
        ce = ny*i+j;
        xm1  = ny*(i-1) + j;
        xm2  = ny*(i-2) + j;

        u[ce] = 2.0*u[xm1] - u[xm2];
        v[ce] = 2.0*v[xm1] - v[xm2];
        /*prs[ce] = 2.0*prs[xm1] - prs[xm2];
        prs[ce] = 0.0;*/

        i = nx - 1;
        j = ny-1;
        ce = ny*i+j;
        xm1  = ny*(i-1) + j;
        xm2  = ny*(i-2) + j;

        u[ce] = 2.0*u[xm1] - u[xm2];
        v[ce] = 2.0*v[xm1] - v[xm2];
        /*prs[ce] = 2.0*prs[xm1] - prs[xm2];
        prs[ce] = 0.0;*/

        gx=alp*dt/(dx*dx);
        gy=alp*dt/(dy*dy);
        gx=0.0;
        gy=0.0;

        for( j = 2; j < ny-1; ++j )
        {
                ce  = ny*(nx-1) + j;
                xm1 = ny*(nx-2) + j;
                xm2 = ny*(nx-3) + j;
                xp1 = ny*(nx) + j;
                ym1 = ny*(nx-1) + (j-1);
                ym2 = ny*(nx-1) + (j-2);
                yp1 = ny*(nx-1) + (j+1);
                xm1ym1 = ny*(nx-2) + (j-1);
                xm1yp1 = ny*(nx-2) + (j+1);
                xp1ym1 = ny*(nx) + (j-1);
                xp1yp1 = ny*(nx) + (j+1);

                uxp1 = 2.0*u[ce] - u[xm1];
                vxp1 = 2.0*v[ce] - v[xm1];
                vxp1ym1 = 2.0*v[ym1] - v[xm1ym1];

                uh = .5*(u[ce]+uxp1)-q/3*(u[xm1]-2*u[ce]+uxp1);
                uhm = .5*(u[ce]+u[xm1])-q/3*(u[xm2]-2*u[xm1]+u[ce]);
                vh = .5*(vxp1+v[ce])-q/3*(v[xm1]-2*v[ce]+vxp1);
                vhm = .5*(vxp1ym1+v[ym1])-q/3*(v[xm1ym1]-2*v[ym1]+vxp1ym1);

                cr = dt*uh/dx;
                cl = dt*uhm/dx;
                cvr = dt*vh/dy;
                cvl = dt*vhm/dy;

                uh2 = .5*(u[yp1]+u[ce])-q/3*(u[ym1]-2*u[ce]+u[yp1]);
                uhm2 = .5*(u[xm1yp1]+u[xm1])-q/3*(u[xm1ym1]-2*u[xm1]+u[xm1yp1]);
                vh2 = .5*(v[ce]+v[yp1])-q/3*(v[ym1]-2*v[ce]+v[yp1]);
                vhm2 = .5*(v[ce]+v[ym1])-q/3*(v[ym2]-2*v[ym1]+v[ce]);

                cr2 = dt*uh2/dx;
                cl2 = dt*uhm2/dx;
                cvr2 = dt*vh2/dy;
                cvl2 = dt*vhm2/dy;

                u[ce] = u[ce]
                     -cr*(.5*(u[ce]+uxp1)-.5*cr*(uxp1-u[ce])
                     -( (1-cr*cr)/6 -gx )*(u[xm1]-2*u[ce]+uxp1))

                     +cl*(.5*(u[xm1]+u[ce])-.5*cl*(u[ce]-u[xm1])
                     -( (1-cl*cl)/6 -gx )*(u[xm2]-2*u[xm1]+u[ce]))

                     -cvr*(.5*(u[ce]+u[yp1])-.5*cvr*(u[yp1]-u[ce])
                     -( (1-cvr*cvr)/6 -gy )*(u[ym1]-2*u[ce]+u[yp1]))

                     +cvl*(.5*(u[ym1]+u[ce])-.5*cvl*(u[ce]-u[ym1])
                     -( (1-cvl*cvl)/6 -gy )*(u[ym2]-2*u[ym1]+u[ce]))

                     +gx*(u[xm1]-2*u[ce]+uxp1)
                     +gy*(u[ym1]-2*u[ce]+u[yp1]);

                v[ce] = v[ce]
                     -cr2*(.5*(v[ce]+vxp1)-.5*cr2*(vxp1-v[ce])
                     -( (1-cr2*cr2)/6 -gx )*(v[xm1]-2*v[ce]+vxp1))

                     +cl2*(.5*(v[xm1]+v[ce])-.5*cl2*(v[ce]-v[xm1])
                     -( (1-cl2*cl2)/6 -gx )*(v[xm2]-2*v[xm1]+v[ce]))

                     -cvr2*(.5*(v[ce]+v[yp1])-.5*cvr2*(v[yp1]-v[ce])
                     -( (1-cvr2*cvr2)/6 -gy )*(v[ym1]-2*v[ce]+v[yp1]))

                     +cvl2*(.5*(v[ym1]+v[ce])-.5*cvl2*(v[ce]-v[ym1])
                     -( (1-cvl2*cvl2)/6 -gy )*(v[ym2]-2*v[ym1]+v[ce]))

                     +gx*(v[xm1]-2*v[ce]+vxp1)
                     +gy*(v[ym1]-2*v[ce]+v[yp1]);

/*
                u[ce] = u[ce]
                     -cr*(.5*(u[xm1]+u[ce])-.5*cr*(u[ce]-u[xm1])
                     -( (1-cr*cr)/6 -gx )*(u[xm1]-2*u[xm2]+u[ce]))

                     +cl*(.5*(u[xm1]+u[ce])-.5*cl*(u[ce]-u[xm1])
                     -( (1-cl*cl)/6 -gx )*(u[xm2]-2*u[xm1]+u[ce]))

                     -cvr*(.5*(u[ce]+u[yp1])-.5*cvr*(u[yp1]-u[ce])
                     -( (1-cvr*cvr)/6 -gy )*(u[ym1]-2*u[ce]+u[yp1]))

                     +cvl*(.5*(u[ym1]+u[ce])-.5*cvl*(u[ce]-u[ym1])
                     -( (1-cvl*cvl)/6 -gy )*(u[ym2]-2*u[ym1]+u[ce]))

                     +gx*(u[xm1]-2*u[ce]+uxp1)
                     +gy*(u[ym1]-2*u[ce]+u[yp1]);

                v[ce] = v[ce]
                     -cr2*(.5*(v[xm1]+v[ce])-.5*cr2*(v[ce]-v[xm1])
                     -( (1-cr2*cr2)/6 -gx )*(v[xm2]-2*v[xm1]+v[ce]))

                     +cl2*(.5*(v[xm1]+v[ce])-.5*cl2*(v[ce]-v[xm1])
                     -( (1-cl2*cl2)/6 -gx )*(v[xm2]-2*v[xm1]+v[ce]))

                     -cvr2*(.5*(v[ce]+v[yp1])-.5*cvr2*(v[yp1]-v[ce])
                     -( (1-cvr2*cvr2)/6 -gy )*(v[ym1]-2*v[ce]+v[yp1]))

                     +cvl2*(.5*(v[ym1]+v[ce])-.5*cvl2*(v[ce]-v[ym1])
                     -( (1-cvl2*cvl2)/6 -gy )*(v[ym2]-2*v[ym1]+v[ce]))

                     +gx*(v[xm1]-2*v[ce]+vxp1)
                     +gy*(v[ym1]-2*v[ce]+v[yp1]);
*/


                     /*prs[ce] = 2.0*prs[xm1] - prs[xm2];
                     prs[ce] = prs[xm1];*/
        }

        return 1;

}
#endif


int applyBC_freestream( double *F, double *G, double *prs, double *u, double *v, SUBGRID subgrid)
{
/* Below is the bondary condition for the upper and lower part of the
   grid.  It reprsents saying the velocity is the freestream
   velocity at the top and bottom of the grid, i.e. dv/dn and du/dn
   are zero where n is the normal to the top of the grid.

                     Updated 3/29/07
*/

        int i, j, k, dum;
        int ce, yp1, yp2, ym1, ym2;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

#if 1
        for( i = 1; i < nx; ++i )
        {
/* For the bottom exit. */

            j = 0;
            ce = ny*i + j;
            yp1  = ny*i + j+1;

            u[ce] = u[yp1];
            v[ce] = v[yp1];
            /*prs[ce] = prs[yp1];*/
            /*prs[ce] = 0.0;*/
            F[ce] = F[yp1];
            G[ce] = G[yp1];

/* For the top exit. */

            j = ny-1;
            ce = ny*i + j;
            ym1  = ny*i + j-1;

            u[ce] = u[ym1];
            v[ce] = v[ym1];
            /*prs[ce] = prs[ym1];*/
            /*prs[ce] = 0.0;*/
            F[ce] = F[ym1];
            G[ce] = G[ym1];

        }
#endif


         return 1;

}

int applyBC_free(double *u, double *v, SUBGRID subgrid)
{
/*
                     Updated 3/29/07
*/

        int i, j, k, dum;
        int ce, xp1, xp2, xm1, xm2, yp1, yp2, ym1, ym2;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

/* After the pressure correction has been made, the boundary
   conditions mentioned previously are reapplied below.
*/
	for( j = 1; j < ny; ++j )
	{
            i = nx-1;
            ce = ny*i + j;
            xp1  = ny*(i+1) + j;
            xp2  = ny*(i+1) + j;

/*
            u[ce] = u[xm1] + (u[xm1] - u[xm2])/dx;
            v[ce] = v[xm1] + (v[xm1] - v[xm2])/dx;
*/
            u[ce] = 2.0*u[xm1] - u[xm2];
            v[ce] = 2.0*v[xm1] - v[xm2];

	}
	/*for( i = 1; i < nx-1; ++i )
	{
            j = 0;
            ce = ny*i + j;
            yp1  = ny*(i+1) + 1;
            yp2  = ny*(i+1) + 2;

            u[ce] = u[yp1];
            v[ce] = v[yp1];

            j = ny-1;
            ce = ny*i + j;
            ym1  = ny*(i+1) - 1;
            ym2  = ny*(i+1) - 2;

            u[ce] = u[ym1];
            v[ce] = v[ym1];
	}*/

     return 1;
}


int applyBC_noslip2(double *prs, double *F, double *G, double *u, double *v, SUBGRID subgrid,
        int hbox, int lbox, int xpbox, int ypbox)
{
/*
                     Updated 3/29/07
*/

        int i, j, k, dum;
        int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        int lboxh, hboxh;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

/* The no slip is reapplied
*/

#if 0
	for( i = xpbox; i < xpbox+lbox-1; ++i )
	{
	   for( j = ypbox; j < ypbox+hbox-1; ++j )
	   {
                ce = ny*i + j;
	    	u[ce] = 0.;
	   	v[ce] = 0;
		u[ny*(xpbox-1)+j] = 0;
		v[ny*(xpbox+lbox-1)+j] = 0;
	   }

           j = ypbox-1;
           ce = ny*i + j;

           u[ce] = 0;

           j = ypbox+hbox-1;
           ce = ny*i + j;

           u[ce] = 0;

/*
           j = ypbox;
           ce = ny*i + j;
           ym1 = ny*i + j-1;

           v[ce] = -v[ym1];

           j = ypbox+hbox-2;
           ce = ny*i + j;
           yp1 = ny*i + j+1;

           v[ce] = -v[yp1];
*/
	}
#endif

/* The boundary condtions inside the box are below. */

        lboxh = lbox/2;
        hboxh = hbox/2;

/* The velocities on the surface of the box is set to zero. */

        for( i = xpbox-1; i < xpbox+lbox; ++i )
        {
               ce = ny*i + ypbox-1;
                u[ce] = 0.0;
                v[ce] = 0.0;
               ce = ny*i + ypbox+hbox-1;
                u[ce] = 0.0;
                v[ce] = 0.0;
        }
        for( j = ypbox-1; j < ypbox+hbox; ++j )
        {
               ce = ny*(xpbox-1) + j;
                u[ce] = 0.0;
                v[ce] = 0.0;
               ce = ny*(xpbox+lbox-1) + j;
                u[ce] = 0.0;
                v[ce] = 0.0;
        }

/* The velocities at the corners of the box is based on quadratic extrapolation of velocities of
   nodes upwind and downwind of the corresponding corner. */

#if 0
/* The code below has given me the most success in terms of stability in runnig the 
   0 angle of attack problem.
*/

/* Bottom left corner */
           ce = ny*(xpbox-1) + ypbox-1;
          xm1 = ny*(xpbox-2) + ypbox-1;
          xm2 = ny*(xpbox-3) + ypbox-1;
          ym1 = ny*(xpbox-1) + ypbox-2;
          ym2 = ny*(xpbox-1) + ypbox-3;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);

/* Bottom right corner */
           ce = ny*(xpbox+lbox-1) + ypbox-1;
          xm1 = ny*(xpbox+lbox-2) + ypbox-1;
          xm2 = ny*(xpbox+lbox-3) + ypbox-1;
          ym1 = ny*(xpbox+lbox-1) + ypbox-2;
          ym2 = ny*(xpbox+lbox-1) + ypbox-3;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);

/* Top left corner */
           ce = ny*(xpbox-1) + ypbox+hbox-1;
          xm1 = ny*(xpbox-2) + ypbox+hbox-1;
          xm2 = ny*(xpbox-3) + ypbox+hbox-1;
          yp1 = ny*(xpbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);

/* Top right corner */
           ce = ny*(xpbox+lbox-1) + ypbox+hbox-1;
          xm1 = ny*(xpbox+lbox-2) + ypbox+hbox-1;
          xm2 = ny*(xpbox+lbox-3) + ypbox+hbox-1;
          yp1 = ny*(xpbox+lbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox+lbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);
#endif

#if 1
/* Bottom left corner */
           ce = ny*(xpbox-1) + ypbox-1;
          xm1 = ny*(xpbox-2) + ypbox-1;
          xm2 = ny*(xpbox-3) + ypbox-1;
          ym1 = ny*(xpbox-1) + ypbox-2;
          ym2 = ny*(xpbox-1) + ypbox-3;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);

/* Bottom right corner */
           ce = ny*(xpbox+lbox-1) + ypbox-1;
          xm1 = ny*(xpbox+lbox-2) + ypbox-1;
          xm2 = ny*(xpbox+lbox-3) + ypbox-1;
          xp1 = ny*(xpbox+lbox) + ypbox-1;
          xp2 = ny*(xpbox+lbox+1) + ypbox-1;
          ym1 = ny*(xpbox+lbox-1) + ypbox-2;
          ym2 = ny*(xpbox+lbox-1) + ypbox-3;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
        if(u[xp1] < 0.0)
        {
                u[ce] = (4.0*u[xp1] - u[xp2])/3.0;
                v[ce] = (4.0*v[xp1] - v[xp2])/3.0;
        }

       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);

/* Top left corner */
           ce = ny*(xpbox-1) + ypbox+hbox-1;
          xm1 = ny*(xpbox-2) + ypbox+hbox-1;
          xm2 = ny*(xpbox-3) + ypbox+hbox-1;
          yp1 = ny*(xpbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);

/* Top right corner */
           ce = ny*(xpbox+lbox-1) + ypbox+hbox-1;
          xm1 = ny*(xpbox+lbox-2) + ypbox+hbox-1;
          xm2 = ny*(xpbox+lbox-3) + ypbox+hbox-1;
          xp1 = ny*(xpbox+lbox) + ypbox+hbox-1;
          xp2 = ny*(xpbox+lbox+1) + ypbox+hbox-1;
          yp1 = ny*(xpbox+lbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox+lbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
        if(u[xp1] < 0.0)
        {
                u[ce] = (4.0*u[xp1] - u[xp2])/3.0;
                v[ce] = (4.0*v[xp1] - v[xp2])/3.0;
        }
       printf(  "%d %d %d %9.5f %9.5f\n", ce, xm1, xm2, u[ce], v[ce]);
#endif

#if 0
/* Bottom left corner */
           ce = ny*(xpbox-1) + ypbox-1;
          xm1 = ny*(xpbox-2) + ypbox-1;
          xm2 = ny*(xpbox-3) + ypbox-1;
          ym1 = ny*(xpbox-1) + ypbox-2;
          ym2 = ny*(xpbox-1) + ypbox-3;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;

/* Bottom right corner */
           ce = ny*(xpbox+lbox-1) + ypbox-1;
          xp1 = ny*(xpbox+lbox) + ypbox-1;
          xp2 = ny*(xpbox+lbox+1) + ypbox-1;
          ym1 = ny*(xpbox+lbox-1) + ypbox-2;
          ym2 = ny*(xpbox+lbox-1) + ypbox-3;
        u[ce] = (4.0*u[ym1] - u[ym2])/3.0;
        v[ce] = (4.0*v[ym1] - v[ym2])/3.0;

/* Top left corner */
           ce = ny*(xpbox-1) + ypbox+hbox-1;
          xm1 = ny*(xpbox-2) + ypbox+hbox-1;
          xm2 = ny*(xpbox-3) + ypbox+hbox-1;
          yp1 = ny*(xpbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
        v[ce] = (4.0*v[xm1] - v[xm2])/3.0;

/* Top right corner */
           ce = ny*(xpbox+lbox-1) + ypbox+hbox-1;
          xp1 = ny*(xpbox+lbox) + ypbox+hbox-1;
          xp2 = ny*(xpbox+lbox+1) + ypbox+hbox-1;
          yp1 = ny*(xpbox+lbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox+lbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[yp1] - u[yp2])/3.0;
        v[ce] = (4.0*v[yp1] - v[yp2])/3.0;
#endif

#if 0
/* Bottom left corner */
           ce = ny*(xpbox-1) + ypbox-1;
          xm1 = ny*(xpbox-2) + ypbox-1;
          xm2 = ny*(xpbox-3) + ypbox-1;
          ym1 = ny*(xpbox-1) + ypbox-2;
          ym2 = ny*(xpbox-1) + ypbox-3;
        u[ce] = (4.0*u[ym1] - u[ym2])/3.0;
        v[ce] = (4.0*v[ym1] - v[ym2])/3.0;

/* Bottom right corner */
           ce = ny*(xpbox+lbox-1) + ypbox-1;
          xp1 = ny*(xpbox+lbox) + ypbox-1;
          xp2 = ny*(xpbox+lbox+1) + ypbox-1;
          ym1 = ny*(xpbox+lbox-1) + ypbox-2;
          ym2 = ny*(xpbox+lbox-1) + ypbox-3;
        u[ce] = (4.0*u[ym1] - u[ym2])/3.0;
        v[ce] = (4.0*v[ym1] - v[ym2])/3.0;

/* Top left corner */
           ce = ny*(xpbox-1) + ypbox+hbox-1;
          xm1 = ny*(xpbox-2) + ypbox+hbox-1;
          xm2 = ny*(xpbox-3) + ypbox+hbox-1;
          yp1 = ny*(xpbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[yp1] - u[yp2])/3.0;
        v[ce] = (4.0*v[yp1] - v[yp2])/3.0;

/* Top right corner */
           ce = ny*(xpbox+lbox-1) + ypbox+hbox-1;
          xp1 = ny*(xpbox+lbox) + ypbox+hbox-1;
          xp2 = ny*(xpbox+lbox+1) + ypbox+hbox-1;
          yp1 = ny*(xpbox+lbox-1) + ypbox+hbox;
          yp2 = ny*(xpbox+lbox-1) + ypbox+hbox+1;
        u[ce] = (4.0*u[yp1] - u[yp2])/3.0;
        v[ce] = (4.0*v[yp1] - v[yp2])/3.0;
#endif


/* The velocities in the inside of the box based on quadratic extrapolation of velocities of
   nodes upwind and downwind of the corresponding sides. */

#if 1
        for( j = ypbox; j < ypbox+hbox-1; ++j )
        {
           for( i = xpbox; i < xpbox+lboxh; ++i )
           {
               ce  = ny*i + j;
               xm1 = ny*(i-1) + j;
               xm2 = ny*(i-2) + j;
               xp1 = ny*(i+1) + j;
               xp2 = ny*(i+2) + j;
               ym1 = ny*i + (j-1);
               ym2 = ny*i + (j-2);
               yp1 = ny*i + (j+1);
               yp2 = ny*i + (j+2);
               xm1ym1 = ny*(i-1) + (j-1);
               xm1yp1 = ny*(i-1) + (j+1);
               xp1ym1 = ny*(i+1) + (j-1);
               xp1yp1 = ny*(i+1) + (j+1);

                u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
                v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
                /*prs[ce] = (4.0*prs[xm1] - prs[xm2])/3.0;*/
           }
           for( i = xpbox+lboxh; i < xpbox+lbox-1; ++i )
           {
               ce  = ny*i + j;
               xm1 = ny*(i-1) + j;
               xm2 = ny*(i-2) + j;
               xp1 = ny*(i+1) + j;
               xp2 = ny*(i+2) + j;
               ym1 = ny*i + (j-1);
               ym2 = ny*i + (j-2);
               yp1 = ny*i + (j+1);
               yp2 = ny*i + (j+2);
               xm1ym1 = ny*(i-1) + (j-1);
               xm1yp1 = ny*(i-1) + (j+1);
               xp1ym1 = ny*(i+1) + (j-1);
               xp1yp1 = ny*(i+1) + (j+1);

                u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
                v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
                /*prs[ce] = (4.0*prs[xm1] - prs[xm2])/3.0;*/

                /*u[ce] = (4.0*u[xp1] - u[xp2])/3.0;
                v[ce] = (4.0*v[xp1] - v[xp2])/3.0;
                prs[ce] = (4.0*prs[xp1] - prs[xp2])/3.0;*/
           }
        }
#endif

#if 0
        for( i = xpbox; i < xpbox+lbox-1; ++i )
        {
           for( j = ypbox; j < ypbox+hboxh; ++j )
           {
               ce  = ny*i + j;
               xm1 = ny*(i-1) + j;
               xm2 = ny*(i-2) + j;
               xp1 = ny*(i+1) + j;
               xp2 = ny*(i+2) + j;
               ym1 = ny*i + (j-1);
               ym2 = ny*i + (j-2);
               yp1 = ny*i + (j+1);
               yp2 = ny*i + (j+2);
               xm1ym1 = ny*(i-1) + (j-1);
               xm1yp1 = ny*(i-1) + (j+1);
               xp1ym1 = ny*(i+1) + (j-1);
               xp1yp1 = ny*(i+1) + (j+1);

                u[ce] = (4.0*u[ym1] - u[ym2])/3.0;
                v[ce] = (4.0*v[ym1] - v[ym2])/3.0;
                /*prs[ce] = (4.0*prs[ym1] - prs[ym2])/3.0;*/
            }
            for( j = ypbox+hboxh; j < ypbox+hbox-1; ++j )
            {
               ce  = ny*i + j;
               xm1 = ny*(i-1) + j;
               xm2 = ny*(i-2) + j;
               xp1 = ny*(i+1) + j;
               xp2 = ny*(i+2) + j;
               ym1 = ny*i + (j-1);
               ym2 = ny*i + (j-2);
               yp1 = ny*i + (j+1);
               yp2 = ny*i + (j+2);
               xm1ym1 = ny*(i-1) + (j-1);
               xm1yp1 = ny*(i-1) + (j+1);
               xp1ym1 = ny*(i+1) + (j-1);
               xp1yp1 = ny*(i+1) + (j+1);

                u[ce] = (4.0*u[yp1] - u[yp2])/3.0;
                v[ce] = (4.0*v[yp1] - v[yp2])/3.0;
                /*prs[ce] = (4.0*prs[yp1] - prs[yp2])/3.0;*/
           }
        }
#endif


        return 1;

}

#if 1
int applyBC_noslip(double *prs, double *F, double *G, double *u, double *v, SUBGRID subgrid,
        int hbox, int lbox, int xpbox, int ypbox)
{
/*
                     Updated 3/29/07
*/

        int i, j, k, dum;
        int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        int lboxh, hboxh;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

/* The no slip BC is reapplied.  Because I am using a staggard grid, there are many nodes
   which do not fall on the edge nodes of the box.  So for some nodes, I have to use
   extrapolation.
*/

/* The nodes for u which lie on the top and bottom edge of the box where the nodes
   of u falls on the nodes of the box are set to 0.0.
*/
   
	for( i = xpbox; i <= xpbox+lbox-1; ++i )
	{
/* bottom */
	   j = ypbox;
           ce   =  ny*i + j;
	   u[ce] = 0.0;
	   F[ce] = 0.0;

/* top */
	   j = ypbox + hbox;
           ce   =  ny*i + j;
	   u[ce] = 0.0;
	   F[ce] = 0.0;
	}

/* The nodes for v which lie on the left and right edge of the box where the nodes
   of v falls on the nodes of the box are set to 0.0.
*/
   
	for( j = ypbox; j <= ypbox+hbox-1; ++j )
	{
/* left */
	   i = xpbox;
           ce   =  ny*i + j;
	   v[ce] = 0.0;
	    G[ce] = 0.0;

/* right */
	   i = xpbox + lbox;
           ce   =  ny*i + j;
	   v[ce] = 0.0;
	    G[ce] = 0.0;
	}

/* For the velocities which fall just inside the edges of the surface of the box,
   I use quadratic extrapolation, where I extrapolate using an imaginary edge node to
   zero and then one of the nodes beyond that. */

/* for u */
        for( j = ypbox+1; j <= ypbox+hbox-1; ++j )
        {
/* u nodes immediately to the right of the box left edge */
                i = xpbox;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                u[ce] = -u[xm1]/3.0;
                F[ce] = -F[xm1]/3.0;
                /*u[ce] = -u[xm1];*/

/* u nodes immediately to the left of the box right edge */

                i = xpbox + lbox - 1;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                xp1 = ny*(i+1) + j;
                u[ce] = -u[xp1]/3.0;
                F[ce] = -F[xp1]/3.0;
                /*u[ce] = -u[xp1];
                u[ce] = (4.0*u[xm1] - u[xm2])/3.0;

                if(u[xp1] < 0.0)
                {
                        u[ce] = -u[xp1]/3.0;
                }*/
        }

/* for v */
        for( i = xpbox+1; i <= xpbox+lbox-1; ++i )
        {
/* v nodes immediately above box bottom edge */
                j = ypbox;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                xp1 = ny*(i+1) + j;
                ym1 = ny*i + j-1;
                v[ce] = -v[ym1]/3.0;
                G[ce] = -G[ym1]/3.0;
                /*v[ce] = -v[ym1];
                v[ce] = (4.0*v[xm1] - v[xm2])/3.0;*/

/* v nodes immediately below box top edge */

                j = ypbox + hbox - 1;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                xp1 = ny*(i+1) + j;
                yp1 = ny*i + j+1;
                v[ce] = -v[yp1]/3.0;
                G[ce] = -G[yp1]/3.0;
                /*v[ce] = -v[yp1];
                v[ce] = (4.0*v[xm1] - v[xm2])/3.0;*/

        }

/* The boundary condtions inside the box are below. */

/* The velocities in the inside of the box based on quadratic extrapolation of velocities of
   nodes upwind of the corresponding sides. */

/* for u */
        for( i = xpbox+1; i <= xpbox+lbox-2; ++i )
        {
           for( j = ypbox+1; j <= ypbox+hbox-1; ++j )
           {

                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;

                u[ce] = (4.0*u[xm1] - u[xm2])/3.0;
                F[ce] = (4.0*F[xm1] - F[xm2])/3.0;
                /*prs[ce] = (4.0*prs[xm1] - prs[xm2])/3.0;*/
           }
        }

/* for v */
        for( i = xpbox+1; i <= xpbox+lbox-1; ++i )
        {
           for( j = ypbox+1; j <= ypbox+hbox-2; ++j )
           {

                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;

                v[ce] = (4.0*v[xm1] - v[xm2])/3.0;
                G[ce] = (4.0*G[xm1] - G[xm2])/3.0;
                /*prs[ce] = (4.0*prs[xm1] - prs[xm2])/3.0;*/
           }
        }

        return 1;

}
#endif

#if 0
int applyBC_noslip(double *prs, double *F, double *G, double *u, double *v, SUBGRID subgrid,
        int hbox, int lbox, int xpbox, int ypbox)
{
/*
                     Updated 3/29/07
*/

        int i, j, k, dum;
        int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        int lboxh, hboxh;
        int nx, ny;

        ny = subgrid.ny;
        nx = subgrid.nx;

/* The no slip BC is reapplied.  Because I am using a staggard grid, there are many nodes
   which do not fall on the edge nodes of the box.  So for some nodes, I have to use
   extrapolation.
*/

/* The nodes for u which lie on the top and bottom edge of the box where the nodes
   of u falls on the nodes of the box are set to 0.0.
*/
   
	for( i = xpbox; i <= xpbox+lbox-1; ++i )
	{
/* bottom */
	   j = ypbox;
           ce   =  ny*i + j;
	   F[ce] = 0.0;

/* top */
	   j = ypbox + hbox;
           ce   =  ny*i + j;
	   F[ce] = 0.0;
	}

/* The nodes for v which lie on the left and right edge of the box where the nodes
   of v falls on the nodes of the box are set to 0.0.
*/
   
	for( j = ypbox; j <= ypbox+hbox-1; ++j )
	{
/* left */
	   i = xpbox;
           ce   =  ny*i + j;
	   G[ce] = 0.0;

/* right */
	   i = xpbox + lbox;
           ce   =  ny*i + j;
	   G[ce] = 0.0;
	}

/* For the velocities which fall just inside the edges of the surface of the box,
   I use quadratic extrapolation, where I extrapolate using an imaginary edge node to
   zero and then one of the nodes beyond that. */

/* for u */
        for( j = ypbox+1; j <= ypbox+hbox-1; ++j )
        {
/* u nodes immediately to the right of the box left edge */
                i = xpbox;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
               F[ce] = -F[xm1]/3.0;
               /*F[ce] = -F[xm1];*/

/* u nodes immediately to the left of the box right edge */

                i = xpbox + lbox - 1;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                xp1 = ny*(i+1) + j;
               F[ce] = -F[xp1]/3.0;
               /*F[ce] = -F[xp1];
               F[ce] = (4.0*F[xm1] - F[xm2])/3.0;

                if(u[xp1] < 0.0)
                {
                       F[ce] = -F[xp1]/3.0;
                }*/
        }

/* for v */
        for( i = xpbox+1; i <= xpbox+lbox-1; ++i )
        {
/* v nodes immediately above box bottom edge */
                j = ypbox;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                xp1 = ny*(i+1) + j;
                ym1 = ny*i + j-1;
               G[ce] = -G[ym1]/3.0;
               /*G[ce] = -G[ym1];
               G[ce] = (4.0*G[xm1] - G[xm2])/3.0;*/

/* v nodes immediately below box top edge */

                j = ypbox + hbox - 1;
                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;
                xp1 = ny*(i+1) + j;
                yp1 = ny*i + j+1;
               G[ce] = -G[yp1]/3.0;
               /*G[ce] = -G[yp1];
               G[ce] = (4.0*G[xm1] - G[xm2])/3.0;*/

        }


/* The boundary condtions inside the box are below. */

/* The velocities in the inside of the box based on quadratic extrapolation of velocities of
   nodes upwind of the corresponding sides. */

/* for u */
        for( i = xpbox+1; i <= xpbox+lbox-2; ++i )
        {
           for( j = ypbox+1; j <= ypbox+hbox-1; ++j )
           {

                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;

               F[ce] = (4.0*F[xm1] - F[xm2])/3.0;
                /*prs[ce] = (4.0*prs[xm1] - prs[xm2])/3.0;*/
           }
        }

/* for v */
        for( i = xpbox+1; i <= xpbox+lbox-1; ++i )
        {
           for( j = ypbox+1; j <= ypbox+hbox-2; ++j )
           {

                ce  = ny*i + j;
                xm1 = ny*(i-1) + j;
                xm2 = ny*(i-2) + j;

               G[ce] = (4.0*G[xm1] - G[xm2])/3.0;
                /*prs[ce] = (4.0*prs[xm1] - prs[xm2])/3.0;*/
           }
        }

        return 1;

}
#endif


