/* This program draws the pathlines for the program which uses the
   finite difference method to calculate an incompressible laminar
   CFD flow field.

                     Updated 4/25/06

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/flwconst.h"
#include "../compressible/flwstruct.h"
#include "cfdshape_struct.h"

extern int imax, jmax;
extern int stn;

typedef struct {
        int i, j;
} INDEX;

int cfdshape(SHAPE *, double , double );

extern double dx, dy, dz;
extern double dt, time_current;
extern int nmax, nmax2, nstep, cstep;

int pathlines(FILE *o3, double *xp, double *yp, double *u, double *v)
{

        INDEX index;
        int i, i2, j, k, dum, dum2;
        int ce, xp1, yp1, xp1yp1;
        int index_i, index_j;
        double perx, pery, xi, eta, fdum, fdum2;
        SHAPE sh;

        fdum2 = (double)nmax;
        fdum2 = 1.0;
#if 0
/* The variables below generate the pathlines 
*/
	for( k = 0; k < stn; ++k )
	{
/*	For pathlines */
	   index.i = (int)(xp[k]/dx);
	   index.j = (int)(yp[k]/dy);

	   /*printf(" %i %i  \n", index.i,index.j );*/
	   if( index.i < imax && index.i > 0 && index.j < jmax && index.j > 0 ) 
	   {
	    perx = xp[k]/dx - (double)(index.i);
	    pery = yp[k]/dy - (double)(index.j);
	    xp[k] = xp[k] + fdum2*dt*(perx*(u[jmax*(index.i)+index.j-1] -
		u[jmax*(index.i-1)+index.j-1]) + u[jmax*(index.i-1)+index.j-1]);
	    yp[k] = yp[k] + fdum2*dt*(perx*(v[jmax*(index.i-1)+index.j] -
		v[jmax*(index.i-1)+index.j-1]) + v[jmax*(index.i-1)+index.j-1]);
	    printf(  "8888 %9.5f %9.5f \n",xp[k],yp[k] );
	    fprintf(o3,"%9.5f %9.5f \n",xp[k],yp[k]);
	   }
        }
#endif

/* The variables below generate the pathlines 
*/

	for( k = 0; k < stn; ++k )
	{
	   index.i = (int)(xp[k]/dx);
	   index.j = (int)(yp[k]/dy);
           ce = jmax*(index.i-1) + index.j-1;
           xp1 = jmax*(index.i) + index.j-1;
           yp1 = jmax*(index.i-1) + index.j;
           xp1yp1 = jmax*(index.i) + index.j;

	   /*printf(" %i %i  \n", index.i,index.j );*/
	   if( index.i < imax && index.i > 0 && index.j < jmax && index.j > 0 ) 
	   {

/* I need to find the values of the current x and y position in terms of the
   isoparametric variables of the shape functions. */

	    perx = xp[k]/dx - (double)(index.i);
	    pery = yp[k]/dy - (double)(index.j);

            xi  = 2.0*perx - 1.0;
            eta = 2.0*pery - 1.0;

            cfdshape( &sh, xi, eta);

            fdum = sh.Nhat[0].dx0*u[ce] +
                   sh.Nhat[1].dx0*u[xp1] +
                   sh.Nhat[2].dx0*u[xp1yp1] +
                   sh.Nhat[3].dx0*u[yp1];

	    xp[k] = xp[k] + fdum2*dt*fdum;

            fdum = sh.Nhat[0].dx0*v[ce] +
                   sh.Nhat[1].dx0*v[xp1] +
                   sh.Nhat[2].dx0*v[xp1yp1] +
                   sh.Nhat[3].dx0*v[yp1];

	    yp[k] = yp[k] + fdum2*dt*fdum;

/*
            fdum = perx*(u[xp1] - u[ce]) + u[ce];
	    printf(  " %9.5f ", fdum );
	    xp[k] = xp[k] + fdum2*dt*fdum;
            fdum = pery*(v[xp1] - v[ce]) + v[ce];
	    yp[k] = yp[k] + fdum2*dt*fdum;
	    printf(  " %9.5f ", fdum );
*/

	    printf(  "8888 %9.5f %9.5f \n",xp[k],yp[k] );
	    fprintf(o3,"%9.5f %9.5f \n",xp[k],yp[k]);
	   }
        }

        return 1;
}


