/* This subroutine solves Poission's equation for the pressure using
   the SOR algorithm.  f represents the pressure.

                     Updated 11/29/06

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/flwconst.h"
#include "../compressible/flwstruct.h"

#define w               1.5
#define BIG             10.0e8

extern double dx, dy, dz;
extern double dt, err, Re;
extern int imax, jmax;

#if 0
int sor( double *u2, double *v2, double *prs)
{
	FILE *out;
	int i, j, kk, m, dum;
        int ce, xm1, xm2, xp1, ym1, ym2, yp1;
	double bets, fdum, pi, df;
	/*out = fopen( "o2", "w" );*/
	
	fdum=0.0;	
	bets=dx*dx/(dy*dy);	
	pi=acos(-1.0);	
	df=0.;

	for( kk = 0; kk < 1001; ++kk )
	{
	   for( j = 1; j < jmax-1; ++j )
	   {
	   	for( i = 1; i < imax-1; ++i )
		{
                   ce  = jmax*i + j;
                   xm1 = jmax*(i-1) + j;
                   xm2 = jmax*(i-2) + j;
                   xp1 = jmax*(i+1) + j;
                   ym1 = jmax*i + (j-1);
                   ym2 = jmax*i + (j-2);
                   yp1 = jmax*i + (j+1);

/* The expression below is based on Equation 17.13 page 339 in

   Fletcher, Clive A. J., Computational Techniques for Fluid Dynamics: Vol II, 2nd
        Ed., Springer-Verlag, Berlin, 1991.

   where the F is replaced with u2 and G is v2.

   Unlike the MAC formulation, where F and G have a complex relationship with u and v (See
   equation Equation 17.9 and 17.11 page 337-338), Equation 17.22 on page 343 shows how
   the uncorrected velocity u = F.  But because of my lack of success with this code,
   maybe I should try the expression given by Equation 17.9 and 17.11.
*/

		   df = 1./(2.+2*bets)*(prs[xp1]+bets*prs[yp1]+prs[xm1] +
			bets*prs[ym1]-2.*(1.+bets)*prs[ce] -
			dx*dx/dt*((u2[ce]-u2[xm1])/dx +
			(v2[ce]-v2[ym1])/dy));
		   	prs[ce]=prs[ce]+w*df;

/*		printf(  "eee %i %i  %9.5f %9.5f \n",i,j,prs[ce],df );*/
		   	if( fabs(df) > fdum ) 
		   	{
				fdum=fabs(df);
		   	}
		 }
                 prs[jmax*(imax-1)+j] = prs[jmax*(imax-2)+j];
	   }
	   if( kk > 999 ) printf(  "Maximum iterations reached %4d %4d %4d  %16.8e %16.8e \n", i, j, kk, prs[ce], df );
	   if( fdum < err || fdum > 3000 ) 
	   {
		m = kk;
	        printf(  "zzz  %4d %4d %4d  %16.8e %16.8e \n", i, j, kk, prs[ce], df );
		kk = 1000000;
	   }
	   fdum=0.0;

	}

        /*fclose(out);*/

        return 1;
}	
#endif

int sor( double *F, double *G, double *u2, double *v2, double *prs)
{
	FILE *out;
	int i, j, kk, m, dum, counter;
        int ce, xm1, xm2, xp1, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        double pxm1, pxp1, pym1, pyp1;
	double bets, fdum;
	double usq1, usq2, vsq1, vsq2, uv1, uv2, pi, df;
	/*out = fopen( "o2", "w" );*/
	
	fdum = 100000.0;	
	bets = dx*dx/(dy*dy);	
	pi = acos(-1.0);	
	df = 0.0;

        fdum = 1000.0;
        counter = 0;
	while( fdum > err && counter < 10000 ) 
        {

           fdum = 0.0;
	   for( j = 1; j < jmax-1; ++j )
	   {
	   	for( i = 1; i < imax-1; ++i )
		{
                   ce  = jmax*i + j;
                   xm1 = jmax*(i-1) + j;
                   xp1 = jmax*(i+1) + j;
                   ym1 = jmax*i + (j-1);
                   yp1 = jmax*i + (j+1);

/* The expression below is based on Equation 17.13 page 339 in

   Fletcher, Clive A. J., Computational Techniques for Fluid Dynamics: Vol II, 2nd
        Ed., Springer-Verlag, Berlin, 1991.

   where the F is replaced with u2 and G is v2. 

   Unlike the MAC formulation, where F and G have a complex relationship with u and v (See
   equation Equation 17.9 and 17.11 page 337-338), Equation 17.22 on page 343 shows how
   the uncorrected velocity u = F.  But because of my lack of success with this code,
   maybe I should try the expression given by Equation 17.9 and 17.11.
*/

#if 1
		   df = 1./(2.+2*bets)*(prs[xp1]+bets*prs[yp1]+prs[xm1] +
			bets*prs[ym1]-2.*(1.+bets)*prs[ce] -
			dx*dx/dt*((F[ce]-F[xm1])/dx + (G[ce]-G[ym1])/dy));
		   	prs[ce]=prs[ce]+w*df;

            /*printf(  "eeee %4d %16.8e %16.8e %16.8e %16.8e\n", ce, F[ce], F[xm1], G[ce], G[ym1]);*/
            /*printf(  "eeee %4d %4d %4d %4d %4d %4d\n", ce, xp1, yp1, yp2, xp1yp1, xm1ym1);
            printf(  "bbbb %16.8e %16.8e %16.8e %16.8e %16.8e %16.8e %16.8e\n",
		F1, F2, G1, G2, u2[ce], v2[ce], prs[ce]);*/

		   	/*if( prs[ce] > 10000.0 ) 
            printf(  "bbbb %4d %16.8e %16.8e %16.8e %16.8e %16.8e %16.8e %16.8e\n",
		counter, F1, F2, G1, G2, u2[ce], v2[ce], prs[ce]);*/
#endif

		   	if( fabs(df) > fdum ) 
		   	{
				fdum = fabs(df);
		   	}
		}
/* Below, I'm applying the BC at the right. */

                i = imax - 1;
                ce = jmax*i+j;
                xm1  = jmax*(i-2) + j;
                xm2  = jmax*(i-3) + j;
                ym1  = jmax*i + j-1;
                yp1  = jmax*i + j+1;
		prs[ce] = 2.0*prs[xm1] - prs[xm2];
		/*prs[ce] = 0.0;*/
		pxp1 = 2.0*prs[ce] - prs[xm1];
                u2[ce] = u2[ce] - dt/dx*(pxp1 - prs[ce]);
                v2[ce] = v2[ce] - dt/dy*(prs[yp1] - prs[ce]);

            /*printf(  "eeee %4d %16.8e %16.8e %16.8e %16.8e\n", ce, prs[dum], prs[xm1], prs[xm2], prs[ce]);*/

	   }
	   for( i = 1; i < imax; ++i )
	   {

/* For pressure only at top exit. */

                j = jmax - 1;
                ce = jmax*i + j;
                ym1  = jmax*i + j - 1;
                prs[ce] = prs[ym1];
                /*prs[ce] = 0.0;*/

/* For pressure only at the bottom exit. */

                j = 0;
                ce = jmax*i + j;
                yp1  = jmax*i + j + 1;
                prs[ce] = prs[yp1];
                /*prs[ce] = 0.0;*/
	   }

	   for( i = 1; i < imax-1; ++i )
	   {
/* For nodes at top exit. */
                j = jmax - 1;
                ce = jmax*i + j;
                xp1 = jmax*(i+1) + j;
                ym1  = jmax*i + j - 1;
                pyp1 = 2.0*prs[ce] - prs[ym1];
                u2[ce] = u2[ce] - dt/dx*(prs[xp1] - prs[ce]);
                v2[ce] = v2[ce] - dt/dy*(pyp1 - prs[ce]);
            /*printf(  "qqqqq %4d %4d %4d %4d %4d %4d\n", ce, xp1, xm1, ym1, ym2, xp1ym1);*/

/* For nodes at bottom exit. */

                j = 0;
                ce = jmax*i + j;
                xp1 = jmax*(i+1) + j;
                yp1  = jmax*i + j + 1;
                yp2  = jmax*i + j + 2;
                pyp1 = 2.0*prs[ce] - prs[ym1];
                u2[ce] = u2[ce] - dt/dx*(prs[xp1] - prs[ce]);
                v2[ce] = v2[ce] - dt/dy*(prs[yp1] - prs[ce]);
            /*printf(  "eeee %4d %4d %4d %4d %4d %16.8e\n", ce, xp1, yp1, yp2, xp1yp1, v2[ce]);*/

	   }

/* For node at top right. */

           i = imax - 1;
           j = jmax - 1;
           ce  = jmax*i + j;
           xm1 = jmax*(i-1) + j;
           ym1 = jmax*i + (j-1);

           pxp1 = 2.0*prs[ce] - prs[xm1];
           pyp1 = 2.0*prs[ce] - prs[ym1];

           u2[ce] = u2[ce] - dt/dx*(pxp1 - prs[ce]);
           v2[ce] = v2[ce] - dt/dy*(pyp1 - prs[ce]);

/* Recalculate F and G at the top and bottom */

	   for( i = 1; i < imax-1; ++i )
	   {
                j = jmax - 2;
                ce  = jmax*i + j;
                xm1 = jmax*(i-1) + j;
                xp1 = jmax*(i+1) + j;
                ym1 = jmax*i + (j-1);
                yp1 = jmax*i + (j+1);
                xm1ym1 = jmax*(i-1) + j-1;
                xm1yp1 = jmax*(i-1) + j+1;
                xp1ym1 = jmax*(i+1) + j-1;

                usq1 = (u2[xm1] + u2[ce])/2.0;
                usq1 = usq1*usq1;
                usq2 = (u2[ce] + u2[xp1])/2.0;
                usq2 = usq2*usq2;
                uv1 = (u2[ym1] + u2[ce])*(v2[xp1ym1] + v2[ym1])/4.0;
                uv2 = (u2[ce] + u2[yp1])*(v2[xp1] + v2[ce])/4.0;

                F[ce] = u2[ce] + dt*(u2[xp1] - 2*u2[ce] + u2[xm1])/dx/dx/Re +
                     dt*(u2[ym1] - 2*u2[ce] + u2[yp1])/dy/dy/Re -
                     dt*(usq2 - usq1)/dx - dt*(uv2 - uv1)/dy;

                vsq1 = (v2[ym1] + v2[ce])/2.0;
                vsq1 = vsq1*vsq1;
                vsq2 = (v2[ce] + v2[yp1])/2.0;
                vsq2 = vsq2*vsq2;
                uv1 = (u2[xm1] + u2[xm1yp1])*(v2[ce] + v2[xm1])/4.0;
                uv2 = (u2[ce] + u2[yp1])*(v2[xp1] + v2[ce])/4.0;

                G[ce] = v2[ce] + dt*(v2[xp1] - 2*v2[ce] + v2[xm1])/dx/dx/Re +
                     dt*(v2[yp1] - 2*v2[ce] + v2[ym1])/dy/dy/Re -
                     dt*(vsq2 - vsq1)/dy - dt*(uv2 - uv1)/dx;

                F[ce] = u2[ce];
                G[ce] = v2[ce];
	   }

/* Recalculate F and G at the right */

	   for( j = 1; j < jmax-1; ++j )
	   {
                i = imax - 2;
                ce  = jmax*i + j;
                xm1 = jmax*(i-1) + j;
                xp1 = jmax*(i+1) + j;
                ym1 = jmax*i + (j-1);
                yp1 = jmax*i + (j+1);
                xm1ym1 = jmax*(i-1) + j-1;
                xm1yp1 = jmax*(i-1) + j+1;
                xp1ym1 = jmax*(i+1) + j-1;

                usq1 = (u2[xm1] + u2[ce])/2.0;
                usq1 = usq1*usq1;
                usq2 = (u2[ce] + u2[xp1])/2.0;
                usq2 = usq2*usq2;
                uv1 = (u2[ym1] + u2[ce])*(v2[xp1ym1] + v2[ym1])/4.0;
                uv2 = (u2[ce] + u2[yp1])*(v2[xp1] + v2[ce])/4.0;

                F[ce] = u2[ce] + dt*(u2[xp1] - 2*u2[ce] + u2[xm1])/dx/dx/Re +
                     dt*(u2[ym1] - 2*u2[ce] + u2[yp1])/dy/dy/Re -
                     dt*(usq2 - usq1)/dx - dt*(uv2 - uv1)/dy;

                vsq1 = (v2[ym1] + v2[ce])/2.0;
                vsq1 = vsq1*vsq1;
                vsq2 = (v2[ce] + v2[yp1])/2.0;
                vsq2 = vsq2*vsq2;
                uv1 = (u2[xm1] + u2[xm1yp1])*(v2[ce] + v2[xm1])/4.0;
                uv2 = (u2[ce] + u2[yp1])*(v2[xp1] + v2[ce])/4.0;

                G[ce] = v2[ce] + dt*(v2[xp1] - 2*v2[ce] + v2[xm1])/dx/dx/Re +
                     dt*(v2[yp1] - 2*v2[ce] + v2[ym1])/dy/dy/Re -
                     dt*(vsq2 - vsq1)/dy - dt*(uv2 - uv1)/dx;

                F[ce] = u2[ce];
                G[ce] = v2[ce];
	   }

           ++counter;
           if( fdum < err ) printf(  "eeee %4d %14.6e %14.6e\n", counter, fdum, df);
           if( counter > 10000 - 1 ) printf(  "max iteration for pressure reached %4d %14.6e %14.6e\n", counter, fdum, df);
	}

        /*fclose(out);*/

        return 1;
}	

