/* This program draws the streaklines for the program which uses the
   finite difference method to calculate an incompressible laminar
   CFD flow field.

                     Updated 5/26/06

    SLFCFD source file
    Version:  1.0
    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/flwconst.h"
#include "../compressible/flwstruct.h"
#include "cfdshape_struct.h"

typedef struct {
        int i, j;
} INDEX;

int cfdshape(SHAPE *, double , double );

extern double dx, dy, dz;
extern double dt, time_current;
extern int nmax, nmax2, nstep, cstep;
extern int imax, jmax;
extern int stn;


int streaklines(int n2, double *xst, double *yst, double *xsto, double *ysto, double *u, double *v)
{

        INDEX index;
        int i, i2, j, k, dum, dum2;
        int ce, xm1, xm2, xp1, xp2, ym1, ym2, yp1, yp2, xm1ym1, xm1yp1, xp1ym1, xp1yp1;
        int index_i, index_j;
        int jxp, kyp;
        double perx, pery, xi, xi2, eta, eta2, fdum, fdum2;
        SHAPE sh;

        fdum2 = (double)nmax;
        fdum2 = 1.0;
#if 1
/* The variables below generate the streaklines
*/
	for( k = 0; k < stn; ++k )
	{

/*	For streaklines */
	   
	   xst[stn*n2+k] = xsto[k];
	   yst[stn*n2+k] = ysto[k];


	   for( i = 0; i < n2 + 1; ++i )
	   {
	     jxp = (int)(xst[stn*i+k]/dx);
	     kyp = (int)(yst[stn*i+k]/dy);

             index.i = (int)(xst[stn*i+k]/dx);
             index.j = (int)(yst[stn*i+k]/dy);
             ce = jmax*(index.i-1) + index.j-1;
             xm1 = jmax*((index.i)-2) + index.j-1;
             xp1 = jmax*(index.i) + index.j-1;
             ym1 = jmax*(index.i-1) + index.j-2;
             yp1 = jmax*(index.i-1) + index.j;
             xp1yp1 = jmax*(index.i) + index.j;
             xm1yp1 = jmax*((index.i)-2) + index.j+2;
             xp1ym1 = jmax*(index.i) + index.j-2;

             if( index.i < imax && index.i > 0 && index.j < jmax && index.j > 0 ) 
	     {
                 perx = xst[stn*i+k]/dx - (double)(index.i);
                 pery = yst[stn*i+k]/dy - (double)(index.j);

                 xi  = 2.0*perx - 1.0;
                 eta = 2.0*pery - 1.0;

/* Because I am using a staggared grid for u(i,j) and v(i,j) which do not fall on node
   points, xi and eta have to be determined based on this shift.

   First, -1 <= xi <= 1.0, and xi = 0.0 is where the node of u lies.  This means that I check whether
   at xi > 0.0. in which case I interpolate between u[ce] and u[xp1].  If not, then the interpolation
   is between u[xm1] and u[ce].  In addition, once it is determined which side of 0.0 xi lies on,
   I have to either do xi - 0.5 if xi < 0.0 or xi + 0.5 if xi > 0.0. 

   The same is done for v with respect to eta.
*/
                 if( xi > 0.0 )
                 {
                     xi2 = xi - 1.0;
                     cfdshape( &sh, xi2, eta);

                     fdum = sh.Nhat[0].dx0*u[ce] +
                       sh.Nhat[1].dx0*u[xp1] +
                       sh.Nhat[2].dx0*u[xp1yp1] +
                       sh.Nhat[3].dx0*u[yp1];
                 }
                 else
                 {
                     xi2 = xi + 1.0;
                     cfdshape( &sh, xi2, eta);

                     fdum = sh.Nhat[0].dx0*u[xm1] +
                       sh.Nhat[1].dx0*u[ce] +
                       sh.Nhat[2].dx0*u[yp1] +
                       sh.Nhat[3].dx0*u[xm1yp1];
                 }

                 xst[stn*i+k] = xst[stn*i+k] + fdum2*dt*fdum;

                 if( eta > 0.0 )
                 {
                     eta2 = eta - 1.0;
                     cfdshape( &sh, xi, eta2);

                     fdum = sh.Nhat[0].dx0*v[ce] +
                       sh.Nhat[1].dx0*v[xp1] +
                       sh.Nhat[2].dx0*v[xp1yp1] +
                       sh.Nhat[3].dx0*v[yp1];
                 }
                 else
                 {
                     eta2 = eta + 1.0;
                     cfdshape( &sh, xi, eta2);

                     fdum = sh.Nhat[0].dx0*v[ym1] +
                       sh.Nhat[1].dx0*v[xp1ym1] +
                       sh.Nhat[2].dx0*v[xp1] +
                       sh.Nhat[3].dx0*v[ce];
                 }

                 yst[stn*i+k] = yst[stn*i+k] + fdum2*dt*fdum;

	      /*xst[stn*i+k] = xst[stn*i+k] + fdum2*dt*(perx*(u[xp1] -
		u[ce]) + u[ce]);
	      yst[stn*i+k] = yst[stn*i+k] + fdum2*dt*(perx*(v[yp1] -
		v[ce]) + v[ce]);*/
	     } 
             else
             {
                 xst[stn*i+k] = -100000.0;
                 yst[stn*i+k] = -100000.0;
             }
	   }
        }
#endif

#if 0
/* The variables below generate the streaklines
*/
	for( k = 0; k < stn; ++k )
	{

/*	For streaklines */
	   
	   xst[stn*n2+k] = xsto[k];
	   yst[stn*n2+k] = ysto[k];
	   for( i = 0; i < n2; ++i )
	   {
	     jxp = (int)(xst[stn*i+k]/dx);
	     kyp = (int)(yst[stn*i+k]/dy);
	     if( jxp < imax && jxp > 0 && kyp < jmax && kyp > 0 ) 
	     {
	      perx = xst[stn*i+k]/dx - (double)(jxp);
	      pery = yst[stn*i+k]/dy - (double)(kyp);
	      xst[stn*i+k] = xst[stn*i+k] + fdum2*dt*(perx*(u[jmax*(jxp)+kyp-1] -
		u[jmax*(jxp-1)+kyp-1]) + u[jmax*(jxp-1)+kyp-1]);
	      yst[stn*i+k] = yst[stn*i+k] + fdum2*dt*(perx*(v[jmax*(jxp-1)+kyp] -
		v[jmax*(jxp-1)+kyp-1]) + v[jmax*(jxp-1)+kyp-1]);
	     } 
	   }
        }
#endif


	      /*printf(  "%d %d %9.5f %9.5f \n", j,k,xp[k],yp[k] );*/
	    /*printf(  "%d %d %9.5f %9.5f \n", j,k,xp[k],yp[k] );*/
	    /*fprintf(o3,"%9.5f %9.5f \n",xp[k],yp[k]);*/


/*here:       printf(  " had to stop %d %d  \n", j,k );*/

     return 1;

}


