/*
    This program contains the mesh display routine for the finite
    difference mesh.
  
                  Last Update 4/2/08

    SLFCFD source file
    Version:  1.1
    Copyright (C) 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

void cfd_qdmeshdraw(void);
void cfd_brmeshdraw(void);
void cfd_qdrender(void);
void cfd_brrender(void);
void velocity_vectors(BOUND , double *, XYZF *);

extern int flag_2D, flag_3D;
extern double *coord;
extern BOUND bc;

extern double left_right, up_down, in_out, left_right0,
	up_down0, in_out0, xAngle, yAngle, zAngle;
extern int bound_choice, gr_optimize_flag;
extern GLuint AxesList, BodyList, BoundList[10], VelocityList0, VelocityList,   /* Display lists */
	PathlineList0, PathlineList, StreaklineList0, StreaklineList;
extern XYZF *velocity_vec;
extern int Render_flag, Boundary_flag, AppliedDisp_flag, VelocityVec_flag,
	Pathline_flag, Streakline_flag, Body_flag, Axes_flag, Before_flag, After_flag; 
extern int CrossSection_flag;

void AxesNumbers2(void);

void AxesNumbers(void);

void AxesLabel(void);

void CrossSetionPlaneDraw(void);

void cfd_MeshDisplay(void)
{
	glClear (GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);

	glLoadIdentity ();  /*  clear the matrix    */

	glTranslatef (left_right, up_down, in_out);
	glRotatef (xAngle, 1, 0, 0);
	glRotatef (yAngle, 0, 1, 0);
	glRotatef (zAngle, 0, 0, 1);

	glPointSize(8);

	if(Axes_flag)
		glCallList(AxesList);
	if(Boundary_flag)
	{
		switch (bound_choice) {
			case 1:
			    glCallList(BoundList[1]);
			break;
			case 3:
			    glCallList(BoundList[3]);
			break;
			case 4:
			    glCallList(BoundList[4]);
			break;
			case 5:
			    glCallList(BoundList[5]);
			break;
			case 6:
			    glCallList(BoundList[6]);
			break;
			case 7:
			    glCallList(BoundList[7]);
			break;
			case 8:
			    glCallList(BoundList[8]);
			break;
			case 9:
			    glCallList(BoundList[9]);
			break;
			default:
			    glCallList(BoundList[0]);
			break;
		}
	}

	if(VelocityVec_flag)
	{
		if(Before_flag ) glCallList(VelocityList0);
		if(After_flag ) glCallList(VelocityList);
	}
	if(CrossSection_flag)
	{
		CrossSetionPlaneDraw();
	}
	glLineWidth (2.0);
	if(Render_flag)
		if(flag_3D && !gr_optimize_flag)
			cfd_brrender();
		else
			cfd_qdrender();
	else
	{
		if(flag_3D && !gr_optimize_flag)
			cfd_brmeshdraw();
		else
			cfd_qdmeshdraw();
	}
	if(Pathline_flag == 1)
		glCallList(PathlineList0);
	if(Pathline_flag == 2)
		glCallList(PathlineList);
	if(Streakline_flag == 1)
		glCallList(StreaklineList0);
	if(Streakline_flag == 2)
		glCallList(StreaklineList);
	if(Body_flag)
		glCallList(BodyList);
	if(Axes_flag)
	{
		AxesNumbers();
		/*AxesNumbers2();*/
	}
	glutSwapBuffers();
}

