/*
    This program contains the mesh key routine for the finite
    difference mesh.
  
			Last Update 3/3/08

    SLFCFD source file
    Version:  1.1
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

extern int nmat, numnp, numel, dof;
extern double dt, dx, dy, dz;
extern double *coord;
extern double *U;
extern int *connecter;
extern BOUND bc;
extern double *velocity;
extern GR_NODE *node;
extern XYZF *velocity_vec, *velocity_vec0;
extern I_GR_NODE *node_color;

extern GLfloat MeshColor[boxnumber+5][4];
extern double step_sizex, step_sizey, step_sizez;
extern int choice_stress;
extern NORM *norm;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
	cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;
extern int reget_VelocityList_flag;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom,
	ortho_left0, ortho_right0, ortho_top0, ortho_bottom0;
extern int ortho_redraw_flag;
extern double xAngle, yAngle, zAngle;
extern XYZF *velocity_vec, *velocity_vec0;
extern int mesh_width, mesh_height;
extern int input_flag, post_flag, color_choice,
    choice, matl_choice, node_choice, ele_choice, bound_choice;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z;
extern int input_color_flag, BackGround_color;
extern int Solid_flag, Perspective_flag, Render_flag,
    Boundary_flag, VelocityVec_flag,
    Material_flag, Node_flag, Element_flag, Axes_flag,
    Outline_flag, Transparent_flag, CrossSection_flag, Body_flag, Pathline_flag, Streakline_flag;
extern int Before_flag, After_flag, Both_flag;
extern double amplify_factor, amplify_step, amplify_step0;

extern GLuint AxesList, DispList, VelocityList0, VelocityList;
extern int chooser, chooser_flag;

extern GR_NODE node_div[boxnumber+1];

void velocity_vectors(int , BOUND , double *, XYZF *);

int ConnectSurfMesher2 (int *, int *, GR_NODE *);

void ScreenShot( int , int );

void MeshInit(void);

int cfd_set( BOUND , XYZF *, GR_NODE *, I_GR_NODE *, GR_NODE * );

void fdReGetparameter( void);

int fdGetNewMesh(void);

int fdnormal_vectors (int *, double *, NORM * );

void MeshReshape(int , int );

void cfd_MeshKeys( unsigned char key, int x, int y )
{

	int i, check, pdum[] = { 1, 2};
	double fpointx, fpointy, fpointz;
/* Define Hotkeys */

/* 'i' zooms in on the mesh, 'o' zooms out */

	switch (key) {
	    case 'i':
		if ( Perspective_flag )
		{
			in_out += step_sizez;
		}
		else
		{
			ortho_left *= 0.90;
			ortho_right *= 0.90;
			ortho_bottom *= 0.90;
			ortho_top *= 0.90;
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		break;
	    case 'o':
		if ( Perspective_flag )
		{
			in_out -= step_sizez;
		}
		else
		{
			ortho_left *= 1.10;
			ortho_right *= 1.10;
			ortho_bottom *= 1.10;
			ortho_top *= 1.10;
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		break;

/* These keys control the selection of viewing stresses and strains 
   and displacements. */

	    case '1':
		Solid_flag = 1;
		color_choice = 1;
		break;
	    case '2':
		Solid_flag = 1;
		color_choice = 2;
		break;
	    case '3':
		Solid_flag = 1;
		color_choice = 3;
		break;
	    case '4':
		Solid_flag = 1;
		color_choice = 4;
		break;
	    case '5':
		Solid_flag = 1;
		color_choice = 5;
		break;
	    case '6':
		Solid_flag = 1;
		color_choice = 6;
		break;
	    case '7':
		Solid_flag = 1;
		color_choice = 7;
		break;
	    case '8':
		Solid_flag = 1;
		color_choice = 8;
		break;
	    case '9':
		Solid_flag = 1;
		color_choice = 9;
		break;

/* 'n' selects the node to be viewed.  */

	    case 'n':
		color_choice = 31;
		input_color_flag = 0;
		VelocityVec_flag = 0;
		Boundary_flag = 0;
		Element_flag = 0;
		Material_flag = 0;
		Node_flag = 1;

		printf("\n What is the desired node number?\n");
		scanf("%d", &node_choice);
		if ( node_choice > numnp - 1 )
		{
			node_choice = 0;
		}
		break;

/* 'e' selects the element to be viewed.  */

	    case 'e':
		color_choice = 32;
		input_color_flag = 0;
		VelocityVec_flag = 0;
		Boundary_flag = 0;
		Element_flag = 1;
		Material_flag = 0;
		Node_flag = 0;
		
		printf("\n What is the desired element number?\n");
		scanf("%d", &ele_choice);
		if ( ele_choice > numel - 1 )
		{
			ele_choice = 0;
		}
		Solid_flag = 1;
		break;

/* 'm' selects the material to be viewed.  */

	    case 'm':
		color_choice = 30;
		input_color_flag = 0;
		VelocityVec_flag = 0;
		Boundary_flag = 0;
		Element_flag = 0;
		Material_flag = 1;
		Node_flag = 0;

		printf("\n What is the desired material number?\n");
		scanf("%d", &matl_choice);
		if ( matl_choice > nmat - 1 )
		{
			matl_choice = 0;
		}
		Solid_flag = 1;
		break;

/* 'd' selects the boundary condition to be viewed. */

	    case 'd':
		color_choice = 33;
		input_color_flag = 0;
		VelocityVec_flag = 0;
		Boundary_flag = 1;
		Element_flag = 0;
		Material_flag = 0;
		Node_flag = 0;

		printf("\n What is the desired boundary number?\n");
		scanf("%d", &bound_choice);
		if ( bound_choice > numbc - 1 )
		{
			bound_choice = 0;
		}
		break; 

#if 0
/* '>' and '<' amplify and shrink the displacements on the deformed object */

	    case '.':
		if( post_flag )
		{
			After_flag = 1;
			amplify_step = amplify_step0;
			if( amplify_factor < 1.0 - SMALL2 )
				amplify_step = .1;
			amplify_factor += amplify_step;
/* Update Coordinates */
			check = fdnormal_vectors(connecter, coord, norm );
			if(!check) printf( " Problems with normal_vectors \n");
		}
		break;
	    case ',':
		if( post_flag )
		{
			After_flag = 1;
			/*printf("amplify factor %f %f\n", amplify_factor,amplify_step);*/
/* Update Coordinates */
			check = fdnormal_vectors(connecter, coord, norm );
			if(!check) printf( " Problems with normal_vectors \n");
		}
		break;
#endif

/* The '<' and '>' keys move the Cross Section Plane on the Z
   Axes in the mesh window. */

	    case '<':
		cross_sec_in_out -= step_sizez;
		if( cross_sec_in_out < AxisMin_z )
			cross_sec_in_out = AxisMin_z;
		check = ConnectSurfMesher2 (connecter, pdum, node);
		if(!check) printf( " Problems with ConnectSurfMesher2 \n");
		check = fdnormal_vectors(connecter, coord, norm );
		if(!check) printf( " Problems with normal_vectors \n");
		break;
	    case '>':
		cross_sec_in_out += step_sizez;
		if( cross_sec_in_out > AxisMax_z )
			cross_sec_in_out = AxisMax_z;
		check = ConnectSurfMesher2 (connecter, pdum, node);
		if(!check) printf( " Problems with ConnectSurfMesher2 \n");
		check = fdnormal_vectors(connecter, coord, norm );
		if(!check) printf( " Problems with normal_vectors \n");
		break;

/* 'a' and 'b' turns on and off the deformed and undeformed mesh */

	    case 'a':
		After_flag = 1 - After_flag;
		if(After_flag) Before_flag = 0;
		if(After_flag)
		    if(Pathline_flag == 1) Pathline_flag = 2;
		    if(Streakline_flag == 1) Streakline_flag = 2;
		break; 
	    case 'b':
		Before_flag = 1 - Before_flag;
		if(Before_flag) After_flag = 0;
		    if(Pathline_flag == 2) Pathline_flag = 1;
		    if(Streakline_flag == 2) Streakline_flag = 1;
		break; 
	    case 'B':
		Body_flag = 1 - Body_flag;
		break; 

/* Reset the rotation */

	    case 'c':
		xAngle = 0.0;
		yAngle = 0.0;
		zAngle = 0.0;
		break; 

/* Reset the translation */

	    case 'v':
		left_right = left_right0;
		up_down = up_down0;
		in_out = in_out0;
		if ( !Perspective_flag )
		{
			left_right = 0.0;
			up_down = 0.0;
			ortho_right = ortho_right0;
			ortho_left = ortho_left0;
			ortho_top = ortho_top0;
			ortho_bottom = ortho_bottom0;
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		cross_sec_left_right = cross_sec_left_right0;
		cross_sec_up_down = cross_sec_up_down0;
		cross_sec_in_out = cross_sec_in_out0;
		break; 

	    case 'f':
		VelocityVec_flag = 1 - VelocityVec_flag;
		if(VelocityVec_flag) Transparent_flag = 1;
		else Transparent_flag = 0;
		break;
	    case 'g':
		check = fdGetNewMesh();
		if(!check) printf( " Problems with fdGetNewMesh\n");
		break;
	    case 'h':
		fdReGetparameter();
		check = cfd_set( bc, velocity_vec0, node, node_color, node_div );
		if(!check) printf( " Problems with cfd_set \n");
		if ( !Perspective_flag )
		{
			MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
				glutGet(GLUT_WINDOW_HEIGHT));
		}
		if(reget_VelocityList_flag)
		{
			if(input_flag)
			{
/* create display list for velocity grapics vectors on input mesh*/
				glDeleteLists(VelocityList0,1);
				VelocityList0 = glGenLists(1);
				velocity_vectors(VelocityList0, bc, coord, velocity_vec0);
			}
			if(post_flag)
			{
/* create display list for velocity grapics vectors on post mesh*/
				glDeleteLists(VelocityList,1);
				VelocityList = glGenLists(1);
				velocity_vectors(VelocityList, bc, coord, velocity_vec);
			}
		}
		break;
	    case 'j':
		Outline_flag = 1 - Outline_flag;
		break;
	    case 'p':
		Perspective_flag = 1 - Perspective_flag;
		MeshReshape( glutGet(GLUT_WINDOW_WIDTH),
			glutGet(GLUT_WINDOW_HEIGHT));
		left_right = left_right0;
		up_down = up_down0;
		/*in_out = in_out0;*/
		if ( !Perspective_flag )
		{
			left_right = 0.0;
			up_down = 0.0;
		}
		break;
	    case 'P':
		Streakline_flag = 0;
		Pathline_flag += 1;
		if ( Pathline_flag > 2 )
		{
			Pathline_flag = 0;
		}
		break;
	    case 'q':
		exit(1);
		break;
	    case 'r':
		Render_flag = 1 - Render_flag;
		VelocityVec_flag = 0;
		Boundary_flag = 0;
		MeshInit();
		break;
	    case 's':
		CrossSection_flag = 1 - CrossSection_flag;
		break;
	    case 'S':
		Pathline_flag = 0;
		Streakline_flag += 1;
		if ( Streakline_flag > 2 )
		{
			Streakline_flag = 0;
		}
		break;
	    case 't':
		Transparent_flag = 1 - Transparent_flag;
		break;
	    case 'w':
		Solid_flag = 1 - Solid_flag;
		if(!Solid_flag) Outline_flag = 1;
		break;
	    case 'W':
		BackGround_color = 1 - BackGround_color;
		MeshBackGround();
		break;
	    case 'x':
		Axes_flag = 1 - Axes_flag;
		break;
	    case 'y':
		ScreenShot( mesh_width, mesh_height );
		break;
	    case 27:
		exit(0);
		break;
	}

	input_color_flag = 0;
	if( VelocityVec_flag )
	{
		if( color_choice > 29) input_color_flag = 1;
		Boundary_flag = 0;
		Element_flag = 0;
		Material_flag = 0;
		Node_flag = 0;
	}

	if(!post_flag) After_flag = 0;
	if(!input_flag) Before_flag = 0;

	glutPostRedisplay();
}


void MeshKey_Special2(int key, int x, int y)
{
/*
    This program contains the mesh special keys routine for every
    FEM GUI program.
 
	                Last Update 2/16/06
*/

	int check, pdum[] = { 1, 2};

  if(CrossSection_flag)
  {
/* The arrow keys move the Cross Section Plane on the X and Y
   Axes in the mesh window. */

	switch (key) {
	    case GLUT_KEY_UP:
		cross_sec_up_down += step_sizey;
		if( cross_sec_up_down > AxisMax_y )
			cross_sec_up_down = AxisMax_y;
		break;
	    case GLUT_KEY_DOWN:
		cross_sec_up_down -= step_sizey;
		if( cross_sec_up_down < AxisMin_y )
			cross_sec_up_down = AxisMin_y;
		break;
	    case GLUT_KEY_LEFT:
		cross_sec_left_right -= step_sizex;
		if( cross_sec_left_right < AxisMin_x )
			cross_sec_left_right = AxisMin_x;
		break;
	    case GLUT_KEY_RIGHT:
		cross_sec_left_right += step_sizex;
		if( cross_sec_left_right > AxisMax_x )
			cross_sec_left_right = AxisMax_x;
		break;
	}
	check = ConnectSurfMesher2 (connecter, pdum, node);
	if(!check) printf( " Problems with ConnectSurfMesher2 \n");
	check = fdnormal_vectors(connecter, coord, norm );
	if(!check) printf( " Problems with normal_vectors \n");
  }
  else
  {

/* The arrow keys move the mesh vertically and horizontally
   in the window. */

	switch (key) {
	    case GLUT_KEY_UP:
		up_down += step_sizey;
		break;
	    case GLUT_KEY_DOWN:
		up_down -= step_sizey;
		break;
	    case GLUT_KEY_LEFT:
		left_right -= step_sizex;
		break;
	    case GLUT_KEY_RIGHT:
		left_right += step_sizex;
		break;
	}
  }
  glutPostRedisplay();
}


