/*
    This program sets viewing and analysis values based on the parameters
    determined in fdparameters for the CFD GUI for finite difference meshes.
  
			Last Update 3/29/08

    SLFCFD source file
    Version:  1.1
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */
#include <assert.h>
#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"
#include "fdstrcgr.h"
#include "../common_gr/control.h"

#define init_far0      -2.0

extern int nmat, numnp, numel, dof;
extern double step_sizex, step_sizey, step_sizez;
extern double left, right, top, bottom, near, far, fscale, line_rescale;
extern int control_height, control_width, mesh_height, mesh_width;
extern double ortho_left, ortho_right, ortho_top, ortho_bottom;
extern double left_right, up_down, in_out, left_right0, up_down0, in_out0;
extern double AxisMax_x, AxisMax_y, AxisMax_z,
	AxisMin_x, AxisMin_y, AxisMin_z,
	IAxisMin_x, IAxisMin_y, IAxisMin_z;
extern double AxisLength_x, AxisLength_y, AxisLength_z,
	AxisLength_max;
extern double AxisPoint_step;
extern double amplify_step0;

extern double init_right, init_left, init_top,
	init_bottom, init_near, init_far, true_far, dim_max;
extern GR_NODE del_node, max_node, min_node;

int cfd_set( BOUND bc, XYZF *velocity_vec0, GR_NODE *node, I_GR_NODE *node_color, GR_NODE *node_div )
{
	int i, j, node_num, check;
	double velocity_vec_length, fdum;

/* Determine nodal quantities of density, temperature, pressure, velocity x-y-z, enthalpy,
  energy, and mach number(or velocity magnitude) color scheme */

	del_node.dns =
		(max_node.dns - min_node.dns + SMALL)/(double)(boxnumber);
	del_node.temp =
		(max_node.temp - min_node.temp + SMALL)/(double)(boxnumber);
	del_node.prs =
		(max_node.prs - min_node.prs + SMALL)/(double)(boxnumber);
	del_node.vx =
		(max_node.vx - min_node.vx + SMALL)/(double)(boxnumber);
	del_node.vy =
		(max_node.vy - min_node.vy + SMALL)/(double)(boxnumber);
	del_node.vz =
		(max_node.vz - min_node.vz + SMALL)/(double)(boxnumber);
	del_node.enthal =
		(max_node.enthal - min_node.enthal + SMALL)/(double)(boxnumber);
	del_node.ene =
		(max_node.ene - min_node.ene + SMALL)/(double)(boxnumber);
	del_node.mach =
		(max_node.mach - min_node.mach + SMALL)/(double)(boxnumber);
	del_node.vmag =
		(max_node.vmag - min_node.vmag + SMALL)/(double)(boxnumber);
	node_div[0].dns = min_node.dns;
	node_div[0].temp = min_node.temp;
	node_div[0].prs = min_node.prs;
	node_div[0].vx = min_node.vx;
	node_div[0].vy = min_node.vy;
	node_div[0].vz = min_node.vz;
	node_div[0].enthal = min_node.enthal;
	node_div[0].ene = min_node.ene;
	node_div[0].mach = min_node.mach;
	node_div[0].vmag = min_node.vmag;
	/*printf(" max min strain dns %10.5e %10.5e \n", max_strain.dns, min_strain.dns);
	printf(" strain div dns %10.5e \n", strain_div[0].dns);*/
	for( i = 0; i < boxnumber; ++i )
	{
		node_div[i+1].dns    = node_div[i].dns + del_node.dns;
		node_div[i+1].temp   = node_div[i].temp + del_node.temp;
		node_div[i+1].prs    = node_div[i].prs + del_node.prs;
		node_div[i+1].vx     = node_div[i].vx + del_node.vx;
		node_div[i+1].vy     = node_div[i].vy + del_node.vy;
		node_div[i+1].vz     = node_div[i].vz + del_node.vz;
		node_div[i+1].enthal = node_div[i].enthal + del_node.enthal;
		node_div[i+1].ene    = node_div[i].ene + del_node.ene;
		node_div[i+1].mach   = node_div[i].mach + del_node.mach;
		node_div[i+1].vmag   = node_div[i].vmag + del_node.vmag;
		/*printf(" node div dns %10.5e \n", node_div[i+1].dns);*/
	}


/* Assign Colors for quantities of density, temperature, pressure, velocity x-y-z, enthalpy,
  energy, and mach number(or velocity magnitude) color scheme */


	for( i = 0; i < numnp; ++i )
	{

/* Assign colors for density */
	       node_color[i].dns = 0;
	       if(  node[i].dns > node_div[1].dns )
	       {
		  node_color[i].dns = 1;
		  if(  node[i].dns > node_div[2].dns )
		  {
		     node_color[i].dns = 2;
		     if(  node[i].dns > node_div[3].dns )
		     {
			node_color[i].dns = 3;
			if(  node[i].dns > node_div[4].dns )
			{
			   node_color[i].dns = 4;
			   if(  node[i].dns > node_div[5].dns )
			   {
			      node_color[i].dns = 5;
			      if(  node[i].dns > node_div[6].dns )
			      {
				 node_color[i].dns = 6;
				 if(  node[i].dns > node_div[7].dns )
				 {
				    node_color[i].dns = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" dns %d %d %10.5e %d \n", i, j,
			node[i].dns, node_color[i].dns);*/
/* Assign colors for temperature */
	       node_color[i].temp = 0;
	       if(  node[i].temp > node_div[1].temp )
	       {
		  node_color[i].temp = 1;
		  if(  node[i].temp > node_div[2].temp )
		  {
		     node_color[i].temp = 2;
		     if(  node[i].temp > node_div[3].temp )
		     {
			node_color[i].temp = 3;
			if(  node[i].temp > node_div[4].temp )
			{
			   node_color[i].temp = 4;
			   if(  node[i].temp > node_div[5].temp )
			   {
			      node_color[i].temp = 5;
			      if(  node[i].temp > node_div[6].temp )
			      {
				 node_color[i].temp = 6;
				 if(  node[i].temp > node_div[7].temp )
				 {
				    node_color[i].temp = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" temp %d %d %10.5e %d \n", i, j,
			node[i].temp, node_color[i].temp);*/
/* Assign colors for pressure */
	       node_color[i].prs = 0;
	       if(  node[i].prs > node_div[1].prs )
	       {
		  node_color[i].prs = 1;
		  if(  node[i].prs > node_div[2].prs )
		  {
		     node_color[i].prs = 2;
		     if(  node[i].prs > node_div[3].prs )
		     {
			node_color[i].prs = 3;
			if(  node[i].prs > node_div[4].prs )
			{
			   node_color[i].prs = 4;
			   if(  node[i].prs > node_div[5].prs )
			   {
			      node_color[i].prs = 5;
			      if(  node[i].prs > node_div[6].prs )
			      {
				 node_color[i].prs = 6;
				 if(  node[i].prs > node_div[7].prs )
				 {
				    node_color[i].prs = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" prs %d %d %10.5e %d \n", i, j,
			node[i].prs, node_color[i].prs);*/
/* Assign colors for velocity x */
	       node_color[i].vx = 0;
	       if(  node[i].vx > node_div[1].vx )
	       {
		  node_color[i].vx = 1;
		  if(  node[i].vx > node_div[2].vx )
		  {
		     node_color[i].vx = 2;
		     if(  node[i].vx > node_div[3].vx )
		     {
			node_color[i].vx = 3;
			if(  node[i].vx > node_div[4].vx )
			{
			   node_color[i].vx = 4;
			   if(  node[i].vx > node_div[5].vx )
			   {
			      node_color[i].vx = 5;
			      if(  node[i].vx > node_div[6].vx )
			      {
				 node_color[i].vx = 6;
				 if(  node[i].vx > node_div[7].vx )
				 {
				    node_color[i].vx = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" vx %d %d %10.5e %d \n", i, j,
			node[i].vx, node_color[i].vx);*/
/* Assign colors for velocity y */
	       node_color[i].vy = 0;
	       if(  node[i].vy > node_div[1].vy )
	       {
		  node_color[i].vy = 1;
		  if(  node[i].vy > node_div[2].vy )
		  {
		     node_color[i].vy = 2;
		     if(  node[i].vy > node_div[3].vy )
		     {
			node_color[i].vy = 3;
			if(  node[i].vy > node_div[4].vy )
			{
			   node_color[i].vy = 4;
			   if(  node[i].vy > node_div[5].vy )
			   {
			      node_color[i].vy = 5;
			      if(  node[i].vy > node_div[6].vy )
			      {
				 node_color[i].vy = 6;
				 if(  node[i].vy > node_div[7].vy )
				 {
				    node_color[i].vy = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" vy %d %d %10.5e %d \n", i, j,
			node[i].vy, node_color[i].vy);*/
/* Assign colors for velocity z */
	       node_color[i].vz = 0;
	       if(  node[i].vz > node_div[1].vz )
	       {
		  node_color[i].vz = 1;
		  if(  node[i].vz > node_div[2].vz )
		  {
		     node_color[i].vz = 2;
		     if(  node[i].vz > node_div[3].vz )
		     {
			node_color[i].vz = 3;
			if(  node[i].vz > node_div[4].vz )
			{
			   node_color[i].vz = 4;
			   if(  node[i].vz > node_div[5].vz )
			   {
			      node_color[i].vz = 5;
			      if(  node[i].vz > node_div[6].vz )
			      {
				 node_color[i].vz = 6;
				 if(  node[i].vz > node_div[7].vz )
				 {
				    node_color[i].vz = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" vz %d %d %10.5e %d \n", i, j,
			node[i].vz, node_color[i].vz);*/
/* Assign colors for enthalpy */
	       node_color[i].enthal = 0;
	       if(  node[i].enthal > node_div[1].enthal )
	       {
		  node_color[i].enthal = 1;
		  if(  node[i].enthal > node_div[2].enthal )
		  {
		     node_color[i].enthal = 2;
		     if(  node[i].enthal > node_div[3].enthal )
		     {
			node_color[i].enthal = 3;
			if(  node[i].enthal > node_div[4].enthal )
			{
			   node_color[i].enthal = 4;
			   if(  node[i].enthal > node_div[5].enthal )
			   {
			      node_color[i].enthal = 5;
			      if(  node[i].enthal > node_div[6].enthal )
			      {
				 node_color[i].enthal = 6;
				 if(  node[i].enthal > node_div[7].enthal )
				 {
				    node_color[i].enthal = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" enthal %d %d %10.5e %d \n", i, j,
			node[i].enthal, node_color[i].enthal);*/
/* Assign colors for energy */
	       node_color[i].ene = 0;
	       if(  node[i].ene > node_div[1].ene )
	       {
		  node_color[i].ene = 1;
		  if(  node[i].ene > node_div[2].ene )
		  {
		     node_color[i].ene = 2;
		     if(  node[i].ene > node_div[3].ene )
		     {
			node_color[i].ene = 3;
			if(  node[i].ene > node_div[4].ene )
			{
			   node_color[i].ene = 4;
			   if(  node[i].ene > node_div[5].ene )
			   {
			      node_color[i].ene = 5;
			      if(  node[i].ene > node_div[6].ene )
			      {
				 node_color[i].ene = 6;
				 if(  node[i].ene > node_div[7].ene )
				 {
				    node_color[i].ene = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" ene %d %d %10.5e %d \n", i, j,
			node[i].ene, node_color[i].ene);*/
/* Assign colors for mach number */
	       node_color[i].mach = 0;
	       if(  node[i].mach > node_div[1].mach )
	       {
		  node_color[i].mach = 1;
		  if(  node[i].mach > node_div[2].mach )
		  {
		     node_color[i].mach = 2;
		     if(  node[i].mach > node_div[3].mach )
		     {
			node_color[i].mach = 3;
			if(  node[i].mach > node_div[4].mach )
			{
			   node_color[i].mach = 4;
			   if(  node[i].mach > node_div[5].mach )
			   {
			      node_color[i].mach = 5;
			      if(  node[i].mach > node_div[6].mach )
			      {
				 node_color[i].mach = 6;
				 if(  node[i].mach > node_div[7].mach )
				 {
				    node_color[i].mach = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" mach %d %d %10.5e %d \n", i, j,
			node[i].mach, node_color[i].mach);*/
/* Assign colors for velocity magnitude */
	       node_color[i].vmag = 0;
	       if(  node[i].vmag > node_div[1].vmag )
	       {
		  node_color[i].vmag = 1;
		  if(  node[i].vmag > node_div[2].vmag )
		  {
		     node_color[i].vmag = 2;
		     if(  node[i].vmag > node_div[3].vmag )
		     {
			node_color[i].vmag = 3;
			if(  node[i].vmag > node_div[4].vmag )
			{
			   node_color[i].vmag = 4;
			   if(  node[i].vmag > node_div[5].vmag )
			   {
			      node_color[i].vmag = 5;
			      if(  node[i].vmag > node_div[6].vmag )
			      {
				 node_color[i].vmag = 6;
				 if(  node[i].vmag > node_div[7].vmag )
				 {
				    node_color[i].vmag = 7;
				 }
			      }
			   }
			}
		     }
		  }
	       }
	       /*printf(" vmag %d %d %10.5e %d \n", i, j,
			node[i].vmag, node_color[i].vmag);*/
	}

/* Calculate the instantaneous velocity vectors for display */

/* All the velocity vectors will be scaled based on the largest velocity vector. */

	velocity_vec_length = SMALL;
	for( node_num = 0; node_num < numnp; ++node_num )
	{
	    if( node[node_num].bc_type < 100)
	    {
		fdum = node[node_num].vx*node[node_num].vx +
		       node[node_num].vy*node[node_num].vy +
		       node[node_num].vz*node[node_num].vz + SMALL;
		if (velocity_vec_length < fdum) velocity_vec_length = fdum;
	    }
	}

	velocity_vec_length = sqrt(velocity_vec_length)*line_rescale;

	for( node_num = 0; node_num < numnp; ++node_num )
	{
	    velocity_vec0[node_num].x = 0.0;
	    velocity_vec0[node_num].y = 0.0;
	    velocity_vec0[node_num].z = 0.0;
	    if( node[node_num].bc_type < 100)
	    {
		fdum = node[node_num].vx*node[node_num].vx +
		       node[node_num].vy*node[node_num].vy +
		       node[node_num].vz*node[node_num].vz;
		if( fdum > 2.0*SMALL)
		{
		    velocity_vec0[node_num].x = node[node_num].vx/velocity_vec_length;
		    velocity_vec0[node_num].y = node[node_num].vy/velocity_vec_length;
		    velocity_vec0[node_num].z = node[node_num].vz/velocity_vec_length;
		}
		/*printf(" force %d %12.5e %12.5e %12.5e %12.5e %12.5e %12.5e %12.5e\n", i, 
		   node[node_num].vx*node[node_num].vx, node[node_num].vx*node[node_num].vy,
		   node[node_num].vx*node[node_num].vz, velocity_vec0[node_num].x, velocity_vec0[node_num].y,
		   velocity_vec0[node_num].z, velocity_vec_length);*/
	    }
	}

	return 1;    /* ANSI C requires main to return int. */
}
