/*
   This program draws the displacement and velocity vectors
   for 3-D elements that have no rotational degrees of freedom. 
 
                  Last Update 3/3/06

    SLFCFD source file
    Version:  1.1
    Copyright (C) 2006, 2007, 2008  San Le 

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.
 
 */

#if WINDOWS
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

extern int numnp, nx, ny, nz;
extern int num_bodies;
extern double line_rescale;
extern GLfloat yellow[4], orange[4], orangeRed[4], red[4], green[4], 
	violetRed[4], magenta[4], purple[4], blue[4], white[4], grey[4],
	black[4], brown[4], aqua[4], blueGreen[4], greenYellow[4];
extern double AxisLength_max;
extern int nx_inc, ny_inc, nz_inc;
extern GLfloat line_width, point_size;
extern int bound_choice;


void bound_vectors(GLuint *BoundList, BOUND bc, double *coord)
{

/* This subrountine draws boundary conditions by representing them as vectors
   in each particular direction.  Each boundary condition is set in the x, y,
   and z directions, so when a boundary is selected, a display is made for
   this boundary in x, y, and z.

                  Last Update 3/8/06
*/

  int i,j, dum;
  double fdum, fpointx, fpointy, fpointz;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
  fdum = 4.0*AxisLength_max/line_rescale;

/* This draws the nodes which have no bondary condition.  This is for
   the boundary condition type 0.
*/
  glNewList(BoundList[0], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);

	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(orange);
	   for( i = 0; i < bc.num_none[0]; ++i)
	   {
		fpointx = *(coord+nsd*bc.none[i]);
		fpointy = *(coord+nsd*bc.none[i] + 1);
		fpointz = *(coord+nsd*bc.none[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();

/* This draws all the symmetric bondary conditions.  This includes
   the types 1-6.
*/
  glNewList(BoundList[1], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_symm[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.xm[i]);
		fpointy = *(coord+nsd*bc.symm.xm[i] + 1);
		fpointz = *(coord+nsd*bc.symm.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_symm[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.ym[i]);
		fpointy = *(coord+nsd*bc.symm.ym[i] + 1);
		fpointz = *(coord+nsd*bc.symm.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_symm[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.zm[i]);
		fpointy = *(coord+nsd*bc.symm.zm[i] + 1);
		fpointz = *(coord+nsd*bc.symm.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_symm[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.xm[i]);
		fpointy = *(coord+nsd*bc.symm.xm[i] + 1);
		fpointz = *(coord+nsd*bc.symm.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_symm[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.ym[i]);
		fpointy = *(coord+nsd*bc.symm.ym[i] + 1);
		fpointz = *(coord+nsd*bc.symm.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_symm[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.zm[i]);
		fpointy = *(coord+nsd*bc.symm.zm[i] + 1);
		fpointz = *(coord+nsd*bc.symm.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_symm[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.xp[i]);
		fpointy = *(coord+nsd*bc.symm.xp[i] + 1);
		fpointz = *(coord+nsd*bc.symm.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_symm[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.yp[i]);
		fpointy = *(coord+nsd*bc.symm.yp[i] + 1);
		fpointz = *(coord+nsd*bc.symm.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_symm[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.zp[i]);
		fpointy = *(coord+nsd*bc.symm.zp[i] + 1);
		fpointz = *(coord+nsd*bc.symm.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_symm[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.xp[i]);
		fpointy = *(coord+nsd*bc.symm.xp[i] + 1);
		fpointz = *(coord+nsd*bc.symm.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_symm[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.yp[i]);
		fpointy = *(coord+nsd*bc.symm.yp[i] + 1);
		fpointz = *(coord+nsd*bc.symm.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_symm[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.symm.zp[i]);
		fpointy = *(coord+nsd*bc.symm.zp[i] + 1);
		fpointz = *(coord+nsd*bc.symm.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();

/* This draws all the far field bondary conditions.  This includes
   the types 7-12.
*/
  glNewList(BoundList[3], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_far[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.xm[i]);
		fpointy = *(coord+nsd*bc.far.xm[i] + 1);
		fpointz = *(coord+nsd*bc.far.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_far[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.ym[i]);
		fpointy = *(coord+nsd*bc.far.ym[i] + 1);
		fpointz = *(coord+nsd*bc.far.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_far[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.zm[i]);
		fpointy = *(coord+nsd*bc.far.zm[i] + 1);
		fpointz = *(coord+nsd*bc.far.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_far[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.xm[i]);
		fpointy = *(coord+nsd*bc.far.xm[i] + 1);
		fpointz = *(coord+nsd*bc.far.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_far[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.ym[i]);
		fpointy = *(coord+nsd*bc.far.ym[i] + 1);
		fpointz = *(coord+nsd*bc.far.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_far[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.zm[i]);
		fpointy = *(coord+nsd*bc.far.zm[i] + 1);
		fpointz = *(coord+nsd*bc.far.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_far[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.xp[i]);
		fpointy = *(coord+nsd*bc.far.xp[i] + 1);
		fpointz = *(coord+nsd*bc.far.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_far[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.yp[i]);
		fpointy = *(coord+nsd*bc.far.yp[i] + 1);
		fpointz = *(coord+nsd*bc.far.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_far[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.zp[i]);
		fpointy = *(coord+nsd*bc.far.zp[i] + 1);
		fpointz = *(coord+nsd*bc.far.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_far[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.xp[i]);
		fpointy = *(coord+nsd*bc.far.xp[i] + 1);
		fpointz = *(coord+nsd*bc.far.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_far[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.yp[i]);
		fpointy = *(coord+nsd*bc.far.yp[i] + 1);
		fpointz = *(coord+nsd*bc.far.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_far[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.far.zp[i]);
		fpointy = *(coord+nsd*bc.far.zp[i] + 1);
		fpointz = *(coord+nsd*bc.far.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();

/* This draws all the free bondary conditions.  This includes
   the types 13-18.
*/
  glNewList(BoundList[4], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_free[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.xm[i]);
		fpointy = *(coord+nsd*bc.free.xm[i] + 1);
		fpointz = *(coord+nsd*bc.free.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_free[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.ym[i]);
		fpointy = *(coord+nsd*bc.free.ym[i] + 1);
		fpointz = *(coord+nsd*bc.free.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_free[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.zm[i]);
		fpointy = *(coord+nsd*bc.free.zm[i] + 1);
		fpointz = *(coord+nsd*bc.free.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_free[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.xm[i]);
		fpointy = *(coord+nsd*bc.free.xm[i] + 1);
		fpointz = *(coord+nsd*bc.free.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.ym[i]);
		fpointy = *(coord+nsd*bc.free.ym[i] + 1);
		fpointz = *(coord+nsd*bc.free.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.zm[i]);
		fpointy = *(coord+nsd*bc.free.zm[i] + 1);
		fpointz = *(coord+nsd*bc.free.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_free[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.xp[i]);
		fpointy = *(coord+nsd*bc.free.xp[i] + 1);
		fpointz = *(coord+nsd*bc.free.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_free[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.yp[i]);
		fpointy = *(coord+nsd*bc.free.yp[i] + 1);
		fpointz = *(coord+nsd*bc.free.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_free[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.zp[i]);
		fpointy = *(coord+nsd*bc.free.zp[i] + 1);
		fpointz = *(coord+nsd*bc.free.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_free[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.xp[i]);
		fpointy = *(coord+nsd*bc.free.xp[i] + 1);
		fpointz = *(coord+nsd*bc.free.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.yp[i]);
		fpointy = *(coord+nsd*bc.free.yp[i] + 1);
		fpointz = *(coord+nsd*bc.free.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free.zp[i]);
		fpointy = *(coord+nsd*bc.free.zp[i] + 1);
		fpointz = *(coord+nsd*bc.free.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();

/* This draws all the inflow bondary conditions.  This includes
   the types 19-24.
*/
  glNewList(BoundList[5], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_inflow[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.xm[i]);
		fpointy = *(coord+nsd*bc.inflow.xm[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_inflow[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.ym[i]);
		fpointy = *(coord+nsd*bc.inflow.ym[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_inflow[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.zm[i]);
		fpointy = *(coord+nsd*bc.inflow.zm[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_inflow[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.xm[i]);
		fpointy = *(coord+nsd*bc.inflow.xm[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_inflow[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.ym[i]);
		fpointy = *(coord+nsd*bc.inflow.ym[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_inflow[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.zm[i]);
		fpointy = *(coord+nsd*bc.inflow.zm[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_inflow[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.xp[i]);
		fpointy = *(coord+nsd*bc.inflow.xp[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_inflow[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.yp[i]);
		fpointy = *(coord+nsd*bc.inflow.yp[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_inflow[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.zp[i]);
		fpointy = *(coord+nsd*bc.inflow.zp[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_inflow[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.xp[i]);
		fpointy = *(coord+nsd*bc.inflow.xp[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_inflow[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.yp[i]);
		fpointy = *(coord+nsd*bc.inflow.yp[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_inflow[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.inflow.zp[i]);
		fpointy = *(coord+nsd*bc.inflow.zp[i] + 1);
		fpointz = *(coord+nsd*bc.inflow.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();










/* This draws all the freestream bondary conditions.  This includes
   the types 25-30.
*/
  glNewList(BoundList[6], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_frestr[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.xm[i]);
		fpointy = *(coord+nsd*bc.frestr.xm[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_frestr[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.ym[i]);
		fpointy = *(coord+nsd*bc.frestr.ym[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_frestr[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.zm[i]);
		fpointy = *(coord+nsd*bc.frestr.zm[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_frestr[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.xm[i]);
		fpointy = *(coord+nsd*bc.frestr.xm[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_frestr[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.ym[i]);
		fpointy = *(coord+nsd*bc.frestr.ym[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_frestr[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.zm[i]);
		fpointy = *(coord+nsd*bc.frestr.zm[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_frestr[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.xp[i]);
		fpointy = *(coord+nsd*bc.frestr.xp[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_frestr[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.yp[i]);
		fpointy = *(coord+nsd*bc.frestr.yp[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_frestr[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.zp[i]);
		fpointy = *(coord+nsd*bc.frestr.zp[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_frestr[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.xp[i]);
		fpointy = *(coord+nsd*bc.frestr.xp[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_frestr[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.yp[i]);
		fpointy = *(coord+nsd*bc.frestr.yp[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_frestr[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.frestr.zp[i]);
		fpointy = *(coord+nsd*bc.frestr.zp[i] + 1);
		fpointz = *(coord+nsd*bc.frestr.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();


/* This draws all the 2nd free(free2) bondary conditions.  This includes
   the types 31-36.
*/
  glNewList(BoundList[7], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_free2[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.xm[i]);
		fpointy = *(coord+nsd*bc.free2.xm[i] + 1);
		fpointz = *(coord+nsd*bc.free2.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_free2[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.ym[i]);
		fpointy = *(coord+nsd*bc.free2.ym[i] + 1);
		fpointz = *(coord+nsd*bc.free2.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_free2[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.zm[i]);
		fpointy = *(coord+nsd*bc.free2.zm[i] + 1);
		fpointz = *(coord+nsd*bc.free2.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_free2[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.xm[i]);
		fpointy = *(coord+nsd*bc.free2.xm[i] + 1);
		fpointz = *(coord+nsd*bc.free2.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free2[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.ym[i]);
		fpointy = *(coord+nsd*bc.free2.ym[i] + 1);
		fpointz = *(coord+nsd*bc.free2.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free2[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.zm[i]);
		fpointy = *(coord+nsd*bc.free2.zm[i] + 1);
		fpointz = *(coord+nsd*bc.free2.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_free2[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.xp[i]);
		fpointy = *(coord+nsd*bc.free2.xp[i] + 1);
		fpointz = *(coord+nsd*bc.free2.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_free2[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.yp[i]);
		fpointy = *(coord+nsd*bc.free2.yp[i] + 1);
		fpointz = *(coord+nsd*bc.free2.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_free2[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.zp[i]);
		fpointy = *(coord+nsd*bc.free2.zp[i] + 1);
		fpointz = *(coord+nsd*bc.free2.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_free2[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.xp[i]);
		fpointy = *(coord+nsd*bc.free2.xp[i] + 1);
		fpointz = *(coord+nsd*bc.free2.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free2[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.yp[i]);
		fpointy = *(coord+nsd*bc.free2.yp[i] + 1);
		fpointz = *(coord+nsd*bc.free2.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_free2[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.free2.zp[i]);
		fpointy = *(coord+nsd*bc.free2.zp[i] + 1);
		fpointz = *(coord+nsd*bc.free2.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();


/* This draws all the no slip bondary conditions.  This includes
   the types 37-62, although for now, I have only done the case for
   2-D meshes.  The reason why there are so many types of no slip
   conditions is because a node may be noslip in multiple directions.
*/
  glNewList(BoundList[8], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
/* Draws the negative side components */

	glLineWidth (line_width);
	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_noslp[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.xm[i]);
		fpointy = *(coord+nsd*bc.noslp.xm[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx - fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_noslp[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.ym[i]);
		fpointy = *(coord+nsd*bc.noslp.ym[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy - fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_noslp[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.zm[i]);
		fpointy = *(coord+nsd*bc.noslp.zm[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz - fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_noslp[0].xm; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.xm[i]);
		fpointy = *(coord+nsd*bc.noslp.xm[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.xm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_noslp[0].ym; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.ym[i]);
		fpointy = *(coord+nsd*bc.noslp.ym[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.ym[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_noslp[0].zm; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.zm[i]);
		fpointy = *(coord+nsd*bc.noslp.zm[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.zm[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();

/* Draws the positive side components */

	glBegin(GL_LINES);
/* Draw the x displacement component */
	   glColor4fv(white);
	   for( i = 0; i < bc.num_noslp[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.xp[i]);
		fpointy = *(coord+nsd*bc.noslp.xp[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx + fdum, fpointy, fpointz);
	   }
/* Draw the y displacement component */
	   glColor4fv(grey);
	   for( i = 0; i < bc.num_noslp[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.yp[i]);
		fpointy = *(coord+nsd*bc.noslp.yp[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy + fdum, fpointz);
	   }
/* Draw the z displacement component */
	   glColor4fv(black);
	   for( i = 0; i < bc.num_noslp[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.zp[i]);
		fpointy = *(coord+nsd*bc.noslp.zp[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
		glVertex3f( fpointx, fpointy, fpointz + fdum);
	   }
	glEnd();
	glPointSize(point_size);
	glBegin(GL_POINTS);
	   glColor4fv(blue);
	   for( i = 0; i < bc.num_noslp[0].xp; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.xp[i]);
		fpointy = *(coord+nsd*bc.noslp.xp[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.xp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_noslp[0].yp; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.yp[i]);
		fpointy = *(coord+nsd*bc.noslp.yp[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.yp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	   for( i = 0; i < bc.num_noslp[0].zp; ++i)
	   {
		fpointx = *(coord+nsd*bc.noslp.zp[i]);
		fpointy = *(coord+nsd*bc.noslp.zp[i] + 1);
		fpointz = *(coord+nsd*bc.noslp.zp[i] + 2);
		glVertex3f( fpointx, fpointy, fpointz);
	   }
	glEnd();
  glPopAttrib();
  glEndList();


/* This draws all the 0 pressure node bondary conditions.  This is for
   the boundary condition type 63.
*/

  glNewList(BoundList[9], GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);

        glPointSize(point_size);
        glBegin(GL_POINTS);
           glColor4fv(orange);
           for( i = 0; i < bc.num_prs0[0]; ++i)
           {
                fpointx = *(coord+nsd*bc.prs0[i]);
                fpointy = *(coord+nsd*bc.prs0[i] + 1);
                fpointz = *(coord+nsd*bc.prs0[i] + 2);
                glVertex3f( fpointx, fpointy, fpointz);
           }
        glEnd();
  glPopAttrib();
  glEndList();

}

void velocity_vectors(int VelocityList, BOUND bc, double *coord,
	XYZF *velocity_vec )
{

/* creates velocity vector display lists for both before and after meshes

                  Last Update 3/3/06
*/

  int i,j, k, dum, node_num;
  double fpointx, fpointy, fpointz, fx, fy, fz, fdum;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };

  double SMALL3 = 1.0e-5;

  glNewList(VelocityList, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);

    glLineWidth (line_width);
    glBegin(GL_LINES);
/* Draw the velocity vectors axis */
	glColor4fv(white);
	for( i = 0; i < nx; i += nx_inc)
	{
	    for( j = 0; j < ny; j += ny_inc)
	    {
		for( k = 0; k < nz; k += nz_inc)
		{
		    node_num = ny*nz*i + nz*j + k;
		    fx = velocity_vec[node_num].x; fy = velocity_vec[node_num].y;
			fz = velocity_vec[node_num].z;
		    fdum = fx*fx + fy*fy + fz*fz;
		    if( fdum > SMALL3 )
		    {
			fpointx = *(coord+nsd*node_num);
			fpointy = *(coord+nsd*node_num + 1);
			fpointz = *(coord+nsd*node_num + 2);
			fx = fpointx - fx;
			fy = fpointy - fy;
			fz = fpointz - fz;
			glVertex3f( fpointx, fpointy, fpointz);
			glVertex3f( fx, fy, fz);
		    }
		}
	    }
	}
    glEnd();
    glPointSize(point_size);
    glBegin(GL_POINTS);
	glColor4fv(red);
	for( i = 0; i < nx; i += nx_inc)
	{
	    for( j = 0; j < ny; j += ny_inc)
	    {
		for( k = 0; k < nz; k += nz_inc)
		{
		    node_num = ny*nz*i + nz*j + k;
		    fx = velocity_vec[node_num].x; fy = velocity_vec[node_num].y;
			fz = velocity_vec[node_num].z;
		    fdum = fx*fx + fy*fy + fz*fz;
		    if( fdum > SMALL3 )
		    {
			fpointx = *(coord+nsd*node_num);
			fpointy = *(coord+nsd*node_num + 1);
			fpointz = *(coord+nsd*node_num + 2);
			glVertex3f( fpointx, fpointy, fpointz);
		    }
		}
	    }
	}
    glEnd();

  glPopAttrib();
  glEndList();
}


void pathline_points(int PathlineList, FLOWLINES path )
{

/* creates pathline display lists

                  Last Update 4/2/08
*/

  int i,j, k, dum, node_num, num_pt0div16;
  double fpointx, fpointy, fpointz, fx, fy, fz, fdum;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };

  double SMALL3 = 1.0e-5;

/* Based on the number of pathlines, I have to divide them into blocks of 16 because
   I have 9 colors to distinguish between them.
*/

  dum = path.num_pt0%16;
  dum = path.num_pt0 - dum;
  num_pt0div16 = (int)(dum/16);

  glNewList(PathlineList, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);

    if(path.num_pt0 <= 16)
    {
	glPointSize(point_size);
	glBegin(GL_POINTS);
	    for( i = 0; i < path.nstep_in; ++i)
	    {
		for( j = 0; j < path.num_pt0; ++j)
		{
		    if( j ==  0 ) glColor4fv(blue);
		    if( j ==  1 ) glColor4fv(aqua);
		    if( j ==  2 ) glColor4fv(blueGreen);
		    if( j ==  3 ) glColor4fv(green);
		    if( j ==  4 ) glColor4fv(greenYellow);
		    if( j ==  5 ) glColor4fv(yellow);
		    if( j ==  6 ) glColor4fv(orange);
		    if( j ==  7 ) glColor4fv(orangeRed);
		    if( j ==  8 ) glColor4fv(red);
		    if( j ==  9 ) glColor4fv(orangeRed);
		    if( j == 10 ) glColor4fv(orange);
		    if( j == 11 ) glColor4fv(yellow);
		    if( j == 12 ) glColor4fv(greenYellow);
		    if( j == 13 ) glColor4fv(green);
		    if( j == 14 ) glColor4fv(blueGreen);
		    if( j == 15 ) glColor4fv(aqua);
		    dum = path.num_pt0*i + j;
		    fx = path.pt[dum].x; fy = path.pt[dum].y;
			fz = path.pt[dum].z;
		    glVertex3f( fx, fy, fz);
		    /*printf( "%9.4f %9.4f %9.4f", fx, fy, fz);*/
		}
	    }
	glEnd();
    }
    if(path.num_pt0 > 16)
    {
	glPointSize(point_size);
	glBegin(GL_POINTS);
	    for( i = 0; i < path.nstep_in; ++i)
	    {
		for( k = 0; k < num_pt0div16; ++k)
		{
		    for( j = k*16; j < 16 + k*16; ++j)
		    {
			if( j ==  0 + k*16 ) glColor4fv(blue);
			if( j ==  1 + k*16 ) glColor4fv(aqua);
			if( j ==  2 + k*16 ) glColor4fv(blueGreen);
			if( j ==  3 + k*16 ) glColor4fv(green);
			if( j ==  4 + k*16 ) glColor4fv(greenYellow);
			if( j ==  5 + k*16 ) glColor4fv(yellow);
			if( j ==  6 + k*16 ) glColor4fv(orange);
			if( j ==  7 + k*16 ) glColor4fv(orangeRed);
			if( j ==  8 + k*16 ) glColor4fv(red);
			if( j ==  9 + k*16 ) glColor4fv(orangeRed);
			if( j == 10 + k*16 ) glColor4fv(orange);
			if( j == 11 + k*16 ) glColor4fv(yellow);
			if( j == 12 + k*16 ) glColor4fv(greenYellow);
			if( j == 13 + k*16 ) glColor4fv(green);
			if( j == 14 + k*16 ) glColor4fv(blueGreen);
			if( j == 15 + k*16 ) glColor4fv(aqua);
			dum = path.num_pt0*i + j;
			fx = path.pt[dum].x; fy = path.pt[dum].y;
			    fz = path.pt[dum].z;
			glVertex3f( fx, fy, fz);
			/*printf( "%4d %4d %4d %8d %9.4f %9.4f %9.4f", i, j, k, dum, fx, fy, fz);*/
		    }
		    /*printf( "\n");*/
		}
		for( j = num_pt0div16*16; j < path.num_pt0; ++j)
		{
		    if( j ==  0 + num_pt0div16*16 ) glColor4fv(blue);
		    if( j ==  1 + num_pt0div16*16 ) glColor4fv(aqua);
		    if( j ==  2 + num_pt0div16*16 ) glColor4fv(blueGreen);
		    if( j ==  3 + num_pt0div16*16 ) glColor4fv(green);
		    if( j ==  4 + num_pt0div16*16 ) glColor4fv(greenYellow);
		    if( j ==  5 + num_pt0div16*16 ) glColor4fv(yellow);
		    if( j ==  6 + num_pt0div16*16 ) glColor4fv(orange);
		    if( j ==  7 + num_pt0div16*16 ) glColor4fv(orangeRed);
		    if( j ==  8 + num_pt0div16*16 ) glColor4fv(red);
		    if( j ==  9 + num_pt0div16*16 ) glColor4fv(orangeRed);
		    if( j == 10 + num_pt0div16*16 ) glColor4fv(orange);
		    if( j == 11 + num_pt0div16*16 ) glColor4fv(yellow);
		    if( j == 12 + num_pt0div16*16 ) glColor4fv(greenYellow);
		    if( j == 13 + num_pt0div16*16 ) glColor4fv(green);
		    if( j == 14 + num_pt0div16*16 ) glColor4fv(blueGreen);
		    if( j == 15 + num_pt0div16*16 ) glColor4fv(aqua);
		    dum = path.num_pt0*i + j;
		    fx = path.pt[dum].x; fy = path.pt[dum].y;
		    fz = path.pt[dum].z;
		    /*fx = (double)i/1000.0; fy = (double)i/1000.0;
			fz = path.pt[dum].z;*/
		    glVertex3f( fx, fy, fz);
		    /*printf( "%9.4f %9.4f %9.4f", fx, fy, fz);*/
		}
		/*printf( "\n");*/
	    }
	glEnd();
    }

  glPopAttrib();
  glEndList();
}


void streakline_points(int StreaklineList, FLOWLINES streak )
{

/* creates streakline display lists

                  Last Update 3/27/08
*/

  int i,j, k, dum, node_num, num_pt0div16;
  double fpointx, fpointy, fpointz, fx, fy, fz, fdum;
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };

  double SMALL3 = 1.0e-5;

/* Based on the number of streaklines, I have to divide them into blocks of 16 because
   I have 9 colors to distinguish between them.
*/

  dum = streak.num_pt0%16;
  dum = streak.num_pt0 - dum;
  num_pt0div16 = (int)(dum/16);

  glNewList(StreaklineList, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);

    if(streak.num_pt0 <= 16)
    {
	glPointSize(point_size);
	glBegin(GL_POINTS);
	    for( i = 0; i < streak.nstep_in; ++i)
	    {
		for( j = 0; j < streak.num_pt0; ++j)
		{
		    if( j ==  0 ) glColor4fv(blue);
		    if( j ==  1 ) glColor4fv(aqua);
		    if( j ==  2 ) glColor4fv(blueGreen);
		    if( j ==  3 ) glColor4fv(green);
		    if( j ==  4 ) glColor4fv(greenYellow);
		    if( j ==  5 ) glColor4fv(yellow);
		    if( j ==  6 ) glColor4fv(orange);
		    if( j ==  7 ) glColor4fv(orangeRed);
		    if( j ==  8 ) glColor4fv(red);
		    if( j ==  9 ) glColor4fv(orangeRed);
		    if( j == 10 ) glColor4fv(orange);
		    if( j == 11 ) glColor4fv(yellow);
		    if( j == 12 ) glColor4fv(greenYellow);
		    if( j == 13 ) glColor4fv(green);
		    if( j == 14 ) glColor4fv(blueGreen);
		    if( j == 15 ) glColor4fv(aqua);
		    dum = streak.num_pt0*i + j;
		    fx = streak.pt[dum].x; fy = streak.pt[dum].y;
			fz = streak.pt[dum].z;
		    glVertex3f( fx, fy, fz);
		    /*printf( "%9.4f %9.4f %9.4f", fx, fy, fz);*/
		}
	    }
	glEnd();
    }
    if(streak.num_pt0 > 16)
    {
	glPointSize(point_size);
	glBegin(GL_POINTS);
	    for( i = 0; i < streak.nstep_in; ++i)
	    {
		for( k = 0; k < num_pt0div16; ++k)
		{
		    for( j = k*16; j < 16 + k*16; ++j)
		    {
			if( j ==  0 + k*16 ) glColor4fv(blue);
			if( j ==  1 + k*16 ) glColor4fv(aqua);
			if( j ==  2 + k*16 ) glColor4fv(blueGreen);
			if( j ==  3 + k*16 ) glColor4fv(green);
			if( j ==  4 + k*16 ) glColor4fv(greenYellow);
			if( j ==  5 + k*16 ) glColor4fv(yellow);
			if( j ==  6 + k*16 ) glColor4fv(orange);
			if( j ==  7 + k*16 ) glColor4fv(orangeRed);
			if( j ==  8 + k*16 ) glColor4fv(red);
			if( j ==  9 + k*16 ) glColor4fv(orangeRed);
			if( j == 10 + k*16 ) glColor4fv(orange);
			if( j == 11 + k*16 ) glColor4fv(yellow);
			if( j == 12 + k*16 ) glColor4fv(greenYellow);
			if( j == 13 + k*16 ) glColor4fv(green);
			if( j == 14 + k*16 ) glColor4fv(blueGreen);
			if( j == 15 + k*16 ) glColor4fv(aqua);
			dum = streak.num_pt0*i + j;
			fx = streak.pt[dum].x; fy = streak.pt[dum].y;
			    fz = streak.pt[dum].z;
			glVertex3f( fx, fy, fz);
			/*printf( "%4d %4d %4d %8d %9.4f %9.4f %9.4f", i, j, k, dum, fx, fy, fz);*/
		    }
		    /*printf( "\n");*/
		}
		for( j = num_pt0div16*16; j < streak.num_pt0; ++j)
		{
		    if( j ==  0 + num_pt0div16*16 ) glColor4fv(blue);
		    if( j ==  1 + num_pt0div16*16 ) glColor4fv(aqua);
		    if( j ==  2 + num_pt0div16*16 ) glColor4fv(blueGreen);
		    if( j ==  3 + num_pt0div16*16 ) glColor4fv(green);
		    if( j ==  4 + num_pt0div16*16 ) glColor4fv(greenYellow);
		    if( j ==  5 + num_pt0div16*16 ) glColor4fv(yellow);
		    if( j ==  6 + num_pt0div16*16 ) glColor4fv(orange);
		    if( j ==  7 + num_pt0div16*16 ) glColor4fv(orangeRed);
		    if( j ==  8 + num_pt0div16*16 ) glColor4fv(red);
		    if( j ==  9 + num_pt0div16*16 ) glColor4fv(orangeRed);
		    if( j == 10 + num_pt0div16*16 ) glColor4fv(orange);
		    if( j == 11 + num_pt0div16*16 ) glColor4fv(yellow);
		    if( j == 12 + num_pt0div16*16 ) glColor4fv(greenYellow);
		    if( j == 13 + num_pt0div16*16 ) glColor4fv(green);
		    if( j == 14 + num_pt0div16*16 ) glColor4fv(blueGreen);
		    if( j == 15 + num_pt0div16*16 ) glColor4fv(aqua);
		    dum = streak.num_pt0*i + j;
		    fx = streak.pt[dum].x; fy = streak.pt[dum].y;
		    fz = streak.pt[dum].z;
		    /*fx = (double)i/1000.0; fy = (double)i/1000.0;
			fz = streak.pt[dum].z;*/
		    glVertex3f( fx, fy, fz);
		    /*printf( "%9.4f %9.4f %9.4f", fx, fy, fz);*/
		}
		/*printf( "\n");*/
	    }
	glEnd();
    }

  glPopAttrib();
  glEndList();
}


void bodies_in_flow(int BodyList, BODY *body )
{

/* creates body display lists

                  Last Update 3/20/08
*/

  int i,j, k, dum, node_num, bod_num, pt;
  double fpointx, fpointy, fpointz, fx, fy, fz, fdum;
  GLfloat norm[3];
  GLfloat axes_ambuse[] =   { 0.5, 0.0, 0.0, 1.0 };
  double coord_el[npel*3];

  double SMALL3 = 1.0e-15;

  norm[0] = 0.0;
  norm[1] = 0.0;
  norm[2] = 1.0;

  glNewList(BodyList, GL_COMPILE);
  glPushAttrib(GL_LIGHTING_BIT);
    glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, axes_ambuse);

/* Draw the objects as white solid bodies. */

    glColor4fv(white);
    for( bod_num = 0; bod_num < num_bodies; ++bod_num)
    {
	dum = body[bod_num].num_points;
	for( pt = 0; pt < body[bod_num].num_points-1; ++pt )
	{
	    *(coord_el)   = body[bod_num].coord[2*pt];
	    *(coord_el+1) = body[bod_num].coord[2*pt+1];
	    *(coord_el+2) = 0.0;

	    *(coord_el+3) = body[bod_num].coord[2*(pt+dum)] + SMALL3;
	    *(coord_el+4) = body[bod_num].coord[2*(pt+dum)+1] + SMALL3;
	    *(coord_el+5) = 0.0;

	    *(coord_el+6) = body[bod_num].coord[2*(pt+dum+1)] + SMALL3;
	    *(coord_el+7) = body[bod_num].coord[2*(pt+dum+1)+1] + SMALL3;
	    *(coord_el+8) = 0.0;

	    *(coord_el+9)  = body[bod_num].coord[2*(pt+1)];
	    *(coord_el+10) = body[bod_num].coord[2*(pt+1)+1];
	    *(coord_el+11) = 0.0;

	    glBegin(GL_TRIANGLES);
		glNormal3fv(norm);
		glVertex3dv((coord_el));
		glVertex3dv((coord_el+3));
		glVertex3dv((coord_el+6));
	    glEnd();

	    glBegin(GL_TRIANGLES);
		glNormal3fv(norm);
		glVertex3dv((coord_el));
		glVertex3dv((coord_el+6));
		glVertex3dv((coord_el+9));
	    glEnd();
	}
    }

/* Draw a black outline around each object. */

    glLineWidth (4.0*line_width);
    glColor4fv(black);
    for( bod_num = 0; bod_num < num_bodies; ++bod_num)
    {
	dum = body[bod_num].num_points;
	glBegin(GL_LINE_LOOP);
	    /*for( pt = 0; pt < 2*body[bod_num].num_points; ++pt )
	    {
		fpointx = body[bod_num].coord[2*pt];
		fpointy = body[bod_num].coord[2*pt+1];
		fpointz = 0.0;
		glVertex3f( fpointx, fpointy, fpointz);
	    }*/
	    fpointz = 0.0;
	    for( pt = body[bod_num].num_points-1; pt >= 0; --pt )
	    {
		fpointx = body[bod_num].coord[2*pt];
		fpointy = body[bod_num].coord[2*pt+1];
		glVertex3f( fpointx, fpointy, fpointz);
	    }
	    dum = body[bod_num].num_points;
	    for( pt = 0; pt < body[bod_num].num_points; ++pt )
	    {
		fpointx = body[bod_num].coord[2*(pt+dum)];
		fpointy = body[bod_num].coord[2*(pt+dum)+1];
		glVertex3f( fpointx, fpointy, fpointz);
	    }
	    pt = body[bod_num].num_points-1;
	    fpointx = body[bod_num].coord[2*pt];
	    fpointy = body[bod_num].coord[2*pt+1];
	    glVertex3f( fpointx, fpointy, fpointz);

	glEnd();
  }

  glPopAttrib();
  glEndList();
}


