/* This library function generates mesh data for a finite difference mesh
   of a compressible CFD flow field. 

                     Updated 2/16/06

    SLFCFD source file
    Version:  1.1
    Copyright (C) 2005, 2006, 2007, 2008  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../compressible/cfdconst.h"
#include "../compressible/cfdstruct.h"

#define SMALL   1.0e-20

extern dof, nmat, numel, numnp, sof, nx, ny, nz;
extern int flag_2D, flag_3D, gr_optimize_flag;
extern double dt, dx, dy, dz;
extern double coord_rescale;
extern double cross_sec_left_right, cross_sec_up_down, cross_sec_in_out,
        cross_sec_left_right0, cross_sec_up_down0, cross_sec_in_out0;

int fdMesher( int *connecter, double *coord, GR_NODE *node)
{
/*
   This utility function generates the nodal and connectivity data
   needed for graphics display.

                     Updated 2/3/06
*/
	int i, j, k, node_num;
	int dof, numed, plane_stress_flag;
	int dum, dum2, dum3, dum4, node_dis_flag;
	double fdum1, fdum2, fdum3;
	double dns, temp, vx, vy, vz, prs, ene, enthal;
	double side_length, side_width, el_length, el_width, el_height;
	int length_el_num, width_el_num, el_type;

	el_length = dx;
	el_width  = dy;
	el_height = dz;

	if(flag_2D)
	{
	    dum = 0;
	    for( i = 0; i < nx - 1; ++i )
	    {
		for( j = 0; j < ny - 1; ++j )
		{
			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum)     = i*ny + j;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = (i+1)*ny + j;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = (i+1)*ny + (j+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = i*ny + (j+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }
	    numel = dum;

	    dum = 0;
	    for( node_num = 0; node_num < numnp; ++node_num )
	    {
		fdum1 = dx*((double)node[node_num].i);
		fdum2 = dy*((double)node[node_num].j);
		fdum3 = 0.0;
		*(coord + node_num*nsd)     = fdum1;
		*(coord + node_num*nsd + 1) = fdum2;
		*(coord + node_num*nsd + 2) = fdum3;
		/*printf("%d ",node_num);
		printf("%16.8e   %16.8e   %16.8e\n",
			*(coord + node_num*nsd),
			*(coord + node_num*nsd + 1),
			*(coord + node_num*nsd + 2));*/
	    }
	}


	if(flag_3D)
	{
	    if(!gr_optimize_flag)
	    {
/* I have replaced the connectivity directly below with that found in ConnectSurfMesher2. */
		dum = 0;
		for( i = 0; i < nx - 1; ++i )
		{
		    for( j = 0; j < ny - 1; ++j )
		    {
			for( k = 0; k < nz - 1; ++k )
			{
			    *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			    *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			    *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			    *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			    *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			    *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			    *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			    *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			    /*printf("%4d ",dum);
			    printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			    ++dum;
			}
		    }
		}
		numel = dum;
	    }

	    dum = 0;
	    for( node_num = 0; node_num < numnp; ++node_num )
	    {
		fdum1 = dx*((double)node[node_num].i);
		fdum2 = dy*((double)node[node_num].j);
		fdum3 = dz*((double)node[node_num].k);
		*(coord + node_num*nsd)     = fdum1;
		*(coord + node_num*nsd + 1) = fdum2;
		*(coord + node_num*nsd + 2) = fdum3;
		/*printf("%8d ",node_num);
		printf("%16.8e   %16.8e   %16.8e\n",
			*(coord + node_num*nsd),
			*(coord + node_num*nsd + 1),
			*(coord + node_num*nsd + 2));*/
	    }
	}

	return 1;
}


int ConnectSurfMesher (int *connecter, int *el_matl, GR_NODE *node)
{

/* 
   This utility function recalculates the connectivity so that
   only elements which lie on the surface are used in the
   graphics display.  This is done to greatly speed up the
   graphics.

                     Updated 2/11/06
*/
	int i, j, k, node_num, ConnectSurfflag;
	int dum, dum2, dum3, dum4, node_dis_flag;
	double fdum1, fdum2, fdum3;
	FILE *o2;


	dum = 0;

/* For the x-y face with the smallest z value. */

/* In creating the surface element connectivity, I don't want to draw elements which have
   discarded nodes.  This is why I use the node_dis_flag. 
*/
	for( i = 0; i < nx - 1; ++i )
	{
	    for( j = 0; j < ny - 1; ++j )
	    {
			k = 0;
			node_dis_flag = 0;
			dum2 = *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
	    }
	}

/* For the x-y face with the largest z value. */

	if( nz > 2)
	{
	    for( i = 0; i < nx - 1; ++i )
	    {
		for( j = 0; j < ny - 1; ++j )
		{
			k = nz - 2;
			node_dis_flag = 0;
			dum2 = *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }
	}

/* For the z-x face with the smallest y value. */

	if( nz > 3)
	{
	    for( i = 0; i < nx - 1; ++i )
	    {
		j = 0;
		for( k = 1; k < nz - 2; ++k )
		{
			node_dis_flag = 0;
			dum2 = *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }

/* For the z-x face with the largest y value. */

	    if( ny > 2 )
	    {
		for( i = 0; i < nx - 1; ++i )
		{
		    j = ny - 2;
		    for( k = 1; k < nz - 2; ++k )
		    {
			node_dis_flag = 0;
			dum2 = *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		    }
		}
	    }

/* For the y-z face with the smallest x value. */

	    i = 0;
	    for( j = 1; j < ny - 2; ++j )
	    {
		for( k = 1; k < nz - 2; ++k )
		{
			node_dis_flag = 0;
			dum2 = *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }

/* For the y-z face with the largest x value. */

	    if( nx > 2 )
	    {
		i = nx - 2;
		for( j = 1; j < ny - 2; ++j )
		{
		    for( k = 1; k < nz - 2; ++k )
		    {
			node_dis_flag = 0;
			dum2 = *(connecter + npel*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 4) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 5) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 6) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel*dum + 7) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel*dum),
				*(connecter + npel*dum + 1),
				*(connecter + npel*dum + 2),
				*(connecter + npel*dum + 3),
				*(connecter + npel*dum + 4),
				*(connecter + npel*dum + 5),
				*(connecter + npel*dum + 6),
				*(connecter + npel*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		    }
		}
	    }
	}

	numel = dum;

	return 1;
}



int ConnectSurfMesher2 (int *connecter, int *el_matl, GR_NODE *node)
{

/* 
   This utility function creates a new connectivity for 3-D meshes.
   Unlike ConnectSurfMesher, the connectivity created here is for
   4 node quad elements which represent only one side of the 6 sided
   3-D cube.  Because of the nature of uniform finite difference meshes,
   it is easy to determine which side should be drawn to produce 
   meshes which appear like those created in ConnectSurfMesher, but 
   are graphically displayed much faster because only 1 of 6 sides are
   drawn.

                     Updated 2/14/06
*/
	int i, j, k, node_num, ConnectSurfflag;
	int dum, dum2, dum3, dum4, node_dis_flag;
	double fdum1, fdum2, fdum3;
	int nx_shift, ny_shift, nz_shift;
	double cross_sec_nx, cross_sec_ny, cross_sec_nz;
	FILE *o2;


	cross_sec_nx = cross_sec_left_right*coord_rescale/(dx + SMALL);
	cross_sec_ny = cross_sec_up_down*coord_rescale/(dy + SMALL);
	cross_sec_nz = cross_sec_in_out*coord_rescale/(dz + SMALL);

/* The quantities n*_shift are used so that when I turn on the cutting planes,
   I will still be drawing the elements which are located near and parallel
   to the particular plane.  This is done by generating a new connectivity whose
   uniform finite difference mesh has values of nx, ny, and nz which are shifted
   so that the dimensions lie within the Cross Section cutting planes.
*/ 

	nx_shift = nx;
	if (nx_shift > (int) cross_sec_nx) nx_shift = cross_sec_nx;
	ny_shift = ny;
	if (ny_shift > (int) cross_sec_ny) ny_shift = cross_sec_ny;
	nz_shift = nz;
	if (nz_shift > (int) cross_sec_nz) nz_shift = cross_sec_nz;

	/*printf(" %4d %4d %4d %12.6e  %12.6e  %12.6e  %12.6e \n",
		nx_shift, ny_shift, nz_shift, coord_rescale, 
		cross_sec_left_right, cross_sec_up_down, cross_sec_in_out);*/

	dum = 0;

/* For the x-y face with the smallest z value. */

/* In creating the surface element connectivity, I don't want to draw elements which have
   discarded nodes.  This is why I use the node_dis_flag. 
*/

	for( i = 0; i < nx_shift - 1; ++i )
	{
	    for( j = 0; j < ny_shift - 1; ++j )
	    {
			k = 0;
			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				*(connecter + npel2*dum + 4),
				*(connecter + npel2*dum + 5),
				*(connecter + npel2*dum + 6),
				*(connecter + npel2*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
	    }
	}

/* For the x-y face with the largest z value. */

	if( nz_shift > 2)
	{
	    for( i = 0; i < nx_shift - 1; ++i )
	    {
		for( j = 0; j < ny_shift - 1; ++j )
		{
			k = nz_shift - 2;
			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				*(connecter + npel2*dum + 4),
				*(connecter + npel2*dum + 5),
				*(connecter + npel2*dum + 6),
				*(connecter + npel2*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }
	}

/* For the z-x face with the smallest y value. */

	if( nz_shift > 3)
	{
	    for( i = 0; i < nx_shift - 1; ++i )
	    {
		j = 0;
		for( k = 1; k < nz_shift - 2; ++k )
		{
			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = (i+1)*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = (i+1)*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				*(connecter + npel2*dum + 4),
				*(connecter + npel2*dum + 5),
				*(connecter + npel2*dum + 6),
				*(connecter + npel2*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }

/* For the z-x face with the largest y value. */

	    if( ny_shift > 2 )
	    {
		for( i = 0; i < nx_shift - 1; ++i )
		{
		    j = ny_shift - 2;
		    for( k = 1; k < nz_shift - 2; ++k )
		    {
			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum) = (i+1)*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = (i+1)*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				*(connecter + npel2*dum + 4),
				*(connecter + npel2*dum + 5),
				*(connecter + npel2*dum + 6),
				*(connecter + npel2*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		    }
		}
	    }

/* For the y-z face with the smallest x value. */

	    i = 0;
	    for( j = 1; j < ny_shift - 2; ++j )
	    {
		for( k = 1; k < nz_shift - 2; ++k )
		{

			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;

			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				*(connecter + npel2*dum + 4),
				*(connecter + npel2*dum + 5),
				*(connecter + npel2*dum + 6),
				*(connecter + npel2*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		}
	    }

/* For the y-z face with the largest x value. */

	    if( nx_shift > 2 )
	    {
		i = nx_shift - 2;
		for( j = 1; j < ny_shift - 2; ++j )
		{
		    for( k = 1; k < nz_shift - 2; ++k )
		    {

			node_dis_flag = 0;
			dum2 = *(connecter + npel2*dum)     = i*ny*nz + j*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 1) = i*ny*nz + j*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 2) = i*ny*nz + (j+1)*nz + (k+1);
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			dum2 = *(connecter + npel2*dum + 3) = i*ny*nz + (j+1)*nz + k;
			if(node[dum2].bc_type > 99) node_dis_flag = 1;
			/*printf("%4d ",dum);
			printf(" %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
				*(connecter + npel2*dum),
				*(connecter + npel2*dum + 1),
				*(connecter + npel2*dum + 2),
				*(connecter + npel2*dum + 3),
				*(connecter + npel2*dum + 4),
				*(connecter + npel2*dum + 5),
				*(connecter + npel2*dum + 6),
				*(connecter + npel2*dum + 7),
				0);*/
			if(!node_dis_flag) ++dum;
		    }
		}
	    }
	}

	numel = dum;

	return 1;
}


