/* 
   This subroutine performs back substitution on a linear system 
   made up of a symetric matrix.  Only the upper diagonal elements
   are used.

        a               coefficient matrix
	solution        the rhs but will return as the solution

    The algorithm is taken from pages 641-642 in:

    Hughes, Thomas J. R., The Finite Element Method, Prentice-Hall Inc., Englewood
        Cliffs,  1987.

    It is the algorithm given by the 3 boxes solving for "b".

    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

        Updated 5/4/06
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


int backsub( double *a, int n, double *solution)
{
	int i, j;
	double *z, *y, sum;
	z = (double *)malloc(n*sizeof(double));
	y = (double *)malloc(n*sizeof(double));
	for( j = 0; j < n; ++j )
	{

/* Forward Reduction */

	   sum = 0.0;
	   for( i = 0; i < j; ++i )
	   {
		sum = sum + *(a+i*n+j)*(*(z+i));
	   }
	   *(z+j) = *(solution+j) - sum;

/* Diagonal Scaling */

	   *(y+j) = *(z+j)/(*(a+j*n+j));
	}

	for( i = n-1; i >= 0; --i )
	{

/* Back Substitution */

	   sum = 0.0;
	   for( j = n-1; j >= i+1; --j )
	   {
		sum = sum + *(a+n*i+j)*(*(solution+j));
	   }
	   *(solution+i) = *(y+i) - sum;
	}
	free(z);
	free(y);
	return 1;
}
