/* This subroutine performs LU decomposition on a symetric linear 
   system.  Only the upper diagonal elements are are used.
	
        a        coefficient matrix

   The algorithm is taken from page 639 in:

    Hughes, Thomas J. R., The Finite Element Method, Prentice-Hall Inc., Englewood
        Cliffs,  1987.

    It is the algorithm in the box.

    Copyright (C) 2006  San Le

    The source code contained in this file is released under the
    terms of the GNU Library General Public License.

        Updated 5/4/06
*/

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


int decomp(double *a, int n)
{
	int i,j,k;
	double T,temp;
	for( j = 0; j < n; ++j )
	{
	  for( i = 1; i < j; ++i )
	  {
		temp = 0.0;
		for( k = 0; k < i; ++k )
		{
			temp = *(a+n*k+i)*(*(a+n*k+j)) + temp;
		}
		*(a+n*i+j) = *(a+n*i+j) - temp;
	  }
	  for( i = 0; i < j; ++i )
	  {
		T = *(a+n*i+j);
		*(a+n*i+j) = T/(*(a+n*i+i));
		*(a+n*j+j) = *(a+n*j+j) - T*(*(a+n*i+j));
	  }
	}
	for( i = 0; i < n; ++i )
	{
	  for( j = i+1; j < n; ++j )
	  {
		*(a+n*j+i) = *(a+n*i+j);
	  }
	}
	return 1;
}

